/** Java class "BiRelationalProperty.java" generated from Poseidon for UML.
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * class representing a relational property which is a non 
 * concrete relation between an existing entity and a second
 * entity wich does NOT exist yet. (a "target" entity). Possible
 * relations are represented in derived classes (Equal, Incident,
 * Orthogonal, Parallel)
 * 
 * 
 * @author Thomas Laebe, ipb
 */
public abstract class BiRelationalProperty
	implements RelationalProperty {

	/**
	 * <p>
	 * Represents the first entity of the binary relation 
	 * </p>
	 */
	protected Entity first;

	/**
	 * Represents the type of the second entity.
	 */
	protected Class secondEntityType;

	/** Create a new relational Property. Can be used (only) by derived classes. 
	 * 
	 * @param _first first Entity
	 * @param _secondEntityType type of second Entity
	 */
	public BiRelationalProperty(Entity _first, Class _secondEntityType) {
		first = _first;
		secondEntityType = _secondEntityType;
	}

	/**
	 * <p>
	 * Return Jacobian Matrix A of the first Entity and the relation type.
	 * </p><p>
	 * 
	 * @return Jacobian Matrix A
	 * </p>
	 */
	public abstract Matrix getJacobian();

	/**
	 * <p>
		 * Compute degrees of freedom of that Relation
		 * </p><p>
		 * 
		 * @return degrees of freedom of that Relation
		 * </p>* Return Matrix A of the first Entity and the relation
	 */
	public abstract int getDof();

	/**
	 * <p>
	 * Returns the Covariance Matrix of the row-vector of the Jacobian Matrix A of the first Entity and the relation type.
	 *</p>
	 *
	 * @param row Row vector, for which the Covariance matrix is to be returned
	 * @return covariance of the row vector of the jacobian
	 */
	public abstract Matrix getCovarianceOfJacobianRow(int row);

	/** Get the first part of the relation.
	 * @return the first part of the relation
	 */
	public Entity getFirstEntity() {
		return first;
	}

	/** Get the type of the second entity
	 * @return the second entity type
	 */
	public Class getTargetType() {
		return secondEntityType;
	}

	/** Clone a relational property. The Entity and the class type of the
	 *  second entity are cloned.
	 *
	 * @return a copy of the element. 
	 */
	public Object clone() {
		try {
			BiRelationalProperty r = (BiRelationalProperty) super.clone();
			r.first = (Entity) first.clone();
			r.secondEntityType = secondEntityType;
			return r;

		} catch (CloneNotSupportedException e) {
			// this shouldn't happen 
			throw new InternalError();
		}

	}

	/** Get observation of that relation. This is per default the
	 *  value of the first entity. Used for estimation.
	 * @return the observation for that relation
	 */
	public Vector getObservations() {
		return first.getV();
	}

	/** Set observations of that relation (as a stacked vector).
	 *  For a binary relation this is setting the vector of the first element.
	 */
	public void setObservations(Vector v) {
		first.setV(v);
	}
	
	public Matrix getJacobianOfObservationConstraints() {
		return first.getJacobianOfConstraint();
	}

	public Vector getValueOfObservationConstraints() {
		return first.getConstraintValue();
	}
	

	/** Do conditioning of the observations.
 	 * @param rprops a set of Bi(!)RelationalProperties. RelationalProperty is used
	 *        for compatibility reasons.
	 * @return the conditioning factor (as one element array, for compatibility reasons)
	 */
	public static double[] doCondition(RelationalProperty[] rprops) {
		Entity[] elist = new Entity[rprops.length];
		for (int i = 0; i < rprops.length; i++)
			elist[i] = ((BiRelationalProperty) rprops[i]).getFirstEntity();

		double f = Entity.doCondition(elist);
		double [] ret=new double[1];
		ret[0]=f;
		return ret;
	}
	
	/** Do conditioning of the observations.
	 * 
	 * @param f the conditioning factor (as one element array, for compatibility reasons)
 	 * @param rprops a set of Bi(!)RelationalProperties. RelationalProperty is used
	 *        for compatibility reasons.
	 */
	public static void doCondition(RelationalProperty[] rprops, double [] f) {
		for (int i = 0; i < rprops.length; i++)
			((BiRelationalProperty) rprops[i]).getFirstEntity().doCondition(f[0]);
	}


	/** The the covariance matrix of the observation of that relation.
	 * This is per default the covariance of the first entity. Used for
	 * estimation.
	 * @return the covariance of the observations.
	 */
	public Matrix getCovarianceOfObservations() {
		return first.getCov();
	}
	
	/** The "reduced" covariance matrix of the observations which takes 
	 *  care of the homogeneous constraint. This is needed to obtain a 
	 *  coherent covariance of the corrected observation from the inner loop 
	 *  of the estimation, see note of WF from May 2005. 
	 *  	
	 * @return the covariance of the observations.
	 */
	public Matrix getReducedCovarianceOfObservations() {
		Matrix Syy = getCovarianceOfObservations();
		Matrix B2 = getFirstEntity().getJacobianOfConstraint().trans();
		Matrix DeltaNSyy = Syy.mult(B2).mult(
                B2.trans().mult(Syy).mult(B2).invert()
				 ).mult(B2.trans()).mult(Syy);
		return  Syy.minus(DeltaNSyy);
	}
	
	public void ensureConstraints() {
		first.ensureConstraints();
	}
	
	public void changeNullspace() {
		first.changeNullspace();
	}


} // end BiRelationalProperty
