package sugr;

import java.lang.reflect.*;
import sugr.linalg.*;

/** Get Instances of classes in an easy way.
 *  @author Thomas Laebe
 * @version
 * <b> 1.0 (04.02.2002 </b> <br>
 * <b> 2.0 (17.06.2003) </b> <br>
 *      delete the constants for the type because they are now represented
 *      as Class objects.
 *  
*/
public class Factory {
	/** Initialize an Entity with given entities and class name.
	 A Matrix and a Vector as initial values should be given.
		@param classname one of an Entity which exists in sugr. 
	 @throws IllegalArgumentException if construction not possible
	 @throws RuntimeException if an Exceptioin while creating the object
	*/
	public static Entity getEntity(Vector v, Matrix cov, String classname)
		throws IllegalArgumentException, RuntimeException {
		return (Entity) getInstance(
			v,
			cov,
			"linalg.Vector",
			"linalg.Matrix",
			classname);
	}
	/** Initialize an Entity with given entities and class ID.
	 A Matrix and a Vector as initial values should be given.
		@param classid  one of POINT_2D,LINE_2D,POINT_3D,LINE_3D PLANE_3D  
	 @throws IllegalArgumentException if construction not possible
	 @throws RuntimeException if an Exceptioin while creating the object
	*/
	public static Entity getEntity(Vector v, Matrix cov, Class classid)
		throws IllegalArgumentException, RuntimeException {
		return (Entity) getInstance(
			v,
			cov,
			"linalg.Vector",
			"linalg.Matrix",
			classid.getName());
	}

	/** Initialize a relation class with given entities and class name.
		@param relation name of the relation: "Incidence", "Identity", 
					  "Parallelity", "Orthogonality", "Distance"
	    @throws IllegalArgumentException if Relation not possible
	    @throws RuntimeException if an Exceptioin while creating the Relation occurs
	*/
	public static Relation getRelationByName(
		Entity e1,
		Entity e2,
		String relation)
		throws IllegalArgumentException, RuntimeException {
                    for(int i=0; i<relationnames.length; i++)
                        if (relation.equals(relationnames[i]))
                            return new Relation(e1, e2, relationclasses[i]);
                    
                    // No Relationanme found
                    throw new IllegalArgumentException("Relationname not found");
	}

	/** Initialize a relation class with given entities and class ID.
		@param relationID name of the relation class: Orthogonal, Equal,
		Incident, Parallel
		@throws IllegalArgumentException if Relation not possible
		@throws RuntimeException if an Exceptioin while creating the Relation occurs
	*/
	public static Relation getRelation(Entity e1, Entity e2, Class relationID) {
		return new Relation(e1, e2, relationID);
	}

	/** Initialize a relation class with given entities and class name.
		@param relationclassname name of the relation class: Orthogonal, Equal,
	           Incident, Parallel, Distance
	    @throws IllegalArgumentException if Relation not possible
	    @throws RuntimeException if an Exceptioin while creating the Relation occurs
	*/
	public static Relation getRelation(
		Entity e1,
		Entity e2,
		String relationclassname)
		throws ClassNotFoundException {
                    	// Get runtime classes
			String firstpart = relationclassname.substring(0, 5);
			Class relationclass;
			if (!firstpart.equals("sugr."))
				relationclass = Class.forName("sugr." + relationclassname);
			else
				relationclass = Class.forName(relationclassname);
		return new Relation(e1, e2, relationclass);
	}

	/** Initialize an object with 2 given parameters  and class name.
		@param classname name of the class. Should be a class available in sugr.
	    @throws IllegalArgumentException if Construction is not possible
	    @throws RuntimeException if an Exception while creating the object occurs
	*/
	public static Object getInstance(Object e1, Object e2, String classname)
		throws IllegalArgumentException, RuntimeException {
		try {
			// Get runtime classes
			String firstpart = classname.substring(0, 5);
			Class relationclass;
			if (!firstpart.equals("sugr."))
				relationclass = Class.forName("sugr." + classname);
			else
				relationclass = Class.forName(classname);

			Class e1class = e1.getClass();
			Class e2class = e2.getClass();

			// Call constructor
			Class[] parameterTypes = new Class[2];
			parameterTypes[0] = e1class;
			parameterTypes[1] = e2class;
			Constructor constructor =
				relationclass.getDeclaredConstructor(parameterTypes);
			Object[] parameters = new Object[2];
			parameters[0] = e1;
			parameters[1] = e2;
			Object r = constructor.newInstance(parameters);

			return r;
		} catch (InstantiationException ex1) {
			throw new IllegalArgumentException();
		} catch (IllegalAccessException ex2) {
			throw new IllegalArgumentException();
		} catch (IllegalArgumentException ex3) {
			throw new IllegalArgumentException();
		} catch (InvocationTargetException ex4) {
			throw new RuntimeException(
				"Error while creating a Relation: "
					+ ex4.getTargetException().toString());
		} catch (ClassNotFoundException ex5) {
			throw new IllegalArgumentException();
		} catch (NoSuchMethodException ex6) {
			throw new IllegalArgumentException();
		}

	}

	/** Initialize an object with 2 given parameters and their tpyes  and class name. The
	    type names are neccessary if the real type ob the object is a subclass of the parameters of the defined constructor.
	@param classname name of the class. Should be a class available in sugr.
	 @throws IllegalArgumentException if Construction is not possible
	 @throws RuntimeException if an Exception while creating the object occurs
	*/
	public static Object getInstance(
		Object e1,
		Object e2,
		String classname_param1,
		String classname_param2,
		String classname)
		throws IllegalArgumentException, RuntimeException {
		try {
			// Get runtime classes
			String firstpart = classname.substring(0, 5);
			Class relationclass;
			if (!firstpart.equals("sugr."))
				relationclass = Class.forName("sugr." + classname);
			else
				relationclass = Class.forName(classname);

			Class e1class;
			firstpart = classname_param1.substring(0, 5);
			if (!firstpart.equals("sugr."))
				e1class = Class.forName("sugr." + classname_param1);
			else
				e1class = Class.forName(classname_param1);
				
			Class e2class;
			firstpart = classname_param2.substring(0, 5);
			if (!firstpart.equals("sugr."))
				e2class = Class.forName("sugr." + classname_param2);
			else
				e2class = Class.forName(classname_param2);

			// Call constructor
			Class[] parameterTypes = new Class[2];
			parameterTypes[0] = e1class;
			parameterTypes[1] = e2class;
			Constructor constructor =
				relationclass.getDeclaredConstructor(parameterTypes);
			Object[] parameters = new Object[2];
			parameters[0] = e1;
			parameters[1] = e2;
			Object r = constructor.newInstance(parameters);

			return r;
		} catch (InstantiationException ex1) {
			throw new IllegalArgumentException();
		} catch (IllegalAccessException ex2) {
			throw new IllegalArgumentException();
		} catch (IllegalArgumentException ex3) {
			throw new IllegalArgumentException();
		} catch (InvocationTargetException ex4) {
			throw new RuntimeException(
				"Error while creating a Relation: "
					+ ex4.getTargetException().toString());
		} catch (ClassNotFoundException ex5) {
			throw new IllegalArgumentException();
		} catch (NoSuchMethodException ex6) {
			throw new IllegalArgumentException();
		}

	}
        
    /// Mapping between relation names and IDs
    private final static String[] relationnames={ "Orthogonality",  "Identity",  "Incidence", "Parallelity"};
    private final static Class [] relationclasses={Orthogonal.class, Equal.class, Incident.class, Parallel.class};
}
