package sugr.GUI;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.net.URL;
import java.util.Random;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import sugr.*;
import sugr.linalg.*;

/**
 * A little Applet to perform statistical tests withing SUGR.
 * A statistical test between two entities can be made. Types
 * of tests are equallity, orthogonality, parallelism and incidence.
 * 
 * 
 * @author Norbert Fischer
 * @version 1.0 initial version. just copied from SwingApplet.java
 * @version 1.1 introduced new methods Factory: simplifies very much!
 * @version 1.1.1 Design of result; Distance removed
 * @version 1.1.2 Fixed: changed
 * 			result = Factory.getRelation(e1,e1,cmbR... into
 * 			result = Factory.getRelation(e1,e2,cmbR
 * @version 1.2  The package lm.jar for formatting the doubles is not needed
 *           any more. Replaced it with simple toString().
 */
public class RelationApplet extends JApplet implements ActionListener
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	//some flags
	private static final int LEFT  =0;
	private static final int RIGHT =1;
	
	//note that the numeration is the same like the 
	//indices in initComboBoxes
	private static final int POINT_2D = 0;
	private static final int LINE_2D  = 1;
	private static final int POINT_3D = 2;
	private static final int LINE_3D  = 3;
	private static final int PLANE_3D = 4;
	
	//Panes containing a table with CovarianceMatrix
	private JTable 	  tblLeft;
	private JTable 	  tblRight;
	private JScrollPane  scPaneLeft;
	private JScrollPane  scPaneRight;
	
	
	//this is visible first when you start the applet
	private Container contentPane;
	private JComboBox cmbLeft;
	private JComboBox cmbRight;
	private JComboBox cmbPreBuild;
	
	//The Textfields containting the coordinates of the entities
	//Note that tfArrayResult must not be editable!!!
	private JTextField[] tfArrayLeft;
	private JTextField[] tfArrayRight;
	
	//Select between covarianceMatrix and confidenceMatrix input
	private JRadioButton bttCovLeft;
	private JRadioButton bttConfLeft;
	private ButtonGroup  bttGroupLeft;
	
	private JRadioButton bttCovRight;
	private JRadioButton bttConfRight;
	private ButtonGroup  bttGroupRight;

	//if two entities exist, this is needed to select whats going on
	private JComboBox    cmbRelation;
	private JLabel       mytext1;
	private JButton 	  bttGo;
	private JLabel		  lblAlpha;
	private JTextField	  tfAlpha;
	
	//needed to show the relation result
	private JLabel		lblDecision;
	private JLabel		lblShowDecision;
	private JLabel 	lblTestValue;
	private JTextField tfTestValue;
	private JLabel		lblThreshold;
	private JTextField	tfThreshold;
	
	//some labels to make it easier to live ;-)
	//contains "homogen", "euklidisch", Bezeichnungen
	private JLabel[] lblArLeft;
	private JLabel[] lblArRight;
	private JLabel   lblVecLeft;
	private JLabel   lblVecRight;
	private JLabel   lblError;
	
	//status line
	private JTextField tfStatus;

	//file containing the prebuidl values
	private static final String preBldFile = "examples.sgr";


	/** Mapping between index of GUI and entity types.*/
	private static final Class [] entitytypes = {Point_2D.class, Line_2D.class, Point_3D.class,
	                                 Line_3D.class, Plane_3D.class};
	                                 
	/** Mapping between index of GUI and relation types.*/
	private static final Class [] relationtypes = {Orthogonal.class, Equal.class,Incident.class,
									 Parallel.class};

	
	/** 
	 * Start RelationApplet as standalone applet.
	 */
	public static void main(String args[])
	{
		JFrame frame = new JFrame("SUGR Relation Applet");

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
       RelationApplet applet = new RelationApplet();
	   applet.init();
	   applet.start(); 
    
	   frame.setContentPane(applet.getContentPane());
	
	   frame.pack();
	   frame.setSize(1000,700);
	   frame.setVisible(true);
	}


  public void init ()
  {
    contentPane = getContentPane();  	
    contentPane.setLayout( null );
    tfStatus = new JTextField("initialising");
    tfStatus.setLocation(0,600);
    tfStatus.setSize(800,20);
    tfStatus.setEditable(false);
    tfStatus.setBackground(Color.lightGray);
    contentPane.add(tfStatus);
    
  	//renders the combobox where someone should select a "bung"
  	initComboPreBuild();
  	
  	initComboBoxes();
  	
	initIcons();
  	tfStatus.setText("Ready");
  }

	/** place icons on the screen */
	private void initIcons()
	{
		try {
			ImageIcon ipblogo;
			try {
				//URL url = ClassLoader.getSystemResource("images/ipblogo.gif");
				URL url = getURL("images/ipblogo.gif");
				ipblogo= new ImageIcon(url,"ipblogo");
			}
			catch (java.lang.NullPointerException e) 
			{
				// No codebase, direct access
				System.err.println("try to direct access of image");
				ipblogo= new ImageIcon("images/ipblogo.gif","ipblogo");
			}
		
			JLabel ipblogolabel= new JLabel(ipblogo);
			ipblogolabel.setToolTipText("Institute for Photogrammetry, University of Bonn");
			ipblogolabel.setLocation(710, 5);
			ipblogolabel.setSize(101,104);
			ipblogolabel.setForeground(Color.gray);
			contentPane.add(ipblogolabel);
		}
		catch (Exception e)
		{
			System.err.println("No icons possible: "+e);
		}
	}


  private void initComboBoxes() 
  {
	//Create the Combo-Box
	//DONT CHANGE this order! If so, change also the 
	//numeration of POINT_2D, LINE_2D ...
    String lang[] = { "Point_2D", "Line_2D", 
    				  "Point_3D", "Line_3D", "Plane_3D" };
    				  
    cmbLeft = new JComboBox();
    cmbRight = new JComboBox();

    for ( int i=0; i<lang.length; i++ ) {
      cmbLeft.addItem( lang[i] );
      cmbRight.addItem( lang[i] );
    }
    cmbLeft.setToolTipText("click to choose an element");
	cmbLeft.setLocation(0, 10);
	cmbLeft.setSize(300,25);
	cmbLeft.addActionListener(
    	//Create new Listener for ComboBoxLeft
    	//Anothor possibility to get this is to make this an oter class
    	new ActionListener() 
    	{
    		public void actionPerformed( ActionEvent e)
    		{
				
				switch (cmbLeft.getSelectedIndex() )
				{
					case POINT_2D: { //Point_2D
						createEntity(LEFT,200,50, 3);break;
					}
					case LINE_2D: { //Line_2D
						createEntity(LEFT,200,50, 3);break;
					}	
					case POINT_3D: { //Point_3D
						createEntity(LEFT,200,50, 4);break;
					}	
					case LINE_3D: { //Line_3D
						createEntity(LEFT,200,50, 6);break;
					}	
					case PLANE_3D: { //Plane_3D
						createEntity(LEFT,200,50, 4);break;
					}
					
					default : break;
				}
    		}
    	} );
    contentPane.add( cmbLeft );
    
    cmbRight.setToolTipText("click to choose an element");
	cmbRight.setLocation(400, 10);
	cmbRight.setSize(300,25);
	cmbRight.addActionListener(
    	//Create new Listener for ComboBoxLeft
    	//Anothor possibility to get this is to make this an oter class
    	new ActionListener() 
    	{
    		public void actionPerformed( ActionEvent e)
    		{
				switch (cmbRight.getSelectedIndex()) 
				{
					case POINT_2D: { //Point_2D
						createEntity(RIGHT,600,50, 3);break;
					}
					case LINE_2D: { //Line_2D
						createEntity(RIGHT,600,50, 3);break;
					}	
					case POINT_3D: { //Point_3D
						createEntity(RIGHT,600,50, 4);break;
					}	
					case LINE_3D: { //Line_3D
						createEntity(RIGHT,600,50, 6);break;
					}	
					case PLANE_3D: { //Plane_3D
						createEntity(RIGHT,600,50, 4);break;
					}	
					
					default : break;
				}
    		}
    	} );
    contentPane.add( cmbRight );
    //End Create Combo-Box
    

    mytext1= new JLabel("Test the above 2 entities for ");
	mytext1.setLocation(80,370);
	mytext1.setSize(200,25);
	mytext1.setVisible(false);
	contentPane.add(mytext1);    

    
	String relations[] = { "Orthogonality" , "Equality", "Incidence", 
					  "Parallelism" };
					  
	cmbRelation = new JComboBox();
	for ( int i=0; i<relations.length; i++ ) {
	  cmbRelation.addItem( relations[i] );
	}
	cmbRelation.setToolTipText("click to choose a Relation you want to test");
	cmbRelation.setLocation(250, 370);
	cmbRelation.setSize(200,25);
	cmbRelation.setVisible(false);
	contentPane.add( cmbRelation );    
	
	
  }
  
  /* 
   * This renders the button which contains some precalculated values for 
   * the entitiesn
   */
  private void initComboPreBuild() 
  {
  	cmbPreBuild = new JComboBox();
  	try{
  		int entries = 0;
		Reader r;
		try {
			URL url = getURL(preBldFile);
			r = new BufferedReader(new InputStreamReader(url.openStream()));
		} 
		catch (java.lang.NullPointerException e) 
		{
			// No codebase, direct access
			System.err.println("type to direct access "+preBldFile);
			r = new BufferedReader(new FileReader(preBldFile));
		}
		StreamTokenizer st = new StreamTokenizer(r);
		// _ ist treated as a part of a word and not as a white space
		st.wordChars('_','_');
		//overread commentar //
		st.slashSlashComments(true);
		st.nextToken();
		//gets number of combobox entries
		if (st.sval.equals("COMBO_ENTRIES"))
		{
			st.nextToken();
			entries = (int)st.nval;
		}
		else {
			tfStatus.setText("Error after ENTRIES");
			return;
		}
		
		//fills the combo-box
		for (int i=0; i<entries; i++)
		{
			st.nextToken();
			while (!st.sval.equals("ELEMENT") )
			{
				st.nextToken();
				while(st.sval==null){
		        	st.nextToken();
		        }
			}
			st.nextToken();
			cmbPreBuild.addItem(st.sval);
		}
		
		r.close();
	
		cmbPreBuild.setToolTipText("click to choose prebuild elements");
		cmbPreBuild.setLocation(710, 180);
		cmbPreBuild.setSize(100,25);
		//Adding the ActionListener for this Combo-Box
		cmbPreBuild.addActionListener(
	    	new ActionListener() 
	    	{
	    		public void actionPerformed(ActionEvent e)
	    		{
	    			try{
						Reader reader;
						try {
							URL url = getURL(preBldFile);
							reader = new BufferedReader(new InputStreamReader(url.openStream()));
						} 
						catch (java.lang.NullPointerException e1) 
						{
						  // No codebase, direct access
						  System.err.println("type to direct access "+preBldFile);
						  reader = new BufferedReader(new FileReader(preBldFile));
						}

	    				StreamTokenizer stok = new StreamTokenizer(reader);
	    				stok.slashSlashComments(true);
						stok.wordChars('_','_');    				
			            stok.nextToken();
			            System.err.println(cmbPreBuild.getSelectedItem());
			            while (!stok.sval.equals(cmbPreBuild.getSelectedItem()) )
			            {
			            	stok.nextToken();
			            	//if number is read no sval will be generated
			            	while(stok.sval==null){
			            		stok.nextToken();
			            	}
			            }
			            
			            stok.nextToken();
			            if (!stok.sval.equals("BEGIN") ){
			            	tfStatus.setText("Keyword BEGIN is missing");
			            	reader.close();
			            	return;
			            }
			            
			            fillElement(LEFT, stok, 200,50);
			            fillElement(RIGHT,stok, 600,50);
			            
			            stok.nextToken();
			            if (!stok.sval.equals("END") ){
			            	tfStatus.setText("Keyword END is missing");
			            }
			            
			            reader.close();
			            
	    			} catch (IOException event) {
	    				tfStatus.setText("Could not read Input-File");
	    			}
	    		}
	    	}
	    );
	    
	    contentPane.add(cmbPreBuild);
	    System.err.println("combo-Box preBuild rendered");
	} catch (FileNotFoundException e) {
		tfStatus.setText("File not found. Not PreBuild-Button rendered");
    } catch (IOException e) {
		tfStatus.setText("File-Input error");
    }	 
  }
  
  private void fillElement (int whatSide, 
  							 StreamTokenizer st, 
  							 int startX, 
  							 int startY)
  {
  	int noOfElements =0;
  	JComboBox cmbBox;
  	if (whatSide==LEFT) cmbBox=cmbLeft;
  	else cmbBox=cmbRight;

	//parses the input-file
  	try
  	{
  		//Check, what element should be created
	  	st.nextToken();
	  	
	  	//falls st.sval == null, dann soll das rechte Element leer bleiben
	  	// bzw _nicht_ verndert werden
	  	if (st.sval.equals("END")) {
	  		tfStatus.setText("right element doesn't need to be rendered");
	  		//StreamTokenizer must be set to one element before
	  		//because input-file contains no double END
	  		st.pushBack();
	  		return;
	  	}
		else if (st.sval.equals("POINT_2D") ) {
			cmbBox.setSelectedIndex(POINT_2D);
			createEntity(whatSide,startX,startY, 3);
			noOfElements = 3;
		}
		else if (st.sval.equals("LINE_2D") ) {
			cmbBox.setSelectedIndex(LINE_2D);
			createEntity(whatSide,startX,startY, 3);
			noOfElements = 3;
		}
		else if (st.sval.equals("POINT_3D") ) {
			cmbBox.setSelectedIndex(POINT_3D);
			createEntity(whatSide,startX,startY, 4);
			noOfElements = 4;
		}
		else if (st.sval.equals("LINE_3D") ) {
			cmbBox.setSelectedIndex(LINE_3D);
			createEntity(whatSide,startX,startY, 6);
			noOfElements = 6;
		}
		else if (st.sval.equals("PLANE_3D") ) {
			cmbBox.setSelectedIndex(PLANE_3D);
			createEntity(whatSide,startX,startY, 4);
			noOfElements = 4;
		}
		else {
			tfStatus.setText("Wrong element in input-file");
			return;
		}
		
		st.nextToken();
		if (!st.sval.equals("Vector") ){
			tfStatus.setText("Keyword _Vector_ is missing");
			return;
		}
		
		//now fill the vector
		for (int i=0; i<noOfElements; i++)
		{
			st.nextToken();
			if (whatSide== LEFT){
				tfArrayLeft[i].setText(new Double(st.nval).toString());
			}
			else {
				tfArrayRight[i].setText(new Double(st.nval).toString());
			}
		}
		
		st.nextToken();
		if (!st.sval.equals("Covariance") ){
			tfStatus.setText("Keyword _Covariance_ is missing");
			return;
		}
		
		//now fill the covariance-matrix
		for (int i=0; i<noOfElements; i++){
			for (int j=i; j<noOfElements; j++){
				st.nextToken();
				if (whatSide == LEFT){
					tblLeft.setValueAt(new Double(st.nval).toString(), i,j);
				}
				else {
					tblRight.setValueAt(new Double(st.nval).toString(), i,j);
				}
			}
		}
		repaint();
		
	} catch (IOException event) {
    	tfStatus.setText("Could not read Input-File");
    }
    tfStatus.setText("Input file read without error");

  }
  	
  
  private void createEntity(int whatSide, 
							 int startPosX,
  							 int startPosY,
  							 int dim)
  {
  	tfStatus.setText("Create Entity");
  	//create local variables of all needed values
  	JTextField[] coordinates;
  	JScrollPane scPane;
  	JComboBox box;
  	JLabel[] lblTxtFields;
  	JLabel lblVec;
  	JRadioButton bttCov;
  	JRadioButton bttConf;
  	ButtonGroup bttGroup;

  	
  	//Decide what side is on
  	if (whatSide == LEFT) 
  	{
  		coordinates = tfArrayLeft;
  		scPane = scPaneLeft;
  		box = cmbLeft;
  		lblTxtFields = lblArLeft;
  		lblVec = lblVecLeft;
  		bttCov = bttCovLeft;
  		bttConf = bttConfLeft;
  		bttGroup = bttGroupLeft;
  	}
  	else 
  	{
  		coordinates =  tfArrayRight;
  		scPane = scPaneRight;
  		box = cmbRight;
  		lblTxtFields = lblArRight;
  		lblVec = lblVecRight;
  		bttCov = bttCovRight;
  		bttConf = bttConfRight;
  		bttGroup = bttGroupRight;
  	}
  	
  	//Delete old components if they exist
  	if (coordinates != null) {
  		tfStatus.setText("removing old values");
  		for (int i=0; i<coordinates.length; i++)
  		{
	  		contentPane.remove(coordinates[i]);
	  		contentPane.remove(lblTxtFields[i]);
  		}
  		contentPane.remove(lblVec);
		coordinates = null;
		lblTxtFields = null;
		lblVec = null;
		contentPane.remove(scPane);
		scPane = null;
		
		contentPane.remove(bttCov);
		contentPane.remove(bttConf);
  	}
	
	tfStatus.setText("setting up!");
	//Create new Labels
	//box.getSelectedIndex has the same numbering as POINT_2D, LINE_2D,...
	lblTxtFields = setUpLabels(box.getSelectedIndex(), startPosX-90, startPosY);
    
    lblVec = new JLabel("Vector");
    lblVec.setLocation(startPosX-200, startPosY);
    lblVec.setSize(50,20);
    lblVec.setForeground(Color.gray);
    contentPane.add(lblVec);
    //end create JLabels. for setting these see below
  	
  	//create Radiobuttons for selection between covariance and confidence
	bttGroup = new ButtonGroup();
	
	bttCov = new JRadioButton("Covariance-Matrix");
	bttCov.setLocation(startPosX-200, 215);
	bttCov.setSize(150,20);
	bttCov.setToolTipText("Select this to handle uncertainty as covariance matrix");
	bttCov.setOpaque(true);
	contentPane.add(bttCov);
	bttGroup.add(bttCov);
	
	bttConf = new JRadioButton("Sigma-Rho-Matrix");
	bttConf.setLocation(startPosX-50, 215);
	bttConf.setSize(150,20);
	bttConf.setToolTipText("Select this to handle uncertainty as confidence matrix");
	bttConf.setOpaque(true);
	contentPane.add(bttConf);
	bttGroup.add(bttConf);
	
	//for defined start
	bttCov.setSelected(true);
	//end create Radiobuttons
  	
  	
  	//create new coordinates
  	coordinates = new JTextField[dim];
  	Random r = new Random();
    for (int i=0; i<dim; i++)
    {
    	coordinates[i] = new JTextField(Integer.toString(r.nextInt(10)) );
    	coordinates[i].setLocation(startPosX, startPosY + 25*i);
    	coordinates[i].setSize(100,20);
    	coordinates[i].setToolTipText("contains coordinates of choosen element");
    	contentPane.add( coordinates[i] );    	

    	//setting is done in setUpLabels. therefore only add them!
		contentPane.add( lblTxtFields[i] );

    }//End create TextFields

    //Create Table

	DefaultTableModel dm = new DefaultTableModel(dim,dim)
	{
		/**
		 * Class version number for serialization
		 */
		private static final long serialVersionUID = 1L;

		public boolean isCellEditable(int row, int column) {
			if(row > column) return false;
			return true;
		}
		
		public void setValueAt(Object aValue, int row, int column){
			super.setValueAt(aValue, row, column);
			super.setValueAt(aValue, column, row);
		}
	};
    JTable table = new JTable(dm);
	table.setShowGrid(true);
	table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
	table.setToolTipText("contains covariance-matrix of choosen element above. Click white fields to modify the values");
	for (int i=0; i<dim; i++){
		for (int j=0; j<dim; j++) {
			if (i==j) 
				table.setValueAt("1.0",i,j);
			else
				table.setValueAt("0.0",i,j);	

		//	radom values: table.setValueAt(Double.toString(r.nextDouble()),i,j);
		}
	}
	table.getTableHeader().setVisible(false);
	table.scrollRectToVisible(table.getCellRect(0,0,false));
	table.setDefaultRenderer(Object.class, new GrayBackgroundTableCellRenderer() );
	
	scPane = new JScrollPane(table);
	scPane.setLocation(startPosX-200,220);
	scPane.setSize(300,125);
	scPane.setToolTipText("contains covariance-matrix of choosen element above. Click white fields to modify the values");
	scPane.setColumnHeader(null);
	contentPane.add( scPane );
	
	repaint();
	//end create Table
    
    if (whatSide == LEFT) 
    {
    	tfArrayLeft = coordinates;
    	scPaneLeft = scPane;
    	tblLeft = table;
    	lblArLeft = lblTxtFields;
    	lblVecLeft = lblVec;
    	bttCovLeft = bttCov;
    	bttConfLeft = bttConf;
    	bttGroupLeft = bttGroup;
    }
  	else 
  	{
  		tfArrayRight = coordinates;
  		scPaneRight = scPane;
  		tblRight = table;
  		lblArRight = lblTxtFields;
  		lblVecRight = lblVec;
  		bttCovRight = bttCov;
    	bttConfRight = bttConf;
    	bttGroupRight = bttGroup;
  	}
  	
  	tfStatus.setText("Create Entity done");
  	//if two Entities exist, render the selection (Join, Intersect)
  	if (scPaneLeft != null && scPaneRight != null && bttGo == null) 
  	{
  		tfStatus.setText("Create Selection");
  		createSelection();
  		tfStatus.setText("Create Selection done");
  	}
  	repaint();
  }
  
  private void createSelection()
  {
 	JSeparator sep = new JSeparator();
  	sep.setLocation(0,350);
  	sep.setSize(821,100);
  	contentPane.add(sep);
  	
  	lblAlpha = new JLabel("Alpha");
	lblAlpha.setLocation(500, 370);
	lblAlpha.setSize(35,20);
	lblAlpha.setToolTipText("significance niveau");
	contentPane.add(lblAlpha);  	

	tfAlpha = new JTextField("0.95");	
	tfAlpha.setLocation(540, 370);
	tfAlpha.setSize(50,20);
	tfAlpha.setToolTipText("significence niveau: Please enter values between 0 and 1");
	contentPane.add(tfAlpha);  	

  	
  	/*
  	 * Ich verstehe es nicht: Das was hier ausgekommentiert steht,
  	 * funktioniert HIER nicht. Die ComboBox ist NICHT sichtbar. Diese ComboBox
  	 * wird nun in initComboBoxes() gebaut und unsichtbar gestellt, hier wieder
  	 * sichtbar. Dort funktioniert es.
  	 * 
  	 * Was ist in dieser Methode anders als in initComboBoxes?!?
  	 */
  	//Distance has been removed
//	String relations[] = { "Incidence", "Identity", 
//					  "Parallelism", "Orthogonality"};
//					  
//	cmbRelation = new JComboBox();
//	for ( int i=0; i<relations.length; i++ ) {
//	  cmbRelation.addItem( relations[i] );
//	}
//	cmbRelation.setToolTipText("click to choose a Relation you want to test");
//	cmbRelation.setLocation(270, 350);
//	cmbRelation.setSize(200,25);
//	contentPane.add( cmbRelation );

	cmbRelation.setVisible(true);
  	mytext1.setVisible(true);
  	
  	bttGo = new JButton("GO!");
  	bttGo.setLocation(300,400);
  	bttGo.setSize(100,30);
  	bttGo.setToolTipText("You know what to do :-)");
  	bttGo.addActionListener(this);
  	contentPane.add(bttGo);
  }
  

  
  private Vector getEntityVector( JTextField[] tfArray)
  {
  	int tfSize = tfArray.length;
  	Vector v = LinearAlgebraFactory.getVector(tfSize);
  	for (int i=0; i<tfSize; i++)
  	{
  		v.set(i, Double.parseDouble(tfArray[i].getText()) );
  	}
  	
  	return v;
  }
  
  
  private Matrix getEntityCovMatrix( JTable table)
  {
  	int dim = table.getRowCount();
  	
  	Matrix cov = LinearAlgebraFactory.getMatrix(dim, dim);
  	for (int i=0; i<dim; i++)
  	{
  		for (int j=0; j<dim; j++)
  		{
  			cov.set(i,j, Double.parseDouble((String)table.getValueAt(i,j)));
  		}
  	}
  	return cov;
  }
  
  //is only registered with the GO!-Button (Selection)
  public void actionPerformed(ActionEvent e)
  {
	removeOldResult();
	repaint();
  	
  	System.err.println("Button Go! was clicked");
  	
  	Vector vLeft = getEntityVector (tfArrayLeft);
  	Vector vRight = getEntityVector (tfArrayRight);
  	
  	Matrix covLeft = getEntityCovMatrix(tblLeft);
  	Matrix covRight = getEntityCovMatrix(tblRight);
  	
	//if covariance -> check positive definite
	//if confidence -> check correlation coefficient between -1 and 1
	System.err.println("checking matrices...");
	if (bttConfLeft.isSelected())
	{
		System.err.println("confidence matrix Left");
		covLeft = checkConfidence(covLeft);
		if (covLeft==null)
		{
			tfStatus.setText("one correlation coefficient of left matrix is not between -1 and 1");
			return;
		}
	}
	else
	{
		System.err.println("covariance matrix Left");
		//check postive definite of covariance matrices
		//here it is enough to test semi definite (ask Prof Frstner)
		if ( !covLeft.isSemiDefinite())
		{
			tfStatus.setText("Covariance-Matrix of left element is not semi definite");
			return;
		}
	}

	if (bttConfRight.isSelected())
	{
		System.err.println("confidence matrix Right");
		covRight = checkConfidence(covRight);
		if (covRight==null)
		{
			tfStatus.setText("one correlation coefficient of right matrix is not between -1 and 1");
			return;
		}
	}
	else
	{  	  	
		System.err.println("covariance matrix Right");
		if ( !covRight.isSemiDefinite())
		{
			tfStatus.setText("Covariance-Matrix of right element is not semi definite");
			return;
		}
	}
	//end check matrices
  	  	
  	  	
  	Relation result=null;
  	
  	tfStatus.setText("Calculating...");
  	/*
  	 * This part simplifies the check if, a combination of entities is possible
  	 * only this three lines are enough to perform this task!!!
  	 */

  	Entity e1 = Factory.getEntity(vLeft,  covLeft,  entitytypes[cmbLeft.getSelectedIndex()]);
	Entity e2 = Factory.getEntity(vRight, covRight, entitytypes[cmbRight.getSelectedIndex()]);  	
  	
  	try{
		result = Factory.getRelation(e1,e2,relationtypes[cmbRelation.getSelectedIndex()]);	
  	//Das hier wird erst mal SEHR grob abgefangen
  	} catch (IllegalArgumentException excep) 
  	{
  		result=null;
  	}
  	//If there exist no join or intersect match, quit
  	if (result == null) 
  	{
  		lblError = new JLabel("This combination of element and relation is not possible!");
  		lblError.setLocation(150, 500);
  		lblError.setSize(400,40);
  		lblError.setForeground(Color.gray);
  		contentPane.add(lblError);
  		repaint();
  		System.err.println("There is no result");
  		tfStatus.setText("no result possible");
  		return;
  	}

	//sets new alpha-value. new testvalue is calculated immediately
	result.setAlpha(Double.parseDouble(tfAlpha.getText()) );
 	
  	System.err.println("render result");
  	//render the labels
	lblDecision = new JLabel("Decision");
	lblDecision.setLocation(200,470);
	lblDecision.setSize(100,20);
	lblDecision.setToolTipText("is the relation true or not");
	contentPane.add(lblDecision);
	contentPane.repaint();
	
	lblTestValue = new JLabel("Testvalue");
	lblTestValue.setLocation(200,500);
	lblTestValue.setSize(100,20);
	lblTestValue.setToolTipText("contains the calculated testvalue = d^t Sigma_dd^+ d");
	contentPane.add(lblTestValue);
	contentPane.repaint();
	
	lblThreshold = new JLabel("Threshold");
	lblThreshold.setLocation(200,540);
	lblThreshold.setSize(100,20);
	lblThreshold.setToolTipText("contains the threshold chi^2_{1-alpha;n} for the testvalue. " +
	"Testvalue must be smaller than this for gaining a true relation");
	contentPane.add(lblThreshold);
	contentPane.repaint();
	
	//end render labels
	
	//renders the calculated results
	lblShowDecision = new JLabel();
	if (result.decision()) lblShowDecision.setText("TRUE");
	else lblShowDecision.setText("FALSE");
	lblShowDecision.setLocation(300,470);
	lblShowDecision.setSize(150,20);
	lblShowDecision.setToolTipText("is the relation true or not");
	lblShowDecision.setFont(new Font(null, Font.BOLD, 18) );
	contentPane.add(lblShowDecision);
	contentPane.repaint();
	

	tfTestValue = new JTextField(new Double(result.testValue()).toString() );
	tfTestValue.setEditable(false);
	tfTestValue.setLocation(270,500);
	tfTestValue.setSize(140,20);
	tfTestValue.setToolTipText("contains the calculated testvalue");
	contentPane.add(tfTestValue);
	contentPane.repaint();
	
	
	tfThreshold = new JTextField(new Double(result.testThreshold()).toString() );
	tfThreshold.setEditable(false);
	tfThreshold.setLocation(270,540);
	tfThreshold.setSize(140,20);
	tfThreshold.setToolTipText("contains the calculated threshold chi^2");
	contentPane.add(tfThreshold);
	contentPane.repaint();
	
	//end render results  	
	
	tfStatus.setText("Result done");
	repaint();
  }

/* called by actionPerformed. Removes former result and cleans  
 * up this area
 */
private void removeOldResult()
{
	//Delete old values and fields
	  if (lblDecision != null)
	  {
		  tfStatus.setText("Deleting old result");
		  System.err.println("Deleting old result");

		  contentPane.remove(lblDecision);
		  contentPane.remove(lblShowDecision);
		  contentPane.remove(lblTestValue);
		  contentPane.remove(tfTestValue);
		  contentPane.remove(lblThreshold);
		  contentPane.remove(tfThreshold);
		  
		  lblDecision 		= null;
		  lblShowDecision 	= null;
		  lblTestValue 		= null;
		  tfTestValue 		= null;
 		  lblThreshold 		= null;
		  tfThreshold 		= null;
	  }
	//if errorMsg "This Combination of..." is visible, just remove it
	if (lblError != null) 
	{
		contentPane.remove(lblError);
		lblError = null;
	}
	
	repaint(); //update the applet
	//end delete
}
  
  
  private Matrix checkConfidence(Matrix confidence)
  {
  	//check wether one corr. coeff is not between -1 and 1
  	for(int i=0; i<confidence.getColumnDimension(); i++){
		for(int j=i+1; j<confidence.getColumnDimension(); j++){
			if ( (confidence.get(i,j)<-1) || (confidence.get(i,j)>1) )
			{
				tfStatus.setText("one correlation coefficient is not between -1 and 1");
				//and so null is returned. check this if this method is called!
				return null;
			}
		}
	}
	//generate covariance Matrix from confidence matrix
	int dim = confidence.getColumnDimension();
	Matrix diagonal = LinearAlgebraFactory.getMatrix( dim, dim);
	for (int i=0; i<dim; i++)
	{
		if (confidence.get(i,i)<0) {
			tfStatus.setText("variances must be >=0 !");
			return null;
		}
		diagonal.set(i,i, confidence.get(i,i));
		confidence.set(i,i, 1);
	}
	//confidence is symmetric -> start calc of covariance-matrix:
	//	cov = diagonal * confidence * diagonal (S*R*S)
	Matrix cov = diagonal.mult(confidence).mult(diagonal);
	//not null but the covariance matrix is returned
	return cov;
  }
  
  
  private JLabel[] setUpLabels(int whatEntity, 
  						       int posX,
  						       int posY)
  {
  	JLabel[] lbl = null;
  	switch (whatEntity)
  	{
  		case POINT_2D:
  		{
  			lbl = new JLabel[3];
			lbl[0] = new JLabel("euclidian");
			lbl[1] = new JLabel("euclidian");
			lbl[2] = new JLabel("homogeneous");
			break;
  		}
  		case LINE_2D:
  		{
  			lbl = new JLabel[3];
			lbl[0] = new JLabel("homogeneous");
			lbl[1] = new JLabel("homogeneous");
			lbl[2] = new JLabel("euclidian");
			break;
  		}
  		case POINT_3D:
  		{
  			lbl = new JLabel[4];
			lbl[0] = new JLabel("euclidian");
			lbl[1] = new JLabel("euclidian");
			lbl[2] = new JLabel("euclidian");
			lbl[3] = new JLabel("homogeneous");
			break;
  		}
  		case LINE_3D:
  		{
  			lbl = new JLabel[6];
			lbl[0] = new JLabel("homogeneous");
			lbl[1] = new JLabel("homogeneous");
			lbl[2] = new JLabel("homogeneous");
			lbl[3] = new JLabel("euclidian");
			lbl[4] = new JLabel("euclidian");
			lbl[5] = new JLabel("euclidian");
			break;
  		}
  		case PLANE_3D:
  		{
  			lbl = new JLabel[4];
			lbl[0] = new JLabel("homogeneous");
			lbl[1] = new JLabel("homogeneous");
			lbl[2] = new JLabel("homogeneous");
			lbl[3] = new JLabel("euclidian");
			break;
  		}
  	}//end switch
  	
  	for (int i=0; i<lbl.length; i++)
  	{
  		lbl[i].setLocation(posX, posY +25*i);
  		lbl[i].setSize(100,20);
  		lbl[i].setToolTipText("shows type of coordinate");
   	}
   	
  	return lbl;

  }

	/** Get an URL out of a filename (to work across a network in a
        browser !*/
	protected URL getURL(String filename) {
		URL codeBase;
		URL url;

		try {
			codeBase = getCodeBase();
			url = new URL(codeBase, filename);
        } catch (java.net.MalformedURLException e) {
            System.err.println("Couldn't create image: " +
                               "badly specified URL");
            return null;
        }
    
        return url;
    }	
}



