/** Java class "Incident.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Contains methods if two elements are incident or not. You can retrieve then a lot 
 * of information about this test, for example: chi-value used, 
 * the testvalue and so on.
 * 
 * 
 * @author Norbert Fischer
 * @version 
 * <b> 1.1 (12.2.2003) </b> <br>
 * 		Fixed: compute_distance -> Point_2D/Point_2D
 * 				changed into Point_2D/Line_2D <br>
 * 		Fixed: compute_distance -> Point_3D/Line_3D
 *				missing .trans() a dist[0]-calculation <br>
 *
 * <b> 1.2 (20.2.2003) </b> <br>
 * 		Fixed: computation of error propagation (ep) was wrong everywhere
 * 			(except Point_3D/Line_3D, Line_3D/Plane_3D). ep needs also the 
 * 			vector of the elements - just adding covariances is not enough ;-) <br>
 *  				 		
 * <b> 1.2.1 (24.2.2003) </b> <br>
 * 		Changed: works now on copies of entities. This preserves the 
 * 			original input values from changing!!!
 * <b> 2.0 (June 2003)</b> <br>
 * 		Is now derived from BiRelationalProperty !!!
 * </p>
 */
public class Incident extends BiRelationalProperty {

	///////////////////////////////////////
	// operations

	/**
	 * <p>
	 * Return Matrix Jacobian A of the first Entity and the relation type.
	 * </p><p>
	 * 
	 * @return Matrix Jacobian A
	 * </p>
	 */
	public Matrix getJacobian() {
		return first.getJacobianOfRelation(Incident.class, secondEntityType);
	}
	/**
	 * <p>
	 * Compute degrees of freedom of that Relation
	 * </p><p>
	 * 
	 * @return degrees of freedom of that Relation
	 * </p>
	 */
	public int getDof() {
		return first.getDofOfRelation(Incident.class, secondEntityType);
	}

	/**
	 * Build up an Incident-Relation between two entities 
	 * @param e an entity
	 * @param secondClass the other class
	 */
	public Incident(Entity e, Class secondClass) {
		super(e, secondClass);
	} // end Incident        

	/**
	 * Build up an Incident-Relation between Point_2D and Line_2D 
	 * @param x  a Point_2D
	 * @param secondClass could be a Line_2D
	 */
	public Incident(Point_2D x, Class secondClass) {
		super(x, secondClass);
	} // end Incident        

	/**
	 * Build up an Incident-Relation between Point_3D and Plane_3D 
	 * @param x  a Point_3D
	 * @param secondClass could be a Plane_3D or Line_3D
	 */
	public Incident(Point_3D x, Class secondClass) {
		super(x, secondClass);
	} // end Incident        

	/**	
	 * Build up an Incident-Relation between two Line_3D 
	 * @param l  first Line_3D
	 * @param secondClass could be a Line_3D or Plane_3D or Point_3D
	 */
	public Incident(Line_3D l, Class secondClass) {
		super(l, secondClass);
	} // end Incident        

	/**
	 * Build up an Incident-Relation between Point_2D and Line_2D 
	 * @param l  a Line_2D
	 * @param secondClass could be a Point_2D
	 */
	public Incident(Line_2D l, Class secondClass) {
		super(l, secondClass);
	} // end Incident        

	/**
	 * Build up an Incident-Relation between Point_3D and Plane_3D
	 * @param a  a Plane_3D 
	 * @param secondClass could be a Point_3D or Line_3D
	 */
	public Incident(Plane_3D a, Class secondClass) {
		super(a, secondClass);
	} // end Incident        

	/** <p>
	 * Returns the Covariance Matrix of the row-vector of the Jacobian Matrix A of the first Entity and the relation type.
	 * </p>
	 *
	 * @param row Row vector, for which the Covariance matrix is to be returned
	 * @return covariance of the row vector of the jacobian
	 */
	public Matrix getCovarianceOfJacobianRow(int row) {
		return first.getCovarianceOfJacobianRowOfRelation(
			Incident.class,
			secondEntityType,
			row);
	}

} // end Incident
