/** Java class "Point_3D.java" generated from Poseidon for UML. 
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr;

import sugr.linalg.*;

/**
 * Implementation of Point_3D
 * 
 * 
 * @author Norbert Fischer
 * 
 * @version 1.1 (25.1.2004)
 * 		Changed Constructor Point_3D(Plane, Line) and Point_3D(Plane,Plane,Plane) to 
 * 		not use Construction class anymore. Calculation is now done in Constructor 
 * 		itself. By Achim Knigs
 */
public class Point_3D extends Object_3D {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	
	private static final int DIM = 4;
	private static final int DOF = 3;

	///////////////////////////////////////
	// operations

	public Point_3D(Line_3D L1, Line_3D L2) {
		double f = getConditionFactorF(L1, L2);
		if (f != 1) {
			L1.doCondition(f);
			L2.doCondition(f);
		}

		RelationalProperty[] rprops = new RelationalProperty[2];

		rprops[0] = new Incident(L1, Point_3D.class);
		rprops[1] = new Incident(L2, Point_3D.class);

		if ((L1.getCov().vec().scalarmult(L1.getCov().vec()) == 0)
			|| (L2.getCov().vec().scalarmult(L2.getCov().vec()) == 0))
			approximate(rprops);
		else {
			RelationalProperty[] yhat = new RelationalProperty[rprops.length];
			double[] sigma0 = new double[1];
			int[] iterations = new int[1];			
			if (!estimate(rprops, true, MAXITER, yhat, sigma0, iterations, false)) {
				throw new IllegalArgumentException("Convergence not reached!");
			}
		}

		if (f != 1) {
			L1.doCondition(1 / f);
			L2.doCondition(1 / f);
			doCondition(1 / f);
		}
	} // end Point_3D 

	/**
	 * Constructor for a new Point_3D
	 * @param _v a Vector with dimension 4. Contains the coordinates of the new Point_3D
	 * @param _cov 4x4 covariance matrix
	 */
	public Point_3D(Vector _v, Matrix _cov) {
		checkElement(_v, DIM);
		checkCovDim(_cov, DIM);
		v = _v.copy();
		cov = _cov.copy();
	}

	/**
	 * Constructs a new point by an intersection of line and plane. Error 
	 * propagation will be done automatically
	 * @param plane  a Plane_3D
	 * @param line   a Line_3D
	 */
	public Point_3D(Plane_3D plane, Line_3D line) {

		Point_3D x = new Point_3D();
        
		x.setV( plane.Pi().trans().mult(line.getV()) );
        
		// Set Covariance Matrix:
		//PiDual(b) ist nach Seite 3 negativ, aber das hebt sich in der Formel raus
		x.setCov( calcCov( plane.Pi().trans(), plane.getCov(), 
						   line.Gamma().trans(), line.getCov())  );
		checkElement(x.getV(), DIM);
		setV(x.getV());
		setCov(x.getCov());
	} // end Point_3D        

	/**
	 * Constructs a new point by an intersection of line and plane. Error 
	 * propagation will be done automatically
	 * @param plane  a Plane_3D
	 * @param line   a Line_3D
	 */
	public Point_3D(Line_3D line, Plane_3D plane) {
		this(plane, line);
	}

	/**
	 * Constructs a new point by an intersection of three planes. Error 
	 * propagation will be done automatically
	 */
	public Point_3D(Plane_3D plane1, Plane_3D plane2, Plane_3D plane3) {

		Point_3D x = new Point_3D(new Line_3D(plane1,plane2), plane3);
		checkElement(x.getV(), DIM);
		setV(x.getV());
		setCov(x.getCov());
	} // end Point_3D        

	/**
	 * Constructs a new Point_3D by setting all values directly. 
	 * covariance matrix is set to 0 so is has no error
	 */
	public Point_3D(double x, double y, double z, double w) {

		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, w);

		checkElement(v, DIM);
		setV(v);

		// A null-Matrix is generated. Makes sense, since Entity
		// is without any error
		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Point_3D        

	/**
	 * Constructs a new point_3D by setting all values directly. 
	 * With covariance. 
	 * @param cov    4x4 Matrix
	 */
	public Point_3D(double x, double y, double z, double w, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, w);

		checkElement(v, DIM);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Point_3D        

	/**
	 * Constructs a new point. homogeneous part is set to 1 and 
	 * covariance matrix is set to 0 so it has no error
	 * @param x   x-part of point
	 * @param y   y-part of point
	 * @param z   z-part of point
	 */
	public Point_3D(double x, double y, double z) {

		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, 1);
		setV(v);

		Matrix cov = LinearAlgebraFactory.getMatrix(DIM, DIM);
		setCov(cov);
	} // end Point_3D     
	
	
	/** Construct a Point with known accuracies for x,y and z.
	 *  If you have the 3 coordinates of a 2D-Point and accuracies for
	 *  these three coordinates (ignoring correlations) then you can
	 *  use this (shortcut) constructor to generate a SUGR-Point with
	 *  covariance matrix. The generated covariance is
	 *  | sigma_x^2    0            0          0|
	 *  |     0     sigma_y^2       0          0|
	 *  |     0        0         sigma_z^2     0|
	 *  |     0        0            0          0|
	 *  and the homogeneous part is set to 1.
	 *  
	 *  @param x first coordinate of point
	 *  @param y second coordinate of point
	 *  @param z third coordinate of point
	 *  @param sigma_x accuracy of x (standard deviation)
	 *  @param sigma_y accuracy of y (standard deviation)
	 *  @param sigma_z accuracy of z (standard deviation)
	 */
	public Point_3D(double x, double y, double z, 
			double sigma_x, double sigma_y, double sigma_z) {
		Matrix cov=LinearAlgebraFactory.getMatrix(DIM,DIM);
		cov.set(0,0, sigma_x*sigma_x);
		cov.set(1,1, sigma_y*sigma_y);
		cov.set(2,2, sigma_z*sigma_z);

		Vector vec = LinearAlgebraFactory.getVector(DIM);
		vec.set(0, x);
		vec.set(1, y);
		vec.set(2, z);
		vec.set(3, 1);
		
		setV(vec);
		setCov(cov);
	}

	

	/**
	/**
	 * Constructs a new point with covariance. homogeneous part is set to 1. 
	 * @param x   x-part of point
	 * @param y   y-part of point
	 * @param z   z-part of point
	 * @param cov   4x4 Matrix
	 */
	public Point_3D(double x, double y, double z, Matrix cov) {
		checkCovDim(cov, DIM);
		Vector v = LinearAlgebraFactory.getVector(DIM);
		v.set(0, x);
		v.set(1, y);
		v.set(2, z);
		v.set(3, 1);
		setV(v);
		checkCovDim(cov, DIM);
		setCov(cov.copy());
	} // end Point_3D        


	/**
	 * Default Constructor
	 */
	public Point_3D() {
	} // end Point_3D

	/**
	 * Estimates a new point.
	 * @param rprops an array of constraints.
	 */
	public Point_3D(RelationalProperty[] rprops) {
		if (!estimate(rprops))
			throw new IllegalArgumentException("Convergence not reached!");
	}

	/**
	 * returns the euclidean part of this point
	 * @return a Vector with 3 coordinates
	 */
	public Vector getEuclideanPart() {
		Vector vec = LinearAlgebraFactory.getVector(3);
		Vector v = getV();
		vec.set(0, v.get(0));
		vec.set(1, v.get(1));
		vec.set(2, v.get(2));

		return vec;
	} // end getEuclideanPart        

	/**
	 * returns the homogeneous part of this point
	 * @return a Vector with 1 coordinate
	 */
	public Vector getHomogeneousPart() {
		Vector vec = LinearAlgebraFactory.getVector(1);
		vec.set(0, getV().get(3));

		return vec;
	} // end getHomogeneousPart        

	/**
	  * Compute the (euclidean) distance to another entity. This
	  * is of course not possible for all combinations.
	  * In the case of an illegal combination an IllegalArgumentException
	  * should be thrown.
	  * @return the distance to the given entity
	  * @param e  the entity to which a distance should be computed.
	 */
	public double getDistance(Entity e) {
		if (e instanceof Point_3D) {
			Vector tmp = e.getEuclideanPart().mult(getHomogeneousPart().get(0));
			tmp =
				tmp.minus(
					getEuclideanPart().mult(e.getHomogeneousPart().get(0)));

			return tmp.getLength()
				/ Math.abs(
					getHomogeneousPart().get(0)
						* e.getHomogeneousPart().get(0));
		}

		if (e instanceof Line_3D) {
			Vector tmp =
				e.getHomogeneousPart().SkewSym().mult(getEuclideanPart());
			tmp =
				tmp.plus(
					e.getEuclideanPart().mult(getHomogeneousPart().get(0)));
			return tmp.getLength()
				/ e
					.getHomogeneousPart()
					.mult(getHomogeneousPart().get(0))
					.getLength();
		}
		
		if (e instanceof Plane_3D)
			return e.getDistance(this); // implemented in Plane_3D
			 
		throw new IllegalArgumentException(
			"Line_3D.distance: distance between "
				+ e.getClass()
				+ " not possible");
	}

	/** Get a matrix W(f) for conditioning. This
	 *  matrix can be used to transform an entity to a conditioned one.
	 * @param f the conditioning factor
	 * @return the condition matrix
	 */
	protected Matrix getConditionMatrix(double f) {
		Matrix w = LinearAlgebraFactory.getMatrix(4,4);
		w.set(0, 0, f);
		w.set(1, 1, f);
		w.set(2, 2, f);
		w.set(3, 3, 1);
		return w;
	}

	/**
	 * returns a copy of this point. Actually it works like clone
	 */
	public Point_3D copy() {
		return new Point_3D(v, cov);
	}

	/**
	 * returns dimension=4 for this point
	 */
	public int getDimension() {
		return DIM;
	}

	/**
	 * returns dof=3 of this point
	 */
	public int getDegreesOfFreedom() {
		return DOF;
	}

	/**
	 * returns the pi-jacobian of this point
	 * @return 6x4 Matrix
	 */
	public Matrix Pi() {
		return Pi(getV());
	}

	/**
	 * returns the pi-jacobian of this point
	 * @return 6x4 Matrix
	 */
	public static Matrix Pi(Vector v_in) {
		Matrix Pi = LinearAlgebraFactory.getMatrix(6, 4);
		double u = v_in.get(0);
		double v = v_in.get(1);
		double w = v_in.get(2);
		double t = v_in.get(3);
		Pi.set(0, 0, t);
		Pi.set(0, 3, -u);

		Pi.set(1, 1, t);
		Pi.set(1, 3, -v);

		Pi.set(2, 2, t);
		Pi.set(2, 3, -w);

		Pi.set(3, 1, -w);
		Pi.set(3, 2, v);

		Pi.set(4, 0, w);
		Pi.set(4, 2, -u);

		Pi.set(5, 0, -v);
		Pi.set(5, 1, u);
		return Pi;
	}
	
	/**
	 * Returns C*PI matrix the dual PI-matrix
	 * @return 6x4 Matrix
	 */
	public Matrix PiDual() {
		return PiDual(getV());
	}


	/**
	 * Returns C*PI matrix the dual PI-matrix
	 * This matrix
	 * differs from that of the dissertation because of
	 * other definition of C  
	 * @return 6x4 Matrix
	 */
	public static Matrix PiDual(Vector v_in) {
		Matrix PiD = LinearAlgebraFactory.getMatrix(6, 4);
		double a = v_in.get(0);
		double b = v_in.get(1);
		double c = v_in.get(2);
		double d = v_in.get(3);
		PiD.set(0, 1, c);
		PiD.set(0, 2, -b);

		PiD.set(1, 0, -c);
		PiD.set(1, 2, a);

		PiD.set(2, 0, b);
		PiD.set(2, 1, -a);

		PiD.set(3, 0, -d);
		PiD.set(3, 3, a);

		PiD.set(4, 1, -d);
		PiD.set(4, 3, b);

		PiD.set(5, 2, -d);
		PiD.set(5, 3, c);
		return PiD;
	} // end PiDual  

	/**
	 * returns the part of the covariance matrix, which contains the
	 * homogeneous part
	 * @return 1x1 Matrix
	 */
	public Matrix getJacobianHomogeneousPart() {
		return cov.getMatrix(3, 3, 3, 3);
	}

	/**
	 * normalizes in that way, that the homogeneous part will be 1
	 */
	public void normalizeEuclidean() {
		double factor = getV().get(3);

		Matrix jacobi = LinearAlgebraFactory.getMatrixId(DIM);
		jacobi = jacobi.mult(1.0 / factor);
		jacobi.set(3, 3, 0);
		jacobi.set(0, 3, -getV().get(0) / (factor * factor));
		jacobi.set(1, 3, -getV().get(1) / (factor * factor));
		jacobi.set(2, 3, -getV().get(2) / (factor * factor));
		setCov(jacobi.mult(getCov()).mult(jacobi.trans()));

		setV(getV().mult(1.0 / factor));

	}

	/**
	 * Return Jacobian Matrix of that entity in combination with a 
	 * relation. If that entity is
	 * part of a relation then this relation has a constraint 
	 * f(firstEntity)*secondEntity=g(secondEntity)*firstEntity=0. f and g
	 * are the Jacobians of these linear formulas. So return f or g (the choice
	 * f or g is given by the type of the entity !)
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Jacobian matrix (or part of the constraint formula)
	 */
	public Matrix getJacobianOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Plane_3D.class))
				return getV().getMatrix().trans();
			if (partnerType.equals(Line_3D.class))
				return PiDual().trans();

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_3D.class))
				return Pi();

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());

	}

	/**
	 * <p>
	 * Compute degrees of freedom of a relation. 
	 * </p><p>
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return degrees of freedom for the relation
	 * </p>
	 */
	public int getDofOfRelation(Class relationType, Class partnerType)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Plane_3D.class))
				return 1;
			if (partnerType.equals(Line_3D.class))
				return 2;
		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_3D.class))
				return 3;
		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());
	}
	
	/** Get the reduced Matrix A and B from a relational 
	 * property. A is the Jacobian of this Entity (the unknown),
	 * B is the Jacobian of the observation (the relational property).
	 * The Jacobian of the oberservation depends (only) on the unknown and
	 * the Jacobian of the unknown depends (only) of the oberservation !
	 * This method is overwritten to handle some the special cases.
	 * 
	 * @param rprop the relational Property
	 * @return A in array element 0, B in array Element 1.
	 */
	protected Matrix[] getReducedAB(RelationalProperty rprop) {
		BiRelationalProperty r = (BiRelationalProperty) rprop;
		if (r.getClass().equals(Incident.class)) {
			if (r.getFirstEntity().getClass().equals(Line_3D.class)) {
				Matrix A = r.getJacobian();
				Matrix B = getJacobianOfRelation(r);
				Matrix[] AB = reduceFirstMatrix(A, B, r.getDof());
				return AB;
			}
			else
				return super.getReducedAB(rprop);
		}
		else
			return super.getReducedAB(rprop);
	}
	

	/** Return Covariance of a Row of the Jacobian Matrix of that entity in combination with a relation.
	 * @param relationType the type of the relation
	 * @param partnerType the type of the other partner in the relation
	 * @return Covariance of the row of the Jacobian matrix
	 */
	public Matrix getCovarianceOfJacobianRowOfRelation(
		Class relationType,
		Class partnerType,
		int row)
		throws IllegalArgumentException {
		if (relationType.equals(Incident.class)) {
			if (partnerType.equals(Plane_3D.class)) {
				if (row != 0)
					throw new RuntimeException("Row out of range!");
				else
					return getCov(); // getV().getMatrix().trans();
			}
			if (partnerType.equals(Line_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(6, 6);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 2, 1, 3 };
							int[] yyy = { 1, 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 2, 0, 3 };
							int[] yyy = { 0, 2, 4 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 1, 0, 3 };
							int[] yyy = { 0, 1, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 0, 1, 2 };
							int[] yyy = { 3, 4, 5 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}

		} else if (relationType.equals(Equal.class)) {
			if (partnerType.equals(Point_3D.class)) {
				Matrix result = LinearAlgebraFactory.getMatrix(4, 4);
				switch (row) {
					case 0 :
						{
							int[] xxx = { 3, 0 };
							int[] yyy = { 0, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 1 :
						{
							int[] xxx = { 3, 1 };
							int[] yyy = { 1, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 2 :
						{
							int[] xxx = { 3, 2 };
							int[] yyy = { 2, 3 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 3 :
						{
							int[] xxx = { 2, 1 };
							int[] yyy = { 1, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 4 :
						{
							int[] xxx = { 2, 0 };
							int[] yyy = { 0, 2 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					case 5 :
						{
							int[] xxx = { 1, 0 };
							int[] yyy = { 0, 1 };
							result.slice(yyy, yyy, getCov().slice(xxx, xxx));
							return result;
						}
					default :
						throw new RuntimeException("Row out of range!");
				}
			}

		}
		throw new IllegalArgumentException(
			"Illegal Relation "
				+ relationType.toString()
				+ " between "
				+ getClass().toString()
				+ " and "
				+ partnerType.toString());
	}
} // end Point_3D
