/*
 * Created on 13.03.2003
 */
package sugr;

import sugr.linalg.*;
/**
 * class containing transformation(2) constructors
 * @author Norbert Fischer
 */
public class Translation_2D extends Motion_2D
{
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * standard constructor. creates an identity matrix
	 */
	public Translation_2D()	{}

	/**
	 * Creates a translation matrix with single parameters 
	 * @param x   moving along the x-Axis
	 * @param y   moving along the y-Axis
	 */
	public Translation_2D(double x, double y)
	{
		this(x,y, LinearAlgebraFactory. getMatrix(DIM*DIM, DIM*DIM));		
	}
	
	/**
	 * Creates a translation matrix with covariance and single parameters 
	 * @param x   moving along the x-Axis
	 * @param y   moving along the y-Axis
	 * @param cov   9x9 Matrix
	 */
	public Translation_2D(double x, double y, Matrix cov)
	{
		Vector tmpV = LinearAlgebraFactory.getMatrixId(DIM).vec();
		tmpV.set(6, x);
		tmpV.set(7, y);
		tmpV.set(8, 1);
		setV(tmpV);
			
		checkCovDim(cov, DIM*DIM);
		setCov(cov);		
	}
	
	/**
	 * Creates a translation by a Point_2D
	 * @param x   an arbitrary Point_2D. Euclidean normalization will be
	 * 			 done automatically in this method
	 */
	public Translation_2D(Point_2D x)
	{
		this(x, LinearAlgebraFactory.getMatrix(DIM*DIM,DIM*DIM));
	}
	
	/**
	 * Creates a translation by a Point_2D
	 * @param x   an arbitrary Point_2D. Euclidean normalization will be
	 * 			 done automatically in this method
	 * @param cov    a 9x9 matrix
	 */
	public Translation_2D(Point_2D x, Matrix cov)
	{
		this(x.getV().get(0)/x.getV().get(2),
			 x.getV().get(1)/x.getV().get(2),
			 cov);
	}

	/**
	 * Returns the number of elements to specify this transformation
	 */
	public int getDegreesOfFreedom()
	{
		return 2;
	}	
		
}
