package sugr.example;
/*
 * Created on 13.01.2004
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */

import sugr.*;
import sugr.linalg.*;

/** Show some features of SUGR. This program constructs a virtual
 *  chair.
 * 
 * @author Daniel Beer, Institute for Photogrammetry, Bonn, Germany
 *
 */
public class MyChair {

	private Point_3D[] points = new Point_3D[12]; // Holds all Points of chair
	private Line_3D[] lines = new Line_3D[10]; // Holds all wireframe lines
	private Plane_3D[] planes = new Plane_3D[2]; // Holds seat and back plane
	private Point_2D[] points2D = new Point_2D[12];
	private Line_2D[] lines2D = new Line_2D[10];

	/** MyChair Simple Constructor
	 * 
	 * @param base Contains base point (lower left leg)
	 */
	public MyChair(Point_3D base) {
		base.normalizeEuclidean();
		calcPoints(base);
		calcLines();
		calcPlanes();
	}

	/** Compute all points relative to base point.
	 * 
	 * @param base Point_3D base point of chair
	 */
	private void calcPoints(Point_3D base) {

		Matrix cov = base.getCov(); // use Cov-Matrix for all calculated points

		points[0] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1),
				(base.getV()).get(2),
				cov);
		// lower left  point  of front leg
		points[1] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1),
				(base.getV()).get(2),
				cov);
		// lower right point  of front leg
		points[2] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1),
				(base.getV()).get(2) + 3,
				cov);
		// lower left  point  of back  leg
		points[3] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1),
				(base.getV()).get(2) + 3,
				cov);
		// lower right point  of back  leg
		points[4] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1) + 4,
				(base.getV()).get(2),
				cov);
		// front left  corner of seat  plane
		points[5] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1) + 4,
				(base.getV()).get(2),
				cov);
		// front right corner of seat  plane
		points[6] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1) + 4,
				(base.getV()).get(2) + 3,
				cov);
		// rear  left  corner of seat  plane
		points[7] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1) + 4,
				(base.getV()).get(2) + 3,
				cov);
		// rear  right corner of seat  plane
		points[8] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1) + 5.5,
				(base.getV()).get(2) + 3,
				cov);
		// lower left  point  of back  plane
		points[9] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1) + 5.5,
				(base.getV()).get(2) + 3,
				cov);
		// lower right point  of back  plane
		points[10] =
			new Point_3D(
				(base.getV()).get(0),
				(base.getV()).get(1) + 7,
				(base.getV()).get(2) + 3,
				cov);
		// upper left  point  of back  plane
		points[11] =
			new Point_3D(
				(base.getV()).get(0) + 3,
				(base.getV()).get(1) + 7,
				(base.getV()).get(2) + 3,
				cov);
	}

	/** Calculate all wireframe lines of the chair. */
	private void calcLines() {
		lines[0] = new Line_3D(points[0], points[4]); // front left  leg
		lines[1] = new Line_3D(points[1], points[5]); // front right leg
		lines[2] = new Line_3D(points[2], points[10]); // rear  left  leg
		lines[3] = new Line_3D(points[3], points[11]); // rear  right leg

		lines[4] = new Line_3D(points[4], points[6]); // left  edge seat
		lines[5] = new Line_3D(points[4], points[5]); // front edge seat
		lines[6] = new Line_3D(points[5], points[7]); // right edge seat
		lines[7] = new Line_3D(points[6], points[7]); // rear  edge seat

		lines[8] = new Line_3D(points[8], points[9]); // lower edge back
		lines[9] = new Line_3D(points[10], points[11]); // upper edge back
	}

	/** Compute two planes from chair. */
	private void calcPlanes() {
		// Build back plane by three Points
		planes[1] = new Plane_3D(points[8], points[9], points[10]);

		// Using 2 lines to create seat plane
		planes[0] = new Plane_3D(lines[4], lines[2]);
	}

	/** Returns array of copied points.
	 * 
	 * @return Point-Array
	 */
	public Point_3D[] getPoints() {
		Point_3D ret[] = new Point_3D[points.length];
		for (int i = 0; i < points.length; i++) {
			ret[i] = (Point_3D) points[i].clone();
		}
		return ret;
	}

	/** Returns copy of array of 2D points.
	 *  @return Point-Array
	 */
	public Point_2D[] getPoints2D() {
		Point_2D ret[] = new Point_2D[points2D.length];
		for (int i = 0; i < points2D.length; i++) {
			ret[i] = (Point_2D) points2D[i].clone();
		}
		return ret;
	}

	/** Returns array of copied lines .
	 * 
	 * @return Line-Array
	 */
	public Line_3D[] getLines() {
		Line_3D ret[] = new Line_3D[lines.length];
		for (int i = 0; i < lines.length; i++) {
			ret[i] = (Line_3D) lines[i].clone();
		}
		return ret;
	}

	/** Returns array of copied 2D-lines .
	 * 
	 * @return Line-Array
	 */
	public Line_2D[] getLines2D() {
		Line_2D ret[] = new Line_2D[lines2D.length];
		for (int i = 0; i < lines2D.length; i++) {
			ret[i] = (Line_2D) lines2D[i].clone();
		}
		return ret;
	}

	/** Returns array of copied planes 
	 * 
	 * @return Plane-Array
	 */
	public Plane_3D[] getPlanes() {
		Plane_3D ret[] = new Plane_3D[planes.length];
		for (int i = 0; i < planes.length; i++) {
			ret[i] = (Plane_3D) planes[i].clone();
		}
		return ret;
	}

	/** Applies Homography H to all elements (points, lines, planes)
	 *
	 * @param H contains Homography
	 */
	public void doHomography(Homography_3D H) {
		for (int i = 0; i < points.length; i++) { //transform points
			points[i] = (Point_3D) H.transform(points[i]);
		}
		for (int i = 0; i < lines.length; i++) { //transform lines
			lines[i] = (Line_3D) H.transform(lines[i]);
		}

		for (int i = 0; i < planes.length; i++) { //transform planes
			planes[i] = (Plane_3D) H.transform(planes[i]);
		}
	}

	/** Project all Points and lines according to given projective camera P.
	 * 
	 * @param P the ProjectiveCamera
	 */
	public void useProjectiveCamera(ProjectiveCamera P) {
		for (int i = 0; i < points.length; i++) {
			points2D[i] = P.transform(points[i]);
		}
		for (int i = 0; i < lines.length; i++) {
			lines2D[i] = P.transform(lines[i]);
		}

	}

	/**
	 * Create Test-instance of class MyChair
	 */
	public static void main(String[] args) {

		double sigmaobs = 0.01;

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D p = new Point_3D(10, 20, 30, cov); //Create base point

		System.out.println("\n CREATING A SUGR CHAIR\n");
		MyChair mChair = new MyChair(p);

		//		Test Homography
		System.out.println(
			"\n------------------------------------------------");
		System.out.println("\nDoing a 3D Homography:\n");

		//params for Homography
		Translation_3D trans = new Translation_3D(5, 6, 7);
		Rotation_3D rot = new Rotation_3D(10, 45, 60);
		Vector scale = LinearAlgebraFactory.getVector(1, 1, 1);
		Vector shear = LinearAlgebraFactory.getVector(6);
		Vector proj = LinearAlgebraFactory.getVector(1, 1, 1);

		Homography_3D target =
			new Homography_3D(trans, rot, scale, shear, proj);

		//declare arrays to store original and transformed elements
		Point_3D[] pts_pre, pts_post;
		Line_3D[] lin_pre, lin_post;
		Plane_3D[] pla_pre, pla_post;

		pts_pre = mChair.getPoints();
		lin_pre = mChair.getLines();
		pla_pre = mChair.getPlanes();

		mChair.doHomography(target); // apply Homography		

		// Save transformed elements
		pts_post = mChair.getPoints();
		lin_post = mChair.getLines();
		pla_post = mChair.getPlanes();

		System.out.println(
			"\n------------------------------------------------");
		System.out.println(" Transformation of Points \n");
		for (int i = 0; i < pts_post.length; i++) {
			pts_pre[i].normalizeEuclidean();
			pts_post[i].normalizeEuclidean();
			System.out.println("Point " + i + " " + pts_pre[i].toString());
			System.out.println(
				"transforms to Point " + i + " " + pts_post[i].toString());
		}

		// Compare lines
		System.out.println(
			"\n------------------------------------------------");
		System.out.println(" Transformed lines\n ");

		for (int i = 0; i < lin_post.length; i++) {
			lin_pre[i].normalizeEuclidean();
			lin_post[i].normalizeEuclidean();
		
			System.out.println("Line " + i + " " + lin_pre[i].toString());
			System.out.println(
				"transforms to Line " + i + " " + lin_post[i].toString());
		}

		// Transformed planes
		System.out.println(
			"\n------------------------------------------------");
		System.out.println(" Transformation of Planes \n");
		for (int i = 0; i < pla_post.length; i++) {
			pla_pre[i].normalizeEuclidean();
			pla_post[i].normalizeEuclidean();
			System.out.println("Plane " + i + " " + pla_pre[i].toString());
			System.out.println(
				"transforms to Plane " + i + " " + pla_post[i].toString());
		}

		//Test ProjectiveCamera
		System.out.println(
			"\n------------------------------------------------");
		System.out.println("Do a Projection with a projective camera:\n");
		
		// Interior orientation with c=1000.0, other parameters=0
		double[][] inner = { { 1000.0, 0.0, 0.0 }, {
				0.0, 1000.0, 0.0 }, {
				0.0, 0.0, 1 }
		};

		Matrix inOrien = LinearAlgebraFactory.getMatrix(inner);
		Translation_3D cTrans = new Translation_3D(20, 15, -10);

		Rotation_3D cRot = new Rotation_3D(10, 20, 30);
		Motion_3D extOrien = new Motion_3D(cTrans, cRot);

		ProjectiveCamera pCam = new ProjectiveCamera(inOrien, extOrien);

		mChair.useProjectiveCamera(pCam);

		Point_2D[] pts_pcam;
		Line_2D[] lin_pcam;

		pts_pcam = mChair.getPoints2D();
		lin_pcam = mChair.getLines2D();

		for (int i = 0; i < pts_pcam.length; i++) {
			pts_pcam[i].normalizeEuclidean();
			System.out.println("2D Point " + i + ": " + pts_pcam[i].toString());
		}

		for (int i = 0; i < lin_pcam.length; i++) {
			lin_pcam[i].normalizeEuclidean();
			System.out.println("2D Line " + i + ": " + lin_pcam[i].toString());
		}

	}
}
