package sugr.example;

import sugr.*;
import sugr.linalg.*;

/** A very simple example for using SUGR for construction and
 *  statistical test.
 * 
 * @author Thomas Laebe, Institute for Photogrammetry, Bonn, Germany
 *
 */
public class SugrSimpleExample {

	/** Simple main: Call function for construction and test.*/
	public static void main(String[] args) {
		simpleConstruction();
		simpleTest();
	}

	/** Construct a Line2D out of two points. */	
	public static void simpleConstruction() {
		System.out.println("Simple construction");
		System.out.println("---------------------");

		// Generate a covariance matrix for points
		double sigma=0.1;
		Matrix cov=LinearAlgebraFactory.getDiagMatrix(3, sigma*sigma);
		// Set homogen accuracy to 0
		cov.set(2,2, 0);
		
		// Generate two points
		Point_2D p1=new Point_2D(2, 2, 1, cov);
		Point_2D p2=new Point_2D(3, 3, 1, cov);
		
		System.out.println("Two points generated: "+p1+"\n"+p2);
		
		// And now construct line
		Line_2D l=new Line_2D(p1,p2);
		
		System.out.println("Constructed line: "+l);
	}
	
	/** Do a simple Test: Two points equal ? */
	public static void simpleTest() {
		System.out.println("Simple statistic test");
		System.out.println("---------------------");
		
		// Generate a covariance matrix for points
		double sigma=0.1;
		Matrix cov=LinearAlgebraFactory.getDiagMatrix(3, sigma*sigma);
		// Set homogen accuracy to 0
		cov.set(2,2, 0);
		
		// Generate two points
		Point_2D p1=new Point_2D(2.0, 2.0, 1, cov);
		Point_2D p2=new Point_2D(2.0, 2.1, 1, cov);
		
		System.out.println("\nTwo points generated: "+p1+"\n"+p2);
		
		// Now test if the are equal
		Relation r=new Relation(p1,p2, Equal.class);
		
		System.out.println("Used alpha          : "+r.getAlpha());
		System.out.println("Decision of the test: "+r.decision());
		System.out.println("Testvalue           : "+r.testValue());
		System.out.println("Test threshold      : "+r.testThreshold());
	}
	
	
}
