/** Java interface "Matrix.java" generated from Poseidon for UML.
 *  Poseidon for UML is developed by <A HREF="http://www.gentleware.com">Gentleware</A>.
 *  Generated with <A HREF="http://jakarta.apache.org/velocity/">velocity</A> template engine.
 */
package sugr.linalg;

import java.io.Serializable;

/**
 * <p>
 * Interface to Matrix-methods. 
 * 
 * @author Daniel Goldbach
 * @author Norbert Fischer
 * @version 1.0.1 (16.2.2003)
 * 		- Added: comments to svd()
 * 		- Added: comments to print(int, int)
 * @version 
 * 	1.0.2 (2.1.2003) 
 * 		- Added: kronecker-product 
 *  1.0.3 (3.4.2003)
 * 		- Added: slice(i0,i1,j0,j1, Matrix B)
 *  1.0.4 (13.4.2003)
 * 		- Added: all missing comments 
 * </p>
 */
public interface Matrix extends Cloneable,Serializable {

	/**
	 * <p>
	 * C=A+B
	 * </p><p>
	 * @return A+B
	 * </p><p>
	 * @param B   Must have same dimensions like this matrix
	 * </p>
	 */
	public Matrix plus(Matrix B);

	/**
	 * <p>
	 * C=A-B
	 * </p><p>
	 * @return A-B
	 * </p><p>
	 * @param B   Must have same dimensions like this matrix
	 * </p>
	 */
	public Matrix minus(Matrix B);

	/**
	 * <p>
	 * C=A*B
	 * </p><p>
	 * @return A*B
	 * </p><p>
	 * @param B   rowDim of B must be the same like colDim from A
	 * </p>
	 */
	public Matrix mult(Matrix B);

	/**
	 * <p>
	 * Matrix transpose
	 * </p><p>
	 * @return A'
	 * </p>
	 */
	public Matrix trans();

	/**
	 * <p>
	 * Get a single element
	 * </p><p>
	 * @return the element as double
	 * </p><p>
	 * @param row   row-index
	 * </p><p>
	 * @param col   column-index
	 * </p>
	 */
	public double get(int row, int col);

	/**
	 * <p>
	 * Set a single element
	 * </p><p>
	 * @param row   row-index
	 * </p><p>
	 * @param col   column-index
	 * </p><p>
	 * @param value   A(i,j):=value
	 * </p><p>
	 */
	public void set(int row, int col, double value);

	/**
	 * <p>
	 * Like the name says :-)
	 * </p><p>
	 * 
	 * @return a int containing the rowDim
	 * </p>
	 */
	public int getRowDimension();

	/**
	 * <p>
	 * Like the name says :-)
	 * </p><p>
	 * 
	 * @return a int containing the columnDim
	 * </p>
	 */
	public int getColumnDimension();

	/**
	 * <p>
	 * Matrix inverse or pseudoinverse
	 * </p><p>
	 * @return inverse(A) if A is square, pseudoinverse otherwise.
	 * </p>
	 */
	public Matrix invert();

	/**
	 * <p>
	 * B=A*f , where f is a scalar
	 * </p><p>
	 * @return A*f
	 * </p><p>
	 * @param f   scalar, given as double
	 * </p>
	 */
	public Matrix mult(double f);

	/**
	 * <p>
	 * B=A*v, where v is a vector. Dimension of v and column dimension of A must agree
	 * </p><p>
	 * @return A*v
	 * </p><p>
	 * @param v   Vector, with dimension the same as column-dimension of A
	 * </p>
	 */
	public Vector mult(Vector v);

	/**
	 * <p>
	 * Computes the SingularValueDecomposition of given Matrix with dimension m x n 
	 * (m>=n), so the result is given by: Matrix = U*S*V^t
	 * </p><p>
	 * 
	 * @return a Matrix[] containing 3 elements: 
	 * <ul>
	 * 		<li>	Matrix[0] = U (orthogonal, m x n) </li>
	 * 		<li>	Matrix[1] = S (the diagonalmatrix, n x n, containing the singularvalues)</li>
	 * 		<li>	Matrix[2] = V (orthogonal, n x n) <br>
	 * 						(note, that the result is NOT transposed) </li>
	 * </ul>
	 * e.g.: <pre> orig_matrix = Matrix[0].mult(Matrix[1]).mult(Matrix[2].trans()); </pre>
	 * 				gives the original matrix
	 * </p>
	 */
	public Matrix[] svd();

	/**
	 * <p>
	 * Get submatrix with given list of rows and given list of columns. If listofRows
	 * is emtpy, get complete column, if listOfCols is empty, get complete rows.
	 * </p><p>
	 * @return a Matrix with ...
	 * </p><p>
	 * @param listOfRows ...
	 * </p><p>
	 * @param listOfCols ...
	 * </p>
	 */
	public Matrix slice(int[] listOfRows, int[] listOfCols);

	/**
	 * <p>
	 * As other slice, only: read values from Matrix B and put them
	 * in the object at the posisitions of the intersections of
	 * given rows and columns.
	 * </p><p>
	 * @param listOfRows ...
	 * </p><p>
	 * @param listOfCols ...
	 * </p><p>
	 * @param B ...
	 * </p><p>
	 * 
	 * </p>
	 */
	public void slice(int[] listOfRows, int[] listOfCols, Matrix B);

	/**
	 * Sets a submatrix of A with B
	 * @param i0   initial row index (upper left is (0,0) )
	 * @param i1   final row index
	 * @param j0   initial column index
	 * @param j1   final column index
	 * @param B    Matrix with dimension (i1-i0+1 x j1-j0+1)
	 */
	public void slice(int i0, int i1, int j0, int j1, Matrix B);

	/**
	 * Gets a submatrix of A
	 * @param i0   initial row index (upper left is (0,0) )
	 * @param i1   final row index
	 * @param j0   initial column index
	 * @param j1   final column index
	 */
	public Matrix slice(int i0, int i1, int j0, int j1);

	/**
	 * returns the matrix as a vector: First the first column, then the second
	 * column and so on.
	 * <pre>
	 * e.g. : if A = | a b |  then A.vec() returns | a |
	 *               | c d |                       | c |
	 *                                             | b |
	 *                                             | d |
	 * </pre>
	 * @return a Vector with dimension m*n
	 */
	public Vector vec();

	/**
	 * Create a new matrix B, independent from A
	 * 
	 * @return B with B=A 
	 */
	public Matrix copy();

	/**
	 * <p>
	 * Sets all elements of the matrix to the given value.
	 * </p><p>
	 * @param value
	 * </p>
	 */
	public void set(double value);

	/**
	 * <p>
	 * Prints this matrix to stdout <br>
	 * It may be useful to select w > d. A common value is w=5, d=3
	 * </p>
	 * @param w   describes the width of a single element
	 * @param d   number of decimals after comma
	 */
	public void print(int w, int d);

	/**
	 * <p>
	 * Prints this matrix to stdout <br>
	 * </p>
	 */
	public void print();

	/**
	 * <p>
	 * Prints this matrix in Matlab format to stdout <br>
	 * </p>
	 */
	public void printMatlab();
	
	/** Delivers a string with the elements of the matrix.
	 * @return a string for printing
	 */
	public String toString();

	/**
	 * Test, whether A and B are identical or not
	 * @param obj   Matrix to test
	 * @return boolean value   true, if A=B or false otherwise
	 */
	//overrides equals from Object. Makes it easier to test
	public boolean equals(Object obj);

	/**
	 * <p>
	 * Returns a Submatrix with dimension (i1-i0) x (j1-j0)
	 * Upper left element begin with indices (0,0) 
	 * </p>
	 * <p>
	 * @param i0   Initial row index
	 * @param i1   Final row index
	 * @param j0   Initial column index
	 * @param j1   Final column index 
	 * </p>
	 */
	public Matrix getMatrix(int i0, int i1, int j0, int j1);

	/**
	 * <p>
	 * EigenvalueDecomposition of this Matrix
	 * </p>
	 * <p>
	 * @return a double-Array containing the eigenvalues. They are sorted in
	 * 			so that [0] is the biggest and [n] is the smallest value
	 * 			 
	 * </p>
	 */
	public double[] eig();

	/**
	 * Test, whether A ist semidefinit or not
	 * @return boolean value: true, if all eigenvalues >= 0
	 */
	public boolean isSemiDefinite();

	/**
	 * <p>
	 * returns the pseudo-inverse A^+ . This is done via SVD with a given rank
	 * </p><p>
	 * 
	 * @param rank   the rank which A with the given elements should have
	 * </p><p>
	 * @return A^+
	 * </p>
	 */
	public Matrix pseudoInverse(int rank);

	/**
	 * <p>
	 * returns the pseudo-inverse A^+ with given Nullspace
	 * </p><p>
	 * 
	 * @param H   nullspace
	 * </p><p>
	 * @return A^+
	 * </p>
	 */
	public Matrix pseudoInverse(Matrix H);
	
	/**
	 * Calculates the kronecker-product of two matrices.
	 * Let A=(a_ij) be a (n x m)-Matrix and B=(a_ij) a (p x q)-Matrix. Then the 
	 * result-matrix C is a (mp x nq)-Matrix with
	 * <pre>
	 *                  |a_11*B ...  a_1n*B |
	 *              C = |...            ... |
	 *                  |a_m1*B ...  a_mn*B |
	 * </pre>
	 * @param B  an arbitrary-Matrix
	 * @return a Matrix with dimension (mp x nq)
	 */
	public Matrix kronecker(Matrix B);

	/** 
	 * Computes the determinant of the matrix
	 * @return  determinant of the matrix 
	 */
	public double det();

	public Vector getRow(int i);

	public Vector getColumn(int j);

	public void setRow(int i, Vector v);

	public void setColumn(int j, Vector v);

	public Vector solve(Vector y);

	/**
	 * delete a column from A. <br>
	 * <pre>
	 * e.g. : if A looks like
	 *        | a11 a12 a13 |
	 *        | a21 a22 a23 |    3x3 Matrix
	 *        | a31 a32 a33 |
	 * 
	 * then A.delColumn(0) returns
	 *            | a12 a13 |
	 *            | a22 a23 |    3x2 Matrix
	 *            | a32 a33 |
	 * </pre>
	 * @param col		column to delete. Must be between 0 and m-1, if A is nxm
	 * @return	nx(m-1) Matrix
	 */
	public Matrix delColumn(int col);

	/**
	 * delete a row from A. <br>
	 * <pre>
	 * e.g. : if A looks like
	 *        | a11 a12 a13 |
	 *        | a21 a22 a23 |    3x3 Matrix
	 *        | a31 a32 a33 |
	 * 
	 * then A.delRow(0) returns
	 * 
	 *        | a21 a22 a23 |    2x3 Matrix
	 *        | a31 a32 a33 |
	 * </pre>
	 * @param row		row to delete. Must be between 0 and n-1, if A is nxm
	 * @return	(n-1)xm Matrix
	 */
	public Matrix delRow(int row);

	/**
	 * <p>
	 * Computes the QR Decomposition of given Matrix
	 * </p><p>
	 * 
	 * @return a Matrix[] containing 3 elements: 
	 * <ul>
	 * 		<li>	Matrix[0] = Q </li>
	 * 		<li>	Matrix[1] = R</br>
	 * </p>
	 */
	public Matrix[] qr();
	
	/** Clone a matrix.
	 * @return a copy of the matrix.
	 */
	public Object clone();
	
	 
} // end Matrix
