/*
 * Created on 14.04.2005
 *
 */
package sugr.linalg;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import cern.colt.matrix.linalg.EigenvalueDecomposition;
import cern.colt.matrix.linalg.QRDecomposition;
import cern.colt.matrix.linalg.SingularValueDecomposition;
import cern.jet.math.Functions;

/**
 * @author beder
 *
 */
public class MatrixImplColt implements Matrix {
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;

	protected DoubleMatrix2D impl;
	
	
	public MatrixImplColt (int no_rows, int no_cols) {
		impl = new DenseDoubleMatrix2D(no_rows, no_cols);
	}

	public MatrixImplColt (double[][] values) {
		impl = new DenseDoubleMatrix2D(values);
	}
	
	public MatrixImplColt (Matrix m) {
		impl = ((MatrixImplColt)m).impl.copy();
	}

	protected MatrixImplColt() {		
	}
	
	
	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#plus(sugr.linalg.Matrix)
	 */
	public Matrix plus(Matrix B) {
		MatrixImplColt result = new MatrixImplColt(this); 
		result.impl.assign(((MatrixImplColt)B).impl, Functions.plus);		
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#minus(sugr.linalg.Matrix)
	 */
	public Matrix minus(Matrix B) {
		MatrixImplColt result = new MatrixImplColt(this); 
		result.impl.assign(((MatrixImplColt)B).impl, Functions.minus);		
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#mult(sugr.linalg.Matrix)
	 */
	public Matrix mult(Matrix B) {
		MatrixImplColt result = new MatrixImplColt(); 
		result.impl = impl.zMult(((MatrixImplColt)B).impl, null); 
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#trans()
	 */
	public Matrix trans() {
		MatrixImplColt result = new MatrixImplColt(); 
		result.impl = impl.viewDice().copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#get(int, int)
	 */
	public double get(int row, int col) {
		return impl.get(row, col);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#set(int, int, double)
	 */
	public void set(int row, int col, double value) {
		impl.set(row,col,value);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#getRowDimension()
	 */
	public int getRowDimension() {
		return impl.rows();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#getColumnDimension()
	 */
	public int getColumnDimension() {
		return impl.columns();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#invert()
	 */
	public Matrix invert() {
		MatrixImplColt result = new MatrixImplColt();		
		result.impl = Algebra.DEFAULT.inverse(impl);
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#mult(double)
	 */
	public Matrix mult(double f) {
		MatrixImplColt result = new MatrixImplColt(this); 
		result.impl.assign(Functions.mult(f));
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#mult(sugr.linalg.Vector)
	 */
	public Vector mult(Vector v) {
		VectorImplColt result = new VectorImplColt(); 
		result.impl = impl.zMult(((VectorImplColt)v).impl, null); 
		return result;		
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#svd()
	 */
	public Matrix[] svd() {
		// Since Colt hangs, if any value is NaN of Infinite, this is check first...
		for (int i = 0; i < getRowDimension(); i++)
			for (int j = 0; j < getColumnDimension(); j++) {
				double d = impl.get(i, j);
				if (Double.isNaN(d))
					throw new RuntimeException("Matrix contains NaN element!");
				else if (Double.isInfinite(d))
					throw new RuntimeException("Matrix contains infinite element!");

			}

		MatrixImplColt[] UDV = new MatrixImplColt[3];
		UDV[0] = new MatrixImplColt();
		UDV[1] = new MatrixImplColt();
		UDV[2] = new MatrixImplColt();
		SingularValueDecomposition svd=null;
		if (impl.rows()<impl.columns()) {
			DenseDoubleMatrix2D filled = new DenseDoubleMatrix2D(impl.columns(),impl.columns());
			filled.viewPart(0,0,impl.rows(),impl.columns()).assign(impl);
			svd = new SingularValueDecomposition(filled);
		}
		else
			svd = new SingularValueDecomposition(impl);
		UDV[0].impl = svd.getU();
		UDV[1].impl = svd.getS();
		UDV[2].impl = svd.getV();
		return UDV;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#slice(int[], int[])
	 */
	public Matrix slice(int[] listOfRows, int[] listOfCols) {
		MatrixImplColt result = new MatrixImplColt(); 
		result.impl = impl.viewSelection(listOfRows, listOfCols).copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#slice(int[], int[], sugr.linalg.Matrix)
	 */
	public void slice(int[] listOfRows, int[] listOfCols, Matrix B) {
		impl.viewSelection(listOfRows, listOfCols).assign(((MatrixImplColt)B).impl);	
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#slice(int, int, int, int, sugr.linalg.Matrix)
	 */
	public void slice(int i0, int i1, int j0, int j1, Matrix B) {
		impl.viewPart(i0,j0,i1-i0+1,j1-j0+1).assign(((MatrixImplColt)B).impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#slice(int, int, int, int)
	 */
	public Matrix slice(int i0, int i1, int j0, int j1) {
		MatrixImplColt result = new MatrixImplColt(); 
		result.impl = impl.viewPart(i0,j0,i1-i0+1,j1-j0+1).copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#vec()
	 */
	public Vector vec() {
		Vector v =
			LinearAlgebraFactoryColt.getVector(impl.rows() * impl.columns());
		int i = 0;
		for (int c = 0; c < impl.columns(); c++) {
			for (int r = 0; r < impl.rows(); r++) {
				((VectorImplColt)v).set(i, impl.get(r, c));
				i++;
			}
		}
		return v;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#copy()
	 */
	public Matrix copy() {
		MatrixImplColt result = new MatrixImplColt();
		result.impl = impl.copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#set(double)
	 */
	public void set(double value) {
		impl.assign(value);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#print(int, int)
	 */
	public void print(int w, int d) {
		DecimalFormat format = new DecimalFormat();
	    format.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.US));
	    format.setMinimumIntegerDigits(1);
	    format.setMaximumFractionDigits(d);
	    format.setMinimumFractionDigits(d);
	    format.setGroupingUsed(false);
	    
	    int width = w+2;
	    System.out.println();	//start on new line, idea copied from jama
	    for (int i = 0; i < getRowDimension(); i++) {
	    	for(int j = 0; j < getColumnDimension(); j++)
	    	{
	    		String s = format.format(get(i,j)); // format the number
	    		int padding = Math.max(1,width-s.length()); // At _least_ 1 space
	    		for (int k = 0; k < padding; k++)
	    			System.out.print(' ');
	    		System.out.print(s);
	    	}
	    	System.out.println();
	    }
	    System.out.println();   // end with blank line.
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#print()
	 */
	public void print() {
		System.out.println();
		for (int i=0; i<impl.rows(); i++) {
			for (int j=0; j<impl.columns(); j++) {
				System.out.print(impl.get(i,j));
				System.out.print ("  ");
			}
			System.out.println();
		}
		System.out.println();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#printMatlab()
	 */
	public void printMatlab() {
		System.out.print ("[");
		for (int i=0; i<impl.rows(); i++) {
			for (int j=0; j<impl.columns(); j++) {
				System.out.print(impl.get(i,j));
				if (j!=impl.columns()-1)
					System.out.print (",");
			}
			if (i!=impl.rows()-1)
				System.out.print(";");
		}
		System.out.println("]");
	}
	
	/** Delivers a string with the elements of the matrix.
	 * @return a string for printing
	 */
	public String toString() {
		StringWriter sw= new StringWriter();
		PrintWriter pr= new PrintWriter(sw);
		
		pr.println();
		for (int i=0; i<impl.rows(); i++) {
			for (int j=0; j<impl.columns(); j++) {
				pr.print(impl.get(i,j));
				pr.print ("  ");
			}
			pr.println();
		}
		pr.println();
		
		
		return sw.toString();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#getMatrix(int, int, int, int)
	 */
	public Matrix getMatrix(int i0, int i1, int j0, int j1) {
		return slice(i0,i1,j0,j1);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#eig()
	 */
	public double[] eig() {
		EigenvalueDecomposition evd = new EigenvalueDecomposition(impl);
		DoubleMatrix2D D = evd.getD();
		int noOfEigenvalues = D.rows();
		double[] eigenvalues = new double[noOfEigenvalues];
		for (int i = 0; i < noOfEigenvalues; i++) {
			eigenvalues[i] = D.get(i, i);
		}
		return eigenvalues;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#isSemiDefinite()
	 */
	public boolean isSemiDefinite() {
		double[] eigenvalues = this.eig();
		for (int i = 0; i < eigenvalues.length; i++) {
			if (eigenvalues[i] < 0)
				return false;
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#pseudoInverse(int)
	 */
	public Matrix pseudoInverse(int rank) {
		Matrix[] UDV = this.svd();
		//invert with rank-constraint
		for (int i = 0; i < rank; i++) {
			UDV[1].set(i, i, 1.0 / UDV[1].get(i, i));
		}
		//set the rest to 0
		for (int i = rank; i < UDV[1].getRowDimension(); i++) {
			UDV[1].set(i, i, 0);
			//getRowDim.. is allowed because sigma_dd is square
		}
		return UDV[2].mult(UDV[1]).mult(UDV[0].trans());
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#kronecker(sugr.linalg.Matrix)
	 */
	public Matrix kronecker(Matrix B) {
		//saves write-time :-)	See above for choosing letters
		int n = this.getRowDimension();
		int m = this.getColumnDimension();
		int p = B.getRowDimension();
		int q = B.getColumnDimension();
		Matrix result = LinearAlgebraFactoryColt.getMatrix(n * p, m * q);
		Matrix tmp;
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < m; j++) {
				//create new matrix for filling. E.g. a_11*B
				tmp = B.mult(this.get(i, j));
				//now copy this tmp matrix into result matrix
				for (int k = 0; k < p; k++) {
					for (int l = 0; l < q; l++) {
						//Please note that i,j show where to insert into result matrix
						result.set(i * p +k, j *q +l, tmp.get(k, l));
					}
				}
			}
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#det()
	 */
	public double det() {
		return Algebra.DEFAULT.det(impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#getRow(int)
	 */
	public Vector getRow(int i) {
		VectorImplColt result = new VectorImplColt(); 
		result.impl = impl.viewRow(i).copy();
		return result;	
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#getColumn(int)
	 */
	public Vector getColumn(int j) {
		VectorImplColt result = new VectorImplColt(); 
		result.impl = impl.viewColumn(j).copy();
		return result;	
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#setRow(int, sugr.linalg.Vector)
	 */
	public void setRow(int i, Vector v) {
		impl.viewRow(i).assign(((VectorImplColt)v).impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#setColumn(int, sugr.linalg.Vector)
	 */
	public void setColumn(int j, Vector v) {
		impl.viewColumn(j).assign(((VectorImplColt)v).impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#solve(sugr.linalg.Vector)
	 */
	public Vector solve(Vector y) {
		VectorImplColt result = new VectorImplColt();
		result.impl = Algebra.DEFAULT.solve(impl, ((MatrixImplColt)y.getMatrix()).impl).viewColumn(0);
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#delColumn(int)
	 */
	public Matrix delColumn(int col) {
		int[] cols = new int[getColumnDimension()-1]; 
		int[] rows = new int[getRowDimension()];
		for (int i=0,j=0; j<cols.length; i++,j++) {
			if (i==col)
				i++;
			cols[j] = i;
		}
		for (int i=0; i<rows.length; i++)
			rows[i] = i;
		return slice(rows, cols);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#delRow(int)
	 */
	public Matrix delRow(int row) {
		int[] cols = new int[getColumnDimension()]; 
		int[] rows = new int[getRowDimension()-1];
		for (int i=0,j=0; j<rows.length; i++,j++) {
			if (i==row)
				i++;
			rows[j] = i;
		}
		for (int i=0; i<cols.length; i++)
			cols[i] = i;
		return slice(rows, cols);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Matrix#qr()
	 */
	public Matrix[] qr() {
		MatrixImplColt[] UDV = new MatrixImplColt[2];
		UDV[0] = new MatrixImplColt();
		UDV[1] = new MatrixImplColt();
		QRDecomposition svd = new QRDecomposition(impl);
		UDV[0].impl = svd.getQ();
		UDV[1].impl = svd.getR();
		return UDV;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		try {
			MatrixImplColt m = (MatrixImplColt) super.clone();
			m.impl = impl.copy();
			return m; // return the clone
		} catch (CloneNotSupportedException e) {
			// this shouldn't happen 
			throw new InternalError();
		}
	}
	
	public boolean equals(Object obj) {
		if (obj == null || !obj.getClass().equals(this.getClass())) {
			return false;
		}

		MatrixImplColt m = (MatrixImplColt) obj;
		if (impl.rows() != m.getRowDimension())
			return false;
		if (impl.columns() != m.getColumnDimension())
			return false;

		for (int i = 0; i < m.getRowDimension(); i++) {
			for (int j = 0; j < m.getColumnDimension(); j++) {
				if (this.get(i, j) != m.get(i, j))
					return false;
			}
		}

		return true;
	}
	
	public Matrix pseudoInverse(Matrix H) {
		Matrix A = LinearAlgebraFactory.getMatrix(getRowDimension()+H.getRowDimension(), getColumnDimension()+H.getRowDimension());
		A.slice(0,getRowDimension()-1,0,getRowDimension()-1,this);
		A.slice(getRowDimension(),getRowDimension()+H.getRowDimension()-1, 0, getColumnDimension()-1, H);
		A.slice(0,getRowDimension()-1,getColumnDimension(), getColumnDimension()+H.getRowDimension()-1, H.trans());
		A = A.invert();
		return A.slice(0,getRowDimension()-1,0,getColumnDimension()-1);
	}

}
