/*
 * Created on 14.04.2005
 *
 */
package sugr.linalg;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Locale;

import cern.colt.matrix.DoubleMatrix1D;
import cern.colt.matrix.impl.DenseDoubleMatrix1D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import cern.jet.math.Functions;

/**
 * @author beder
 *
 */
public class VectorImplColt implements Vector {	
	/**
	 * Class version number for serialization
	 */
	private static final long serialVersionUID = 1L;
	
	protected DoubleMatrix1D impl;
	
	public VectorImplColt (int size) {
		impl = new DenseDoubleMatrix1D(size);
	}

	public VectorImplColt (double[] values) {		
		impl = new DenseDoubleMatrix1D(values);
	}
	
	public VectorImplColt (Vector v) {
		impl = ((VectorImplColt)v).impl.copy();
	}
	
	protected VectorImplColt() {		
	}
	
	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#plus(sugr.linalg.Vector)
	 */
	public Vector plus(Vector v) {
		VectorImplColt result = new VectorImplColt(this); 
		result.impl.assign(((VectorImplColt)v).impl, Functions.plus);		
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#minus(sugr.linalg.Vector)
	 */
	public Vector minus(Vector v) {
		VectorImplColt result = new VectorImplColt(this); 
		result.impl.assign(((VectorImplColt)v).impl, Functions.minus);		
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#scalarmult(sugr.linalg.Vector)
	 */
	public double scalarmult(Vector v) {
		return impl.zDotProduct(((VectorImplColt)v).impl); 
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#crossProduct(sugr.linalg.Vector)
	 */
	public Vector crossProduct(Vector v) {
		if ((this.getDim() != 3) || (v.getDim() != 3))
			throw new RuntimeException("Only dimension of thre is supported!");
		VectorImplColt result = new VectorImplColt (3);
		result.set(0, this.get(1) * v.get(2) - this.get(2) * v.get(1));
		result.set(1, this.get(2) * v.get(0) - this.get(0) * v.get(2));
		result.set(2, this.get(0) * v.get(1) - this.get(1) * v.get(0));
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#mult(double)
	 */
	public Vector mult(double f) {
		VectorImplColt result = new VectorImplColt(this); 
		result.impl.assign(Functions.mult(f));
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#copy()
	 */
	public Vector copy() {
		VectorImplColt result = new VectorImplColt();
		result.impl = impl.copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#set(int, double)
	 */
	public void set(int pos, double value) {
		impl.set(pos, value);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#set(int, sugr.linalg.Vector)
	 */
	public void set(int pos, Vector v) {
		impl.viewPart(pos, v.getDim()).assign(((VectorImplColt)v).impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#get(int)
	 */
	public double get(int pos) {		
		return impl.get(pos);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#get(int, int)
	 */
	public Vector get(int i0, int i1) {
		VectorImplColt result = new VectorImplColt();
		result.impl = impl.viewPart(i0, i1-i0+1).copy();
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#getDim()
	 */
	public int getDim() {
		return impl.size();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#getLength()
	 */
	public double getLength() {
		return Math.sqrt(Algebra.DEFAULT.norm2(impl));
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#normF()
	 */
	public double normF() {
		return getLength();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#normM()
	 */
	public double normM() {
		return Algebra.DEFAULT.normInfinity(impl);
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#print(int, int)
	 */
	public void print(int w, int d) {
		DecimalFormat format = new DecimalFormat();
	    format.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.US));
	    format.setMinimumIntegerDigits(1);
	    format.setMaximumFractionDigits(d);
	    format.setMinimumFractionDigits(d);
	    format.setGroupingUsed(false);
	    
	    int width = w+2;
	    System.out.println();	//start on new line, idea copied from jama
	    for (int i = 0; i < getDim(); i++) {
	    	String s = format.format(get(i)); // format the number
	    	int padding = Math.max(1,width-s.length()); // At _least_ 1 space
	    	for (int k = 0; k < padding; k++)
	    		System.out.print(' ');
	    	System.out.println(s);
	    }
	    System.out.println();   // end with blank line.
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#print()
	 */
	public void print() {
		System.out.println();
		for (int i=0; i<impl.size(); i++)
			System.out.println(impl.get(i));
		System.out.println();		
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#printMatlab()
	 */
	public void printMatlab() {
		System.out.print("[");
		for (int i=0; i<impl.size(); i++) {
			System.out.print(impl.get(i));
			if (i!=impl.size()-1)
				System.out.print(",");
		}
		System.out.println("]");
	}
	
	/** Delivers a string with the elements of the matrix.
	 * @return a string for printing
	 */
	public String toString() {
		StringWriter sw= new StringWriter();
		PrintWriter pr= new PrintWriter(sw);
		
		pr.println();
		for (int i=0; i<impl.size(); i++)
			pr.println(impl.get(i));
		pr.println();		

		return sw.toString();
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#getMatrix()
	 */
	public Matrix getMatrix() {
		MatrixImplColt result=new MatrixImplColt(); 
		result.impl=new DenseDoubleMatrix2D(impl.size(),1);		
		result.impl.viewColumn(0).assign(impl);		
		return result;
	}

	/* (non-Javadoc)
	 * @see sugr.linalg.Vector#SkewSym()
	 */
	public Matrix SkewSym() {
		if (impl.size() != 3) {
			throw new IllegalArgumentException("Vector dimension must be 3!");
		}
		Matrix S = LinearAlgebraFactoryColt.getMatrix(3,3);
		double a = get(0);
		double b = get(1);
		double c = get(2);
		S.set(0, 1, -c);
		S.set(0, 2, b);
		S.set(1, 0, c);
		S.set(1, 2, -a);
		S.set(2, 0, -b);
		S.set(2, 1, a);
		return S;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone()  {
		try {
			VectorImplColt m = (VectorImplColt) super.clone();
			m.impl = impl.copy();
			return m; // return the clone
		} catch (CloneNotSupportedException e) {
			// this shouldn't happen 
			throw new InternalError();
		}
	}
}
