/*
 * Created on 18.05.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 */
public class Affinity_2DTest extends TestCase {

	/**
	 * Constructor for Affinity_2DTest.
	 * @param arg0
	 */
	public Affinity_2DTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Affinity_2D aff = new Affinity_2D();
		int dof = aff.getDegreesOfFreedom();
		if (dof!=6)fail();
	}

	/**
	 * Test for void Affinity_2D()
	 */
	public void testAffinity_2D() {
		Affinity_2D aff = new Affinity_2D();
		if (aff==null)fail();
	}

	/**
	 * Test for void Affinity_2D(double, double, double, double, double, double)
	 */
	public void testAffinity_2Ddoubledoubledoubledoubledoubledouble() {
		//construct Affinity_2D aff
		Affinity_2D aff = new Affinity_2D(1,2,3,4,5,6);
		//check constructor (transformation matrix)
		Matrix transMat=aff.getTransformationMatrix();
		double[][] doub = {{1,3,5},{2,4,6},{0,0,1}};
		Matrix checkMat = LinearAlgebraFactory.getMatrix(doub);
		Matrix zeroM = checkMat.minus(transMat);
		double zeroD = zeroM.vec().getLength();
		if (zeroD>1e-14)fail();
		//covarince should be zero-Matrix
		zeroD = aff.getCov().vec().normF();
		if (zeroD>1e-14)fail();
	}

	/**
	 * Test for void Affinity_2D(double, double, double, double, double, double, Matrix)
	 */
	public void testAffinity_2DdoubledoubledoubledoubledoubledoubleMatrix() {
		//initialize symmetric covariance matrix
		double[][] covArr=new double[9][9];
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(j>=i)covArr[i][j]=1e-2*Math.random();
				else covArr[i][j]=covArr[j][i];
			}
		}
		Matrix cov = LinearAlgebraFactory.getMatrix(covArr);
		//construct Affinity_2D aff
		Affinity_2D aff = new Affinity_2D(1,2,3,4,5,6,cov);
		//check constructor (transformation Matrix)
		double[][] doub = {{1,3,5},{2,4,6},{0,0,1}};
		Matrix checkMat = LinearAlgebraFactory.getMatrix(doub);
		Matrix transMat=aff.getTransformationMatrix();
		Matrix zeroM = checkMat.minus(transMat);
		double zeroD = zeroM.vec().getLength();
		if (zeroD>1e-14){
			System.out.println("Affinity: " + aff);
			System.out.println("Matrix: " + checkMat);
			fail();
		}
		//check constructor (covariance matrix)
		Matrix transCov = aff.getCov();
		zeroM = transCov.minus(cov);
		zeroD = zeroM.vec().getLength();
		if (zeroD>1e-12)fail();
	}

	/**
	 * Test for void Affinity_2D(Rotation_2D, Translation_2D, Vector, double, boolean)
	 * No error propagation is done for scale or shear, thus it must not be tested here.
	 */
	public void testAffinity_2DRotation_2DTranslation_2DVectordoubleboolean() {
		//unsymmetric shear
		//set up Parameters 
		Rotation_2D rot = new Rotation_2D(Math.PI/2);
		Translation_2D trans = new Translation_2D(2,4);
		Vector scale = LinearAlgebraFactory.getVector(2.5,0.25);
		double[][] scaleArr = {{2.5,0,0},{0,0.25,0},{0,0,1}};
		Matrix scaleMat = LinearAlgebraFactory.getMatrix(scaleArr);
		double shear = 5;
		double[][] shearArr = {{1,5,0},{0,1,0},{0,0,1}};
		Matrix shearMat = LinearAlgebraFactory.getMatrix(shearArr);
		//constrction with SUGR
		Affinity_2D affSugr = new Affinity_2D(rot,trans,scale,shear,false);
		Matrix affSugrMat = affSugr.getTransformationMatrix();
		//construction via hand
		Motion_2D mot = new Motion_2D(trans,rot);
		Matrix affHandMat = mot.getTransformationMatrix().mult(scaleMat).mult(shearMat);
		//compare Matrices
		double zero = affHandMat.minus(affSugrMat).vec().getLength();
		if(zero>1e-12)fail();
		//symmetric shear
		//set up shearMat
		//{{1,2.5,0},{2.5,1,0},{0,0,1}}
		shearArr[0][1] = 2.5;
		shearArr[1][0] = 2.5;
		shearMat = LinearAlgebraFactory.getMatrix(shearArr);
		//constrction with SUGR
		affSugr = new Affinity_2D(rot,trans,scale,shear,true);
		affSugrMat = affSugr.getTransformationMatrix();
		//construction via hand
		mot = new Motion_2D(trans,rot);
		affHandMat = mot.getTransformationMatrix().mult(scaleMat).mult(shearMat);
		//compare Matrices
		zero = affHandMat.minus(affSugrMat).vec().getLength();
		if(zero>1e-12)fail();
	}

	/**
	 * Test for void Affinity_2D(Rotation_2D, Translation_2D, Vector, double, boolean, Matrix)
	 */
	public void testAffinity_2DRotation_2DTranslation_2DVectordoublebooleanMatrix() {
		//initialize symmetric covariance matrix
		double[][] covArr=new double[9][9];
		for(int i=0;i<9;i++){
			for(int j=0;j<9;j++){
				if(j>=i)covArr[i][j]=1e-2*Math.random();
				else covArr[i][j]=covArr[j][i];
			}
		}
		Matrix cov = LinearAlgebraFactory.getMatrix(covArr);
		//unsymmetric shear
		//set up Parameters 
		Rotation_2D rot = new Rotation_2D(Math.PI/2);
		Translation_2D trans = new Translation_2D(2,4);
		Vector scale = LinearAlgebraFactory.getVector(2.5,0.25);
		double[][] scaleArr = {{2.5,0,0},{0,0.25,0},{0,0,1}};
		Matrix scaleMat = LinearAlgebraFactory.getMatrix(scaleArr);
		double shear = 5;
		double[][] shearArr = {{1,5,0},{0,1,0},{0,0,1}};
		Matrix shearMat = LinearAlgebraFactory.getMatrix(shearArr);
		//constrction with SUGR
		Affinity_2D affSugr = new Affinity_2D(rot,trans,scale,shear,false,cov);
		Matrix affSugrMat = affSugr.getTransformationMatrix();
		//construction via hand
		Motion_2D mot = new Motion_2D(trans,rot);
		Matrix affHandMat = mot.getTransformationMatrix().mult(scaleMat).mult(shearMat);
		//compare Matrices
		double zero = affHandMat.minus(affSugrMat).vec().getLength();
		if(zero>1e-12)fail();
		//check covariance Matrix
		Matrix zeroM = affSugr.getCov().minus(cov);
		double zeroD = zeroM.vec().normF();
		if(zeroD>1e-14)fail();
		//symmetric shear
		//set up shearMat
		//{{1,2.5,0},{2.5,1,0},{0,0,1}}
		shearArr[0][1] = 2.5;
		shearArr[1][0] = 2.5;
		shearMat = LinearAlgebraFactory.getMatrix(shearArr);
		//constrction with SUGR
		affSugr = new Affinity_2D(rot,trans,scale,shear,true,cov);
		affSugrMat = affSugr.getTransformationMatrix();
		//construction via hand
		mot = new Motion_2D(trans,rot);
		affHandMat = mot.getTransformationMatrix().mult(scaleMat).mult(shearMat);
		//compare Matrices
		zero = affHandMat.minus(affSugrMat).vec().getLength();
		if(zero>1e-12)fail();
		//check covariance Matrix
		zeroM = affSugr.getCov().minus(cov);
		zeroD = zeroM.vec().normF();
		if(zeroD>1e-14)fail();
	}

}
