/*
 * Created on 17.06.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Test for BiRelationalProperty.
 * 
 * @author Achim Knigs
 */
public class BiRelationalPropertyTest extends TestCase {

	/**
	 * Constructor for BiRelationalPropertyTest.
	 * @param arg0
	 */
	public BiRelationalPropertyTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for getFirstEntity()
	 * Using Equal, because BiRelationalProperty is abstract
	 */
	public void testGetFirstEntity() {
		//construct some entity
		Point_2D p = new Point_2D(1,Double.MAX_VALUE);
		
		//build equal, and test if you get back the right point again :)
		Equal eq = new Equal(p);
		
		if(!eq.getFirstEntity().equals(p)) fail();
	}

	/**
	 * Test for getTargetType()
	 * Using Equal and Incident, because BiRelationalProperty is abstract
	 */
	public void testGetTargetType() {
		//construct some entity for the relation
		Point_2D p = new Point_2D(Double.MIN_VALUE,Double.POSITIVE_INFINITY);
		
		//build the relations
		Equal eq = new Equal(p);
		Incident inc = new Incident(p,Line_2D.class);
		
		if(eq.getTargetType() != Point_2D.class) fail();
		if(inc.getTargetType() != Line_2D.class) fail();
	}

	/**
	 * Test for Object clone()
	 */
	public void testClone() {
		//construct some entity for the relation
		Point_2D p = new Point_2D(Double.MIN_VALUE,Double.POSITIVE_INFINITY);
		
		//build the relations
		Equal eq = new Equal(p);
		Incident inc = new Incident(p,Line_2D.class);
		
		//clone objects:
		Equal eq2 = (Equal)eq.clone();
		Incident inc2 = (Incident)inc.clone();
		
		if(eq.getFirstEntity().getV().minus(eq2.getFirstEntity().getV()).getLength() >= 0.00000001) fail();
		if(eq.getTargetType() != eq2.getTargetType()) fail();
		if((eq.getFirstEntity().getV().minus(eq2.getFirstEntity().getV()).getLength() >= 0.00000001) || 
				inc.getTargetType() != inc2.getTargetType()) fail();
	}

	/**
	 * Test for getObservations
	 */
	public void testGetObservations() {
		//construct some entity
		Point_2D p = new Point_2D(1,Double.MAX_VALUE);
		
		//build equal, and test if you get back the right point again :)
		Equal eq = new Equal(p);
		
		//this gets only the vector, so do some arithmetically compare
		Vector zero = eq.getObservations().minus(p.getV());
		
		if(zero.getLength() != 0) fail();
	}

	/**
	 * Test for setObservations
	 */
	public void testSetObservations() {
		//construct some entity
		Point_2D p = new Point_2D(1,Double.MAX_VALUE);
		
		//build equal, and test if you get back the right point again :)
		Equal eq = new Equal(p);
		
		//construct new point and set as Observation of eq:
		Point_2D p2 = new Point_2D(Double.NEGATIVE_INFINITY,2);
		eq.setObservations(p2.getV());
		
		//test values
		Vector zero = eq.getObservations().minus(p2.getV());
		
		if(zero.getLength() >= 0.00000001) fail();
	}

	/**
	 * Test for getCovarianceOfObservations
	 */
	public void testGetCovarianceOfObservations() {
		//with cov this time.
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.018);
		Point_2D p = new Point_2D(1,2,cov);
		
		Equal eq = new Equal(p);
		
		double zero = cov.minus(eq.getCovarianceOfObservations()).vec().getLength(); 
		
		if(zero != 0) fail();
	}

}
