/*
 * Created on 04.10.2004
 *
 */
package sugr.test;

import java.util.Random;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * Test of estimation functionality of sugr.
 * 
 * @author Achim Knigs
 */
public class EstimationTest extends TestCase {
	
	/** 
	 * Test estimation of points from points, lines from lines,
	 *  and so on.
	 */
	public static void testEstimationOfSameType() {
		Entity[] est = new Entity[5];
		est[0] = new Point_2D();
		est[1] = new Line_2D();
		est[2] = new Point_3D();
		est[3] = new Line_3D();
		est[4] = new Plane_3D();

		int nrobs = 500;

		for (int i = 0; i < 5; i++) {
			{
				// Generate gaussian distributed input 
				Entity[] obs = new Entity[nrobs];
				RelationalProperty[] rprops = new RelationalProperty[nrobs];
				for (int j = 0; j < nrobs; j++) {
					double r1 = getNextGaussDist(0, 0.1, false);
					double r2 = getNextGaussDist(0, 0.2, false);
					double r3 = getNextGaussDist(0, 0.3, false);
					double r4 = getNextGaussDist(0, 0.2, false);
					double r5 = getNextGaussDist(0, 0.3, false);
					double r6 = getNextGaussDist(0, 0.1, false);

					if (est[i].getClass().equals(Point_2D.class)) {
						Matrix cov1 =
							LinearAlgebraFactory.getDiagMatrix(3, 0.1 * 0.1);
						cov1.set(1, 1, 0.2 * 0.2);
						cov1.set(2, 2, 0);
						obs[j] = new Point_2D(10.0 + r1, 2000.0 + r2, cov1);
					}
					if (est[i].getClass().equals(Line_2D.class)) {
						Matrix cov1 =
							LinearAlgebraFactory.getDiagMatrix(3, 0.1 * 0.1);
						cov1.set(1, 1, 0.2 * 0.2);
						cov1.set(2, 2, 0.3 * 0.3);
						obs[j] =
							new Line_2D(
								10.0 + r1,
								2000.0 + r2,
								302.0 + r3,
								cov1);
					}
					if (est[i].getClass().equals(Point_3D.class)) {
						Matrix cov1 =
							LinearAlgebraFactory.getDiagMatrix(4, 0.1 * 0.1);
						cov1.set(1, 1, 0.2 * 0.2);
						cov1.set(2, 2, 0.3 * 0.3);
						cov1.set(3, 3, 0);
						obs[j] =
							new Point_3D(
								10.0 + r1,
								2000.0 + r2,
								3000 + r3,
								cov1);
					}

					if (est[i].getClass().equals(Line_3D.class)) {
						Matrix cov1 =
							LinearAlgebraFactory.getDiagMatrix(6, 0.4 * 0.4);
						cov1.set(1, 1, 0.5 * 0.5);
						cov1.set(2, 2, 0.6 * 0.6);
						cov1.set(3, 3, 0.1 * 0.1);
						cov1.set(4, 4, 0.2 * 0.2);
						cov1.set(5, 5, 0.3 * 0.3);
						obs[j] =
							new Line_3D(
								40.0 + r4,
								-40.0 + r5,
								80.0 + r6,
								2.0 + r1,
								4.0 + r2,
								1.0 + r3,
								cov1,
								false);

						obs[j].normalizeSphere();
						obs[j].ensureConstraints();
					}

					if (est[i].getClass().equals(Plane_3D.class)) {
						Matrix cov1 =
							LinearAlgebraFactory.getDiagMatrix(4, 0.1 * 0.1);
						cov1.set(1, 1, 0.2 * 0.2);
						cov1.set(2, 2, 0.3 * 0.3);
						cov1.set(3, 3, 0.4 * 0.4);
						obs[j] =
							new Plane_3D(
								10.0 + r1,
								2000.0 + r2,
								3000 + r3,
								-432.0 + r4,
								cov1);
					}

					rprops[j] = new Equal(obs[j]);
				}

				boolean doConditioning = true;
				int maxiter = 200;
				RelationalProperty[] yhat = new RelationalProperty[nrobs];
				double[] sigma0 = new double[1];
				int[] no_of_iterations = new int[1];

				if (i == 2) {
					transformToCenter(obs);
				}

				boolean ok =
					est[i].estimate(
						rprops,
						doConditioning,
						maxiter,
						yhat,
						sigma0,
						no_of_iterations,true);

				
				Entity er = null;
				if (i == 1)
					er = new Line_2D(10.0, 2000.0, 302.0);
				if (i == 3)
					er = new Line_3D(40.0, -40.0, 80.0, 2.0, 4.0, 1.0);
				if (i == 4)
					er = new Plane_3D(10.0, 2000.0, 3000, -432.0);
				if (er != null) {
					er.normalizeEuclideanComparable();
				}

				est[i].normalizeEuclideanComparable();

				if(!ok)
				{
					System.out.println("=============================================");
					System.out.println("Estimation of a " + est[i].getClass()
							+  " (testEstimationOfSameType)");		

					System.out.println("Convergence NOT reached !");
					
					System.out.println("Sigma0:" + sigma0[0]);
					System.out.println("No of iterations: " + no_of_iterations[0]);
					System.out.println("Excepted Result");
					if(er != null){
						er.print(10,10);
					}
					else System.out.println("   ... not available");
					
					System.out.println("Result:");
					est[i].print(10,10);
					
					System.out.println();
					
					fail();
				}
				if(er != null){
					if(!TestStatistics.check(er,est[i],0.1)) {
						System.out.println("=============================================");
						System.out.println("Estimation of a " + est[i].getClass()
								+ " (testEstimationOfSameType)");
						System.out.println("Convergence was reached,"
								+ " but values differed to much:");
						
						System.out.println("Expected Result:");
						er.print(10,10);
						
						System.out.println("Result:");
						est[i].print(10,10);
						
						System.out.println();
						
						fail();
					}
				}

			}
		}
	}
	
	/** 
	 * Test, if a construction (without an estimation) with two entities
	 * is possible.
	 * @param result: The type of the resulting entity.
	 * @param first, second: The two arguments.
	 */
	protected static boolean constructionPossible(Class result, Class first, Class second) {
		Class [] [] possible = {
				{ sugr.Line_2D.class, sugr.Point_2D.class, sugr.Point_2D.class },
				{ sugr.Point_2D.class, sugr.Line_2D.class, sugr.Line_2D.class },
				{ sugr.Line_3D.class, sugr.Plane_3D.class, sugr.Plane_3D.class },
				{ sugr.Line_3D.class, sugr.Point_3D.class, sugr.Point_3D.class },
				{ sugr.Plane_3D.class, sugr.Line_3D.class, sugr.Point_3D.class },
				{ sugr.Plane_3D.class, sugr.Point_3D.class, sugr.Line_3D.class },
				{ sugr.Point_3D.class, sugr.Line_3D.class, sugr.Plane_3D.class },
				{ sugr.Point_3D.class, sugr.Plane_3D.class, sugr.Line_3D.class },
				{ sugr.Point_3D.class, sugr.Line_3D.class, sugr.Line_3D.class  }
			};
		
		for(int i=0; i<possible.length; i++)
			if (result.equals(possible[i][0]) && first.equals(possible[i][1]) &&
					second.equals(possible[i][2]))
				return true;
			
		return false;
	}

	/** 
	 * Doing for every possible construction an estimation
	 *  and compare the results.
	 */
	public static void testAllConstructorsAsEstimation() {

		// Input entities
		Entity[] eset1 = new Entity[5];
		{
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.02);
			cov.set(2, 2, 0.01);
			Line_2D l = new Line_2D(4234, 243231, 2342341, cov);
			eset1[0] = l;

			Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
			cov2.set(2, 2, 0.01);
			Point_2D p1 = new Point_2D(103450.5, 10.5, 1, cov2);
			eset1[1] = p1;

			Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(6, 0.02);
			Line_3D L = new Line_3D(5.08, 1, -2.93, 2, 8, 6, cov3);
			eset1[2] = L;

			Matrix cov4 = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
			cov4.set(3, 3, 0.1);
			Plane_3D A = new Plane_3D(10.5, 10.5, 10.5, cov4);
			eset1[3] = A;

			Matrix cov5 = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
			cov5.set(3, 3, 0.01);
			Point_3D P = new Point_3D(10.5, 10.5, 1.5, cov5);
			eset1[4] = P;
		}

		Entity[] eset2 = new Entity[5];
		{
			Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.02);
			cov.set(2, 2, 0.01);
			Line_2D l = new Line_2D(-3, 2, 234321, cov);
			eset2[0] = l;

			Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
			cov2.set(2, 2, 0.01);
			Point_2D p1 = new Point_2D(10123210.5, 21213120.5, 1, cov2);
			eset2[1] = p1;

			Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(6, 0.02);
			Line_3D L = new Line_3D(1.5, 4.07, 3, -4, -3, 6, cov3);
			eset2[2] = L;

			Matrix cov4 = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
			cov4.set(3, 3, 0.1);
			Plane_3D A = new Plane_3D(120.5, 10.5, 14410.5, cov4);
			eset2[3] = A;

			Matrix cov5 = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
			cov5.set(3, 3, 0.01);
			Point_3D P = new Point_3D(15440.5, 140.5, 10.5, cov5);
			eset2[4] = P;
		}

		boolean fail = false;
		
		// For all combinations: do construction and then estimation
		Entity constructed = null, est = null;
		for (int k = 0; k < 5; k++)
			for (int i = 0; i < 5; i++)
				for (int j = 0; j < 5; j++) {

					boolean go_on = true;
					try {
						constructed =
							(Entity) Factory.getInstance(
								eset1[i],
								eset2[j],
								eset1[k].getClass().getName());
					} catch (IllegalArgumentException e) {
						go_on = false;
					}

					if (go_on && constructionPossible(eset1[k].getClass(), eset1[i].getClass(), eset2[j].getClass())) { // construction possible !
						Class[] para = new Class[0];

						RelationalProperty[] rp = new RelationalProperty[2];
						rp[0] = new Incident(eset1[i], eset1[k].getClass());
						rp[1] = new Incident(eset2[j], eset1[k].getClass());
						Object[] args = new Object[0];

						try {
							est =
								(Entity) eset1[k].getClass().getConstructor(
									para).newInstance(
									args);

							boolean doConditioning = true;
							int maxiter = 10;
							RelationalProperty[] yhat =
								new RelationalProperty[2];
							double[] sigma0 = new double[1];
							int[] noit = new int[1];
							boolean estok =
								est.estimate(
									rp,
									doConditioning,
									maxiter,
									yhat,
									sigma0,
									noit,true);
							
							//normalizations needed for comparison
							((BiRelationalProperty) yhat[0])
								.getFirstEntity().normalizeEuclideanComparable();
							eset1[i].normalizeEuclideanComparable();
							((BiRelationalProperty) yhat[1])
								.getFirstEntity().normalizeEuclideanComparable();
							eset2[j].normalizeEuclideanComparable();
							est.normalizeEuclideanComparable();
							constructed.normalizeEuclideanComparable();
							
							if(!compareEntities(
									((BiRelationalProperty) yhat[0])
									.getFirstEntity(),
									eset1[i],
									0.4)){
								System.out.println("=============================================");
								System.out.println("First observation is not identical !");
								System.out.println(
										"\tCompute a "
											+ eset1[k].getClass()
											+ " out of a "
											+ eset1[i].getClass()
											+ " and a "
											+ eset2[j].getClass()
											+ ". (testAllConstructosAsEstimation)");
								System.out.println("Sigma0: " + sigma0[0]);
								System.out.println("no of iterations: " + noit[0]);
								
								System.out.println("Convergence: " + estok);
								
								System.out.println("Expected result:\n"+
									((BiRelationalProperty)yhat[0])
									.getFirstEntity());
								
								System.out.println("Estimated:\n" +
										eset1[i]);
								
								System.out.println();
								
								fail = true;
							}
							if(!compareEntities(
									((BiRelationalProperty) yhat[1])
									.getFirstEntity(),
									eset2[j],0.4)){
								System.out.println("=============================================");
								System.out.println("Second observation is not identical !");
								System.out.println(
										"\tCompute a "
											+ eset1[k].getClass()
											+ " out of a "
											+ eset1[i].getClass()
											+ " and a "
											+ eset2[j].getClass()
											+ ". (testAllConstructosAsEstimation)");
								System.out.println("Sigma0: " + sigma0[0]);
								System.out.println("no of iterations: " + noit[0]);
								
								System.out.println("Convergence: " + estok);
								
								System.out.println("Expected result:");
								((BiRelationalProperty)yhat[1]).
									getFirstEntity().print(10,10);
								
								System.out.println("Estimated: ");
								eset2[j].print(10,10);
								System.out.println();
								
								fail = true;
							}
							

							if(!compareEntities(
									constructed,
									est,0.4)){
								System.out.println("=============================================");
								System.out.println("Result of estimation not identical!");
								System.out.println(
										"\tCompute a "
											+ eset1[k].getClass()
											+ " out of a "
											+ eset1[i].getClass()
											+ " and a "
											+ eset2[j].getClass()
											+ ". (testAllConstructosAsEstimation)");
								System.out.println("Sigma0: " + sigma0[0]);
								System.out.println("no of iterations: " + noit[0]);
								
								System.out.println("Convergence: " + estok);
								
								System.out.println("Expected result:");
								constructed.print(10,10);
								
								System.out.println("Estimated: ");
								est.print(10,10);
								
								System.out.println();
								
								fail = true;
							}

						} catch (Exception e) {
							System.out.println("=============================================");
							System.out.println(
									"\tCompute a "
										+ eset1[k].getClass()
										+ " out of a "
										+ eset1[i].getClass()
										+ " and a "
										+ eset2[j].getClass()
										+ ". (testAllConstructosAsEstimation)");
							System.out.println(
								"Exception while estimate: " + e);
							System.out.println();
							fail = true;
						}

					}
				}
		
		if(fail) {
			System.out.println("At least one problem occured. (testAllConstructosAsEstimation)");
			fail();
		}
	}
	
	/** 
	 * Do a very simple estimation with point2d out of points2d. 
	 **/
	public static void testEstimation1() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
		cov.set(2, 2, 0.00000);

		Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
		cov2.set(0, 0, 100.00000);
		cov2.set(2, 2, 0.00000);

		//expected result == p1
		Point_2D p1 = new Point_2D(10.5, 10.5, cov);
		Point_2D p2 = new Point_2D(10.6, 10.6, cov);
		Point_2D p3 = new Point_2D(10.4, 10.4, cov);

		RelationalProperty rprops[] = new RelationalProperty[3];

		rprops[0] = new Equal(p1, Point_2D.class);
		rprops[1] = new Equal(p2, Point_2D.class);
		rprops[2] = new Equal(p3, Point_2D.class);

		Point_2D est = new Point_2D();

		//approximation
		est.approximate(rprops);
		est.normalizeEuclidean();

		//estimation
		RelationalProperty[] yhat = new RelationalProperty[3];
		double[] sigma0 = new double[1];
		int[] no_of_iterations = new int[1];
		boolean ok =
			est.estimate(rprops, true, 200, yhat, sigma0, no_of_iterations,true);
		est.normalizeEuclidean();

		if (ok) {
			if(!TestStatistics.check(p1.getV(),est.getV(),0.3)){
				System.out.println("=============================================");
				System.out.println("	Estimation of a Point out of three Points (testEstimation1)");

				
				System.out.println("Estimation ok, " +
						"but result differst from expectation." +
						"\nApproximation was ok.");
				System.out.println("Result should be:");
				p1.print(6, 6);
				
				System.out.println("Estimated value:");
				est.print(6, 6);
				
				System.out.println("Sigma0^2:" + sigma0[0] * sigma0[0]);
				System.out.println("Residuals: ");
				for (int i = 0; i < rprops.length; i++) {
					Entity y = ((BiRelationalProperty) yhat[i]).getFirstEntity();
					y.normalizeEuclidean();
					y.getV().print(6, 6);
				}
				
				fail();
			}
		} 
		else
		{
			System.out.println("=============================================");
			System.out.println("	Estimation of a Point out of three Points (testEstimation1)");
			
			System.out.println("Estimation did not reach convergence!");
			System.out.println("Result should be:");
			p1.print(6, 6);
			
			System.out.println("Estimated value:");
			est.print(6, 6);
			
			fail();
		}
	}
	
	/** 
	 * Estimate point3d from line3d.
	 */
	public static void testEstimation2() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		cov.set(3, 3, 0.00000);

		Point_3D P1 = new Point_3D(0.5, 0.5, 0.5, cov);
		Point_3D P1s = new Point_3D(0.51, 0.51, 0.51, cov);
		Point_3D P2 = new Point_3D(1, 2, 4, cov);
		Point_3D P3 = new Point_3D(7, 8, -4, cov);
		Point_3D P4 = new Point_3D(4, 2, 4, cov);
		Point_3D P5 = new Point_3D(7, 5, -4, cov);

		Line_3D L1 = new Line_3D(P1, P2);
		Line_3D L2 = new Line_3D(P1, P3);
		Line_3D L3 = new Line_3D(P1, P4);
		Line_3D L4 = new Line_3D(P1, P5);

		Line_3D L1s = new Line_3D(P1s, P2);
		Line_3D L2s = new Line_3D(P1s, P3);
		Line_3D L3s = new Line_3D(P1s, P4);
		Line_3D L4s = new Line_3D(P1s, P5);

		RelationalProperty rprops[] = new RelationalProperty[8];

		rprops[0] = new Incident(L1, Point_3D.class);
		rprops[1] = new Incident(L2, Point_3D.class);
		rprops[2] = new Incident(L3, Point_3D.class);
		rprops[3] = new Incident(L4, Point_3D.class);

		rprops[4] = new Incident(L1s, Point_3D.class);
		rprops[5] = new Incident(L2s, Point_3D.class);
		rprops[6] = new Incident(L3s, Point_3D.class);
		rprops[7] = new Incident(L4s, Point_3D.class);

		Point_3D est = new Point_3D();

		//expected result
		Point_3D res = new Point_3D(0.505, 0.505, 0.505, cov);

		//approximation
		est.approximate(rprops);
		est.normalizeEuclidean();
		
		//estimation:
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] nri = new int[1];
		boolean ok = est.estimate(rprops, true, 200, yhat, sigma0, nri,true);
		est.normalizeEuclidean();
		
		if(!TestStatistics.check(res.getV(),est.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("\tEstimation of a Point3D of Line3Ds (testEstimation2)");
			if(ok){
				System.out.println("Convergence reached");
				

				System.out.println("Expected Result:");
				res.print(6, 6);
				
				System.out.println("Estimated:");
				est.print(6, 6);
				
				System.out.println("sigma0: " + sigma0[0]);
				System.out.println("no of iterations: " + nri[0]);
				
				fail();
			}
			else {
				System.out.println("Convergence not reached");	
				
				System.out.println("Expected Result:");
				res.print(6, 6);
				
				System.out.println("Estimated:");
				est.print(6, 6);
				
				System.out.println("sigma0: " + sigma0[0]);
				System.out.println("no of iterations: " + nri[0]);
				
				fail();
			}
		}
	}

	
	/** 
	 * Estimate a plane from points3d. 
	 */
	public static void testEstimation3() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		cov.set(3, 3, 0.00000);

		Point_3D P1 = new Point_3D(0.5, 0.5, 4.1, cov);
		Point_3D P2 = new Point_3D(1, 2, 4.1, cov);
		Point_3D P3 = new Point_3D(7, 8, 4.1, cov);
		Point_3D P4 = new Point_3D(4, 2, 3.9, cov);
		Point_3D P5 = new Point_3D(7, 5, 3.9, cov);
		Point_3D P6 = new Point_3D(37, 5, 3.9, cov);

		RelationalProperty rprops[] = new RelationalProperty[6];

		rprops[0] = new Incident(P1, Plane_3D.class);
		rprops[1] = new Incident(P2, Plane_3D.class);
		rprops[2] = new Incident(P3, Plane_3D.class);
		rprops[3] = new Incident(P4, Plane_3D.class);
		rprops[4] = new Incident(P5, Plane_3D.class);
		rprops[5] = new Incident(P6, Plane_3D.class);

		Plane_3D est = new Plane_3D();

		//expected result
		Plane_3D res = new Plane_3D(0, 0, 1, -4);

		//approximation
		est.approximate(rprops);
		est.normalizeEuclidean();

		//estimation
		est.estimate(rprops);
		est.normalizeEuclidean();

		
		if(!TestStatistics.check(res.getV(),est.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("\tEstimation of a plane of 3d Points (testEstimation3)\n");
			
			System.out.println("Excepted result:");
			res.getV().print(6, 6);
			
			System.out.println("New Estimation:");
			est.print(6, 6);
			
			fail();
		}
	}
	
	/** 
	 * Estimate a point3d from points3d. 
	 */
	public static void testEstimation4() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		cov.set(3, 3, 0.00000);

		//expected result == p1
		Point_3D p1 = new Point_3D(0.5, 0.5, 3, cov);
		Point_3D p2 = new Point_3D(0.6, 0.6, 3, cov);
		Point_3D p3 = new Point_3D(0.4, 0.4, 3, cov);



		RelationalProperty rprops[] = new RelationalProperty[3];

		rprops[0] = new Equal(p1, Point_3D.class);
		rprops[1] = new Equal(p2, Point_3D.class);
		rprops[2] = new Equal(p3, Point_3D.class);

		Point_3D est = new Point_3D();

		//approximation
		est.approximate(rprops);

		//estimation
		est.estimate(rprops);
		est.normalizeEuclidean();
		
		if(!TestStatistics.check(p1.getV(),est.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("\tEstimation of a point3d from 3d Points (testEstimation4)\n");
			
			System.out.println("Result should be:");
			p1.print(6, 6);
			
			System.out.println("New Estimation:");
			est.print(6, 6);
			
			fail();
		}
	}
	
	/** 
	 * Estimate a line2d from points2d.
	 */
	public static void testEstimation5() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.0001);
		cov.set(2, 2, 0.00000);

		Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
		cov2.set(2, 2, 0.00000);

		Point_2D p1 = new Point_2D(1.0, 1.0, cov);
		Point_2D p2 = new Point_2D(2, 2.1, cov2);
		Point_2D p3 = new Point_2D(3, 2.9, cov2);
		Point_2D p4 = new Point_2D(4, 4, cov);

		RelationalProperty rprops[] = new RelationalProperty[4];

		rprops[0] = new Incident(p1, Line_2D.class);
		rprops[1] = new Incident(p4, Line_2D.class);
		rprops[2] = new Incident(p2, Line_2D.class);
		rprops[3] = new Incident(p3, Line_2D.class);

		Line_2D est = new Line_2D();
		
		//approximation
		est.approximate(rprops);

		//estimation
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] nri = new int[1];

		boolean ok = est.estimate(rprops, true, 200, yhat, sigma0, nri,true);

		est.normalizeEuclideanComparable();

		// Construction:
		Line_2D l2 = new Line_2D(p1, p4);
		l2.normalizeEuclideanComparable();

		if(!TestStatistics.check(l2.getV(),est.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("\tEstimation of a line2d from points 2d (testEstimation5)\n");
			
			System.out.println("Constructed line:");
			l2.print(15, 15);
			
			System.out.println("Estimation:");
			System.out.print("Result: ");		
			est.print(15, 15);
			
			System.out.println("sigma0: " + sigma0[0]);
			System.out.println("no of iterations: " + nri[0]);
			System.out.println("Ok" + ok);
			
			for (int i = 0; i < yhat.length; i++) {
				System.out.println("resudual no " + i);
				((BiRelationalProperty) yhat[i]).getFirstEntity().print(6, 6);
			}
			
			fail();
		}
	}
	
	/** 
	 * Estimate a point3d from point3d. 
	 */
	public static void testEstimation6() {

		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		cov.set(3, 3, 0.00000);

		Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		cov2.set(3, 3, 0.00000);

		//expected result == p1
		Point_3D p1 = new Point_3D(10.5, 10.5, 5, cov);
		Point_3D p2 = new Point_3D(10.6, 10.6, 5.1, cov);
		Point_3D p3 = new Point_3D(10.4, 10.4, 4.9, cov);

		RelationalProperty rprops[] = new RelationalProperty[3];

		rprops[0] = new Equal(p1, Point_3D.class);
		rprops[1] = new Equal(p2, Point_3D.class);
		rprops[2] = new Equal(p3, Point_3D.class);

		Point_3D est = new Point_3D();

		RelationalProperty[] yhat = new RelationalProperty[3];
		double[] sigma0 = new double[1];
		int[] no_of_iterations = new int[1];
		boolean ok =
			est.estimate(rprops, true, 200, yhat, sigma0, no_of_iterations,true);
		est.normalizeEuclidean();
		
		if(!TestStatistics.check(p1.getV(),est.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("\tEstimation of a point3d from points 3d (testEstimation6)\n");
		
			System.out.println("Result should be:");
			p1.print(6, 6);
			
			System.out.println("Estimation:");
			est.print(6, 6);
			
			if (ok) {

				System.out.println("Estimation ok");
				System.out.println("Sigma0^2:" + sigma0[0] * sigma0[0]);
				System.out.println("No of iterations:" + no_of_iterations[0]);
				
			} else
				System.out.println("Estimation did not reach convergence!");
			
			fail();
		}
	}
	
	/** 
	 * Test estimation with orthogonal and parallel conditions.
	 */
	public static void testEstimationOP() {
		Entity[] est = new Entity[3];
		est[0] = new Line_2D();
		est[1] = new Line_3D();
		est[2] = new Plane_3D();

		Entity[] est1 = new Entity[3];
		est1[0] = new Line_2D();
		est1[1] = new Line_3D();
		est1[2] = new Plane_3D();

		int nrobs = 50;

		boolean failure = false;
		
		for (int i = 0; i < 3; i++) {
			// Generate gaussian distributed input 
			Entity[] obs = new Entity[nrobs];
			RelationalProperty[] rprops = new RelationalProperty[nrobs + 1];
			RelationalProperty[] ort = new RelationalProperty[nrobs + 1];
			for (int j = 0; j < nrobs; j++) {
				double r1 = getNextGaussDist(0, 0.01, false);
				double r2 = getNextGaussDist(0, 0.02, false);
				double r3 = getNextGaussDist(0, 0.03, false);
				double r4 = getNextGaussDist(0, 0.04, false);
				double r5 = getNextGaussDist(0, 0.05, false);
				double r6 = getNextGaussDist(0, 0.06, false);

				if (i == 0) {
					Matrix cov1 =
						LinearAlgebraFactory.getDiagMatrix(3, 0.01 * 0.01);
					cov1.set(1, 1, 0.02 * 0.02);
					cov1.set(2, 2, 0.03 * 0.03);
					obs[j] =
						new Line_2D(0.707 + r1, 0.707 + r2, -3.0 + r3, cov1);
					rprops[j] = new Parallel((Line_2D) obs[j], Line_2D.class);
					ort[j] = new Orthogonal((Line_2D) obs[j], Line_2D.class);
				}

				if (i == 1) {
					Matrix cov1 =
						LinearAlgebraFactory.getDiagMatrix(6, 0.01 * 0.01);
					cov1.set(1, 1, 0.02 * 0.02);
					cov1.set(2, 2, 0.03 * 0.03);
					cov1.set(3, 3, 0.01 * 0.01);
					cov1.set(4, 4, 0.01 * 0.01);
					cov1.set(5, 5, 0.01 * 0.01);
					obs[j] =
						obs[j] =
							new Line_3D(
								40.0 + r1,
								-40.0 + r2,
								0.0 + r3,
								2.0 + r4,
								2.0 + r5,
								1.0 + r6,
								cov1,
								false);
					obs[j].normalizeSphere();
					obs[j].ensureConstraints();
					rprops[j] = new Parallel((Line_3D) obs[j], Line_3D.class);
					ort[j] = new Orthogonal((Line_3D) obs[j], Line_3D.class);
				}
				if (i == 2) {
					Matrix cov1 =
						LinearAlgebraFactory.getDiagMatrix(4, 0.01 * 0.01);
					cov1.set(1, 1, 0.02 * 0.02);
					cov1.set(2, 2, 0.03 * 0.03);
					cov1.set(3, 3, 0.04 * 0.04);
					obs[j] =
						new Plane_3D(0 + r1, 0 + r2, -5 + r3, 4 + r4, cov1);
					rprops[j] = new Parallel((Plane_3D) obs[j], Plane_3D.class);
					ort[j] = new Orthogonal((Plane_3D) obs[j], Plane_3D.class);
				}

			}

			// A 2D Point must be added to constraints !
			Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.01 * 0.01);
			cov2.set(2, 2, 0.0);
			Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(4, 0.01 * 0.01);
			cov3.set(3, 3, 0.0);

			Point_2D p1 = new Point_2D(0.8, 0.8, 1, cov2);
			Point_2D p11 = new Point_2D(0.8 - 1, 0.8 + 1, 1, cov2);

			Point_2D p2 = new Point_2D(-0.8, 0.8, 1, cov2);
			Point_2D p21 = new Point_2D(-0.8 + 1, 0.8 + 1, 1, cov2);

			Point_3D P1 = new Point_3D(0.8, 0.8, 0.2, 1, cov3);
			Point_3D P11 = new Point_3D(0.3, 0.2, 0.2, 1, cov3);
			Point_3D P12 = new Point_3D(3.3, 0.2, 0.2, 1, cov3);

			Point_3D P2 = new Point_3D(-0.8, 0.8, 0.2, 1, cov3);
			Point_3D P3 = new Point_3D(-0.8, 0.7, 0.2, 1, cov3);

			if (i == 0) {
				rprops[nrobs] = new Incident(p1, Line_2D.class);
				ort[nrobs] = new Incident(p2, Line_2D.class);
			}
			if (i == 1 || i == 2) {

				if (i == 1) {
					rprops[nrobs] = new Incident(P1, Line_3D.class);
					ort[nrobs] = new Incident(P2, Line_3D.class);
				}
				if (i == 2) {
					rprops[nrobs] = new Incident(P1, Plane_3D.class);

					ort[nrobs] = new Incident(P2, Plane_3D.class);
					ort[nrobs - 1] = new Incident(P3, Plane_3D.class);
				}

			}

			//parallel estimation
			boolean doConditioning = true;
			int maxiter = 200;
			RelationalProperty[] yhat = new RelationalProperty[nrobs + 1];
			double[] sigma0 = new double[1];
			int[] no_of_iterations = new int[1];

			boolean ok =
				est[i].estimate(
					rprops,
					doConditioning,
					maxiter,
					yhat,
					sigma0,
					no_of_iterations,true);

			Entity er = null;
			if (i == 2) {
				er = new Plane_3D(P1, P11, P12);
			}
			if (i == 0) {
				er = new Line_2D(p1, p11);
			}

			if (er != null){
				er.normalizeEuclideanComparable();
			}

			est[i].normalizeEuclideanComparable();

			if(er != null){
				if(!TestStatistics.check(er.getV(),est[i].getV(),0.3)){
					System.out.println("=============================================");
					System.out.println("\tParallel Estimation of a " + est[i].getClass()
							+ " (testEstimationOP)");
					
					if (ok){
						System.out.println("Convergence reached.");
					}
					else
						System.out.println("Convergence NOT reached !");

					System.out.println("Sigma0:" + sigma0[0]);
					System.out.println("No of iterations: " + no_of_iterations[0]);
					System.out.println("Excepted Result");

					er.getV().print(6, 6);
					
					System.out.println("Result:");
					est[i].print(10, 10);
					
					failure = true;
				}
			}
			else {
				if(!ok){
					System.out.println("=============================================");
					System.out.println("\tParallel Estimation of a " + est[i].getClass()
							+ " (testEstimationOP)");
					
					System.out.println("Convergence NOT reached !");

					System.out.println("Sigma0:" + sigma0[0]);
					System.out.println("No of iterations: " + no_of_iterations[0]);
					System.out.println("Excepted Result not available");
					
					System.out.println("Result:");
					est[i].print(10, 10);
					
					failure = true;
				}
			}
			
			//orthogonal estimation
			ok =
				est1[i].estimate(
					ort,
					doConditioning,
					maxiter,
					yhat,
					sigma0,
					no_of_iterations,true);

			er = null;
			if (i == 0) {
				er = new Line_2D(p2, p21);
			}

			if (er != null){
				er.normalizeEuclideanComparable();
			}

			est1[i].normalizeEuclideanComparable();
			
			if(er != null){
				if(!TestStatistics.check(er.getV(),est1[i].getV(),0.3)){
					System.out.println("=============================================");
					System.out.println(
							"Orthogonal Estimation of a " + est[i].getClass()
							+ " (testEstimationOP)");

					if (ok){
						System.out.println("Convergence reached.");
					}
					else
						System.out.println("Convergence NOT reached !");

					System.out.println("Sigma0:" + sigma0[0]);
					System.out.println("No of iterations: " + no_of_iterations[0]);
					System.out.println("Excepted Result");
					er.getV().print(6, 6);
					
					System.out.println("Result:");
					est1[i].print(10, 10);
					failure = true;
				}
			}
			else {
				if(!ok){
					System.out.println("=============================================");
					System.out.println(
							"Orthogonal Estimation of a " + est[i].getClass()
							+ " (testEstimationOP)");

					if (ok){
						System.out.println("Convergence reached.");
					}
					else
						System.out.println("Convergence NOT reached !");

					System.out.println("Sigma0:" + sigma0[0]);
					System.out.println("No of iterations: " + no_of_iterations[0]);
					
					System.out.println("Result:");
					est1[i].print(10, 10);
					failure = true;
				}
			}
		} // for i
		if(failure){
			System.out.println("\nAt least one problem occured in testEstimationOP");
			fail();
		}
	}
	
	/** 
	 * Test estimation of planes with parallel or orthogonal lines.
	 */
	public static void testEstimationPlaneOP() {
		Entity[] est = new Entity[1];
		est[0] = new Plane_3D();

		Entity[] est1 = new Entity[1];
		est1[0] = new Plane_3D();

		int nrobs = 200;
		
		random = new Random();

		Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(4, 0.01 * 0.01);
		cov3.set(3, 3, 0.0);

		Point_3D P1 = new Point_3D(3, 4, 7, cov3);
		Point_3D P2 = new Point_3D(3, 4, 2, cov3);
		//Point_3D P3 = new Point_3D(24, 4, 7, cov3);
		//Point_3D P4 = new Point_3D(3, 14, 7, cov3);
		Point_3D P5 = new Point_3D(24, 4, 2, cov3);
		Point_3D P6 = new Point_3D(3, 14, 2, cov3);

		// Generate gaussian distributed input 
		Entity[] obs = new Entity[nrobs];
		RelationalProperty[] rprops = new RelationalProperty[nrobs + 1];
		RelationalProperty[] ort = new RelationalProperty[nrobs / 2 + 1];
		for (int j = 0; j < nrobs / 2; j++) {
			double r1 = getNextGaussDist(0, 0.01, false);
			double r2 = getNextGaussDist(0, 0.01, false);
			double r3 = getNextGaussDist(0, 0.01, false);
			double r4 = getNextGaussDist(0, 0.01, false);
			double r5 = getNextGaussDist(0, 0.01, false);
			double r6 = getNextGaussDist(0, 0.01, false);
			double r7 = getNextGaussDist(0, 0.01, false);
			double r8 = getNextGaussDist(0, 0.01, false);
			double r9 = getNextGaussDist(0, 0.01, false);

			Point_3D cP1 = new Point_3D(3 + r1, 4 + r2, 7 + r3, cov3);
			Point_3D cP3 = new Point_3D(2 + r4, 5 + r5, 7 + r6, cov3);
			Point_3D cP4 = new Point_3D(3 + r7, 14 + r8, 7 + r9, cov3);

			obs[2 * j] = new Line_3D(cP1, cP3);
			obs[2 * j + 1] = new Line_3D(cP1, cP4);
			rprops[2 * j] = new Parallel((Line_3D) obs[2 * j], Plane_3D.class);
			rprops[2 * j + 1] =
				new Parallel((Line_3D) obs[2 * j + 1], Plane_3D.class);

			ort[j] = new Orthogonal((Line_3D) obs[2 * j], Plane_3D.class);
		}

		rprops[nrobs] = new Incident(P2, Plane_3D.class);
		ort[nrobs / 2] = new Incident(P2, Plane_3D.class);

		boolean doConditioning = true;
		int maxiter = 200;
		RelationalProperty[] yhat = new RelationalProperty[nrobs + 1];
		double[] sigma0 = new double[1];
		int[] no_of_iterations = new int[1];

		//parallel estimation
		boolean ok =
			est[0].estimate(
				rprops,
				doConditioning,
				maxiter,
				yhat,
				sigma0,
				no_of_iterations,true);

		//expected result
		Entity er = new Plane_3D(P2, P6, P5);
		
		//normalization needed for comparisation
		er.normalizeEuclideanComparable();
		est[0].normalizeEuclideanComparable();
		
		if(!TestStatistics.check(er.getV(),est[0].getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("Parallel Estimation of a " + est[0].getClass()
					+ " (testEstimationPlaneOP)");
			
			if (ok){
				System.out.println("Convergence reached.");
				System.out.println("Sigma0:" + sigma0[0]);
				System.out.println("No of iterations: " + no_of_iterations[0]);
				System.out.println("Excepted Result");
				er.print(6, 6);
				
				System.out.println("Result:");
				est[0].print(10, 10);
				
				fail();				
			}
			else{
				System.out.println("Convergence NOT reached !");
				System.out.println("Sigma0:" + sigma0[0]);
				System.out.println("No of iterations: " + no_of_iterations[0]);
				System.out.println("Excepted Result");
				er.print(6, 6);
				
				System.out.println("Result:");
				est[0].print(10, 10);
				
				fail();
			}
		}

		//orthogonal estimation
		yhat = new RelationalProperty[nrobs / 2 + 1];
		ok =
			est1[0].estimate(
				ort,
				doConditioning,
				maxiter,
				yhat,
				sigma0,
				no_of_iterations,true);

		//expected Result
		Vector v = P1.getV().copy();
		v.set(0, v.get(0) + 5);
		v.set(1, v.get(1) + 5);
		Point_3D pH = new Point_3D(v, P1.getCov());
		er = new Plane_3D(P1, P2, pH);
		
		//normalization needed for comparation
		er.normalizeEuclideanComparable();
		est1[0].normalizeEuclideanComparable();
		
		if(!TestStatistics.check(er.getV(),est1[0].getV(),0.3)
				&& !TestStatistics.check(er.getV().mult(-1.0),est1[0].getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("Orthogonal Estimation of a " + est1[0].getClass()
					+ " (testEstimationPlaneOP)");
			
			if (ok){
				System.out.println("Convergence reached.");
				System.out.println("Sigma0:" + sigma0[0]);
				System.out.println("No of iterations: " + no_of_iterations[0]);
				System.out.println("Excepted Result");
				er.print(6, 6);
				
				System.out.println("Result:");
				est1[0].print(10, 10);
				
				fail();
			}
			else{
				System.out.println("Convergence NOT reached !");
				System.out.println("Sigma0:" + sigma0[0]);
				System.out.println("No of iterations: " + no_of_iterations[0]);
				System.out.println("Excepted Result");
				er.print(6, 6);
				
				System.out.println("Result:");
				est1[0].print(10, 10);
				
				fail();
			}
		}
	}
	
	/** 
	 * Estimate a homography 2D, from points and lines.
	 */
	public static void testHomography2D() {
		int nr = 300;
		double sigmaobs = 0.0001;
		
		random = new Random();

		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, sigmaobs * sigmaobs);
		cov.set(2, 2, 0);

		Point_2D[] points = new Point_2D[nr];
		for (int i = 0; i < nr; i++)
			points[i] =
				new Point_2D(
					getNextGaussDist(0,10,false),
					getNextGaussDist(0,10,false),
					1,
					cov);

		// Original lines
		Matrix cov3 =
			LinearAlgebraFactory.getDiagMatrix(3, sigmaobs * sigmaobs);

		Line_2D[] lines = new Line_2D[nr];
		for (int i = 0; i < nr; i++)
			lines[i] =
				new Line_2D(
						getNextGaussDist(0,10,false),
						getNextGaussDist(0,10,false),
						getNextGaussDist(0,10,false),
						cov3);

		// "Real" homography  
		double[] harray =
			{
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				1};

		Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(9, sigmaobs * sigmaobs);
		cov2.set(8,8,0);
		Homography_2D target = new Homography_2D(harray,cov2);

		// Transformed points
		Point_2D[] hpoints = new Point_2D[nr];
		for (int i = 0; i < nr; i++)
			hpoints[i] = (Point_2D) target.transform(points[i]);

		// noise on points
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
		}

		// Transformed lines
		Line_2D[] hlines = new Line_2D[nr];
		for (int i = 0; i < nr; i++)
			hlines[i] = (Line_2D) target.transform(lines[i]);

		// noise on lines
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nr];
		RelationalProperty[] rpropsLines = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rpropsLines[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					Homography_2D.class);
		for (int i = 0; i < nr; i++)
			rprops[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					Homography_2D.class);

		//erstimate points
		Homography_2D est = new Homography_2D();
		nowEstimate(target, est, rprops,"testHomography2D - Points", 0.3);
		
		//estimate lines
		est = new Homography_2D();
		nowEstimate(target, est, rpropsLines,"testHomography2D - Lines", 0.3);
	}
	
	/** 
	 * Estimate a homography 3D using points, lines and planes
	 */
	public static void testHomography3D() {
		int nr = 300;
		double sigmaobs_lines = 0.0001;
		double sigmaobs = 0.001;
		
		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D[] points = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			points[i] =
				new Point_3D(
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					1,
					cov);

		// Original lines
		Line_3D[] lines = new Line_3D[nr];
		Matrix cov6 =
			LinearAlgebraFactory.getDiagMatrix(
				6,
				sigmaobs_lines * sigmaobs_lines);

		for (int i = 0; i < nr; i++) {
			Point_3D P1 =
				new Point_3D(
					getNextGaussDist(2, 1, false),
					getNextGaussDist(2, 1, false),
					getNextGaussDist(2, 1, false),
					1,
					cov);
			Point_3D P2 =
				new Point_3D(
					getNextGaussDist(-2, 1, false),
					getNextGaussDist(-2, 1, false),
					getNextGaussDist(-2, 1, false),
					1,
					cov);

			lines[i] = new Line_3D(P1, P2);
			lines[i].ensureConstraints();
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}

		// Original planes
		// Construct planes out of points, so that the planes
		// have a correct covariance
		Plane_3D[] planes = new Plane_3D[nr];
		for (int i = 0; i < nr/2; i++) {
			Point_3D P1 = new Point_3D(
					random.nextDouble()*40+5,
					random.nextDouble()*40+0,
					random.nextDouble()*40+0,
					1,
					cov);
			Point_3D P2 = new Point_3D(
					random.nextDouble()*40+0,
					random.nextDouble()*40+5,
					random.nextDouble()*40+0,
					1,
					cov);
			Point_3D P3 = new Point_3D(
					random.nextDouble()*40+0,
					random.nextDouble()*40+0,
					random.nextDouble()*40+5,
					1,
					cov);
			
			planes[i] = new Plane_3D(P1, P2, P3);
		}
		for (int i = nr/2; i < nr; i++) {
			Point_3D P1 = new Point_3D(
					-random.nextDouble()*40+5,
					-random.nextDouble()*40+0,
					-random.nextDouble()*40+0,
					1,
					cov);
			Point_3D P2 = new Point_3D(
					-random.nextDouble()*40+0,
					-random.nextDouble()*40+5,
					-random.nextDouble()*40+0,
					1,
					cov);
			Point_3D P3 = new Point_3D(
					random.nextDouble()*40+0,
					random.nextDouble()*40+0,
					random.nextDouble()*40+5,
					1,
					cov);
			
			planes[i] = new Plane_3D(P1, P2, P3);
		}


		// "Real" homography  
		double[] harray =
			{
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				getNextGaussDist(0, 5, false),
				1};

		Matrix covHom = LinearAlgebraFactory.getDiagMatrix(16,
				0.01*0.01);
		covHom.set(15,15,0);
		Homography_3D target = new Homography_3D(harray,covHom);

		// Transformed lines
		Line_3D[] hlines = new Line_3D[nr];
		for (int i = 0; i < nr; i++) {
			hlines[i] = (Line_3D) target.transform(lines[i]);
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}

		// noise on lines
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			lines[i].normalizeSphere();
			lines[i].ensureConstraints();
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			hlines[i].normalizeSphere();
			hlines[i].ensureConstraints();
		}
		
		covHom = LinearAlgebraFactory.getDiagMatrix(16,sigmaobs * sigmaobs);
		covHom.set(15,15,0);
		target = new Homography_3D(harray,covHom);
		
		// Transformed points
		Point_3D[] hpoints = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			hpoints[i] = (Point_3D) target.transform(points[i]);

		// noise on points
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Transformed planes
		Plane_3D[] hplanes = new Plane_3D[nr];
		for (int i = 0; i < nr; i++) {
			hplanes[i] = (Plane_3D) target.transform(planes[i]);
		}

		// Noise on planes
		for (int i = 0; i < nr; i++) {
			RandomVector r=new RandomVector(planes[i].getV(), planes[i].getCov());
			planes[i].setV(r.getNextRandom());
			
			r=new RandomVector(hplanes[i].getV(), hplanes[i].getCov());
			hplanes[i].setV(r.getNextRandom());
		}

			
		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rprops[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					Homography_3D.class);
			
		RelationalProperty[] rpropsPoints = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rpropsPoints[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					Homography_3D.class);
	
		RelationalProperty[] rpropsPlanes = new RelationalProperty[nr];

		for (int i = 0; i < nr; i++)
				rpropsPlanes[i] =
					new TriRelationalProperty(
						planes[i],
						hplanes[i],
						Homography_3D.class);

		//estimate using lines
		Homography_3D est = new Homography_3D();
		nowEstimate(target, est, rprops, "testHomography3D - Lines", 0.2);
		
		//estimate using points
		est = new Homography_3D();
		nowEstimate(target, est, rpropsPoints, "testHomography3D - Points", 0.2);

		//estimate using planes
		est = new Homography_3D();
		nowEstimate(target, est, rpropsPlanes, "testHomography3D - Planes", 0.4);
	}
	
	/** 
	 * Test Estimation of projective camera.
	 * Uses random values for camera and observations.
	 */
	public static void testProjectiveCamera() {
		int nr = 300;
		double sigmaobs = 0.001;
		double sigmaobsLines = 0.0001;
		random = new Random();

		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D[] points = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			points[i] =
				new Point_3D(
					getNextGaussDist(0, 2, false),
					getNextGaussDist(0, 2, false),
					getNextGaussDist(0, 2, false),
					1,
					cov);

		
		// "Real" camera  
		double[][] inner = { 
				{ 1000.0, 1000.0, 2 }, 
				{      0, 1000.0, 1 }, 
				{      0,      0, 1 }};

		Matrix inOrien = LinearAlgebraFactory.getMatrix(inner);
		Translation_3D trans =
			new Translation_3D(
				getNextGaussDist(1, 0.5, true),
				getNextGaussDist(1, 0.5, false),
				getNextGaussDist(1, 0.5, false));
		Rotation_3D rot =
			new Rotation_3D(
				getNextGaussDist(0, 1, false),
				getNextGaussDist(0, 1, false),	
				getNextGaussDist(0, 1, false));
		Motion_3D extOrien = new Motion_3D(trans, rot);
		ProjectiveCamera target = new ProjectiveCamera(inOrien, extOrien,
				LinearAlgebraFactory.getDiagMatrix(12,0.01));
		
		
		// Original lines
		Matrix cov6 =
			LinearAlgebraFactory.getDiagMatrix(6, sigmaobsLines * sigmaobsLines);
		Line_3D[] lines = new Line_3D[nr];
		Random rand = new Random();
		Point_3D projCenter = target.getProjectionCenter();
		for (int i = 0; i < nr; i++) {
			Point_3D P2;
			Point_3D P1;
			do {
				P1 =
					new Point_3D(
						-2+rand.nextDouble()*4,
						-2+rand.nextDouble()*4,
						-2+rand.nextDouble()*4,
						1,
						cov);
				
				P2 =
					new Point_3D(
						-2.5+rand.nextDouble()*5,
						-2.5+rand.nextDouble()*5,
						-2.5+rand.nextDouble()*5,
						1,
						cov);
				
				lines[i] = new Line_3D(P1, P2);
				//don't go through projection centre,
				//because then the projected line would be a point.
			} while (lines[i].isIncident(projCenter));
	
			
			
			lines[i].normalizeSphere();
			lines[i].ensureConstraints();
			lines[i].setCov(cov6);
		}

		// Transformed points
		Matrix cov5 =
			LinearAlgebraFactory.getDiagMatrix(3, sigmaobs * sigmaobs);
		cov5.set(2, 2, 0);
		Point_2D[] hpoints = new Point_2D[nr];
		for (int i = 0; i < nr; i++) {
			hpoints[i] = target.transform(points[i]);
			hpoints[i].normalizeEuclidean();
			hpoints[i].setCov(cov5);
		}

		// noise on points
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
		}

		// Transformed lines
		Matrix cov7 =
			LinearAlgebraFactory.getDiagMatrix(3, sigmaobsLines * sigmaobsLines);
		Line_2D[] hlines = new Line_2D[nr];
		for (int i = 0; i < nr; i++) {
			hlines[i] = target.transform(lines[i]);
			hlines[i].setCov(cov7);
		}

		// noise on lines
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobsLines, false);
			double r2 = getNextGaussDist(0, sigmaobsLines, false);
			double r3 = getNextGaussDist(0, sigmaobsLines, false);
			double r4 = getNextGaussDist(0, sigmaobsLines, false);
			double r5 = getNextGaussDist(0, sigmaobsLines, false);
			double r6 = getNextGaussDist(0, sigmaobsLines, false);

			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			lines[i].ensureConstraints();
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobsLines, false);
			double r2 = getNextGaussDist(0, sigmaobsLines, false);
			double r3 = getNextGaussDist(0, sigmaobsLines, false);

			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rprops[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					ProjectiveCamera.class);
		
		RelationalProperty[] rpropsPoints = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rpropsPoints[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					ProjectiveCamera.class);

		//estimation using points
		ProjectiveCamera est = new ProjectiveCamera();
		nowEstimate(target, est, rpropsPoints, "testProjectiveCamera - Points", 0.3);
		
		//estimation using lines
		est = new ProjectiveCamera();
		nowEstimate(target, est, rprops, "testProjectiveCamera - Lines", 0.3);
	}
	
	/**
	 * Tests Estimation of a ProjectiveCamera 
	 * with a fixed Situation, that can realy exist. 
	 */
	public static void testProjectiveCameraReal(){
		double sigmaobs = 0.01;
		int nrobs = 50;
		
		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D[] points = new Point_3D[nrobs];
		for (int i = 0; i < nrobs; i++)
			points[i] =
				new Point_3D(
					Math.random()*20,
					Math.random()*20,
					Math.random()*2,
					1,
					cov);
		
		// Original Lines
		Matrix cov6 = LinearAlgebraFactory.getDiagMatrix(6,sigmaobs*sigmaobs*0.01);
		
		Line_3D[] lines = new Line_3D[nrobs];
		for (int i = 0; i < nrobs; i++){
			Point_3D P1 =
				new Point_3D(
					Math.random()*20,
					Math.random()*20,
					Math.random()*2,
					1,
					cov);
			
			Point_3D P2 =
				new Point_3D(
					Math.random()*20,
					Math.random()*20,
					Math.random()*2,
					1,
					cov);
			
			lines[i] = new Line_3D(P1,P2);
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}
		
		//the real Camera
		double[][] inner = { 
				{ 2.0, 0.5, 2 }, 
				{   0, 2.0, 1 }, 
				{   0,   0, 1 }};

		Matrix inOrien = LinearAlgebraFactory.getMatrix(inner);
		Translation_3D trans = new Translation_3D(-10,-10,10);
		Rotation_3D rot = new Rotation_3D(
				Math.PI/4.0,
				Math.PI/4.0,
				1);
		Motion_3D outer = new Motion_3D(trans,rot);
		ProjectiveCamera target = new ProjectiveCamera(inOrien,outer);
		
		// Transformed points
		Matrix cov3 =
			LinearAlgebraFactory.getDiagMatrix(3, sigmaobs * sigmaobs);
		cov3.set(2, 2, 0);
		Point_2D[] hpoints = new Point_2D[nrobs];
		for (int i = 0; i < nrobs; i++) {
			hpoints[i] = target.transform(points[i]);
			hpoints[i].normalizeEuclidean();
			hpoints[i].setCov(cov3);
		}

		// noise on points
		for (int i = 0; i < nrobs; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nrobs; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
		}

		// Transformed lines
		Matrix cov3L = LinearAlgebraFactory.getDiagMatrix(3,sigmaobs*sigmaobs*0.01);
		Line_2D[] hlines = new Line_2D[nrobs];
		for (int i = 0; i < nrobs; i++) {
			hlines[i] = target.transform(lines[i]);
			hlines[i].setCov(cov3L);
		}
		
		// noise on lines
		for (int i = 0; i < nrobs; i++) {
			double r1 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r2 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r3 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r4 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r5 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r6 = getNextGaussDist(0, sigmaobs*0.01, false);

			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			lines[i].ensureConstraints();
		}
		for (int i = 0; i < nrobs; i++) {
			double r1 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r2 = getNextGaussDist(0, sigmaobs*0.01, false);
			double r3 = getNextGaussDist(0, sigmaobs*0.01, false);

			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nrobs];
		for (int i = 0; i < nrobs; i++)
			rprops[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					ProjectiveCamera.class);
		
		RelationalProperty[] rpropsPoints = new RelationalProperty[nrobs];
		for (int i = 0; i < nrobs; i++)
			rpropsPoints[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					ProjectiveCamera.class);

		//estimation using points
		ProjectiveCamera est = new ProjectiveCamera();
		nowEstimate(target, est, rpropsPoints, "testProjectiveCameraReal - Points", 0.3);
		
		//estimation using lines
		est = new ProjectiveCamera();
		nowEstimate(target, est, rprops, "testProjectiveCameraReal - Lines", 0.3);
	}

	
	/** 
	 * Do a prepared estimation of a transformation and print out result,
	 * if estimation "fails". 
	 * 
	 * @param target the correct transformation
	 * @param est the transformation where the estimation is done.
	 * @param rprops the relational properties for this transformation.
	 * @param functionName a name for the calling function, to be printed, if something fails.
 	 * @param percentage - percentage of tolerance accpeted in vector
	 */
	private static void nowEstimate(
		Transformation target,
		Transformation est,
		RelationalProperty[] rprops,
		String functionName, double percentage) {
		
		// And now go !
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] it = new int[1];
		boolean ok = est.estimate(rprops, true, 200, yhat, sigma0, it,true);

		target.normalizeSphereComparable();
		est.normalizeSphereComparable();
		if(!ok){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (" 
					+ functionName +")");
			System.out.println("Convergence not reached");
			
			System.out.println("No of iterations: " + it[0]);
			System.out.println("sigma0: " + sigma0[0]);
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			fail();
		}
		
		if(!compareEntities(target,est,percentage)){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (" 
					+ functionName +")");
			
			
			System.out.println("Convergence reached");
		
			System.out.println("No of iterations: " + it[0]);
			System.out.println("sigma0: " + sigma0[0]);
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			
			fail();
		}
	}
	
	/** 
	 * Test entity.ensureConstrains()
	 */
	public static void testConstraintFulfill() {
		double threshold = 0.0001;
		
		// Test a point
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3, 0.01);

		Point_2D p1 = new Point_2D(0.6, 0.5, cov);
		Point_2D p2 = p1.copy();

		p1.ensureConstraints();
		
		//length should be one
		if(Math.abs(p1.getV().getLength()-1) > threshold){
			System.out.println("=============================================");
			System.out.println("Test of ensureConstraints (Point_2D)");
			
			System.out.println("P1 before ensureConstraints:");
			p2.print(6, 6);
			System.out.println("P1 length:" + p2.getV().normF());
			
			System.out.println("P1 after ensureConstraints:");
			p1.print(6, 6);
			System.out.println("P1 length:" + p1.getV().normF());
			
			System.out.println("Normalize sphere:");
			p2.normalizeSphere();
			p2.print(6, 6); 
			
			fail();
		}
		
		// Test a Line3D
		Matrix cov1 = LinearAlgebraFactory.getDiagMatrix(6, 0.01);

		Line_3D L1 = new Line_3D(1, 1, 1, 0, -1, 1.2, cov1);
		Line_3D L2 = L1.copy();

		L1.ensureConstraints();
		
		//length should be one 
		//and Lh * Lo (* = scalarmult) should be zero.
		if(Math.abs(L1.getV().getLength()-1) > threshold || 
				Math.abs(L1.getHomogeneousPart().
					scalarmult(L1.getEuclideanPart())) > threshold){
			System.out.println("=============================================");
			System.out.println("Test of ensureConstraints (Line_3D)");
			
			System.out.println("L1 before ensureConstraints:");
			L2.print(6, 6);
			System.out.println("L1 length:" + L2.getV().normF());
			System.out.println(
				"Lh * Lo:"
					+ L2.getHomogeneousPart().scalarmult(L2.getEuclideanPart()));

			
			System.out.println("L1 after ensureConstraints:");
			L1.print(6, 6);
			System.out.println("L1 length:" + L1.getV().normF());
			System.out.println(
				"Lh * Lo:"
					+ L1.getHomogeneousPart().scalarmult(L1.getEuclideanPart()));

			L2.normalizeSphere();
			System.out.println("Normalize sphere:");
			L2.print(6, 6);
			
			fail();
		}

		// Test a point with different accuracies
		Matrix cov3 = LinearAlgebraFactory.getDiagMatrix(3, 0.01);
		cov3.set(0, 0, 0.1);

		Point_2D p3 = new Point_2D(0.6, 0.5, cov3);
		Point_2D p4 = p3.copy();

		p3.ensureConstraints();

		if(Math.abs(p3.getV().getLength()-1) > threshold){
			System.out.println("=============================================");
			System.out.println("Test of ensureConstraints (second Point_2D)");
			
			System.out.println("P3 before ensureConstraints:");
			p4.print(6, 6);
			System.out.println("P3 length:" + p4.getV().normF());
			
			System.out.println("P3 after ensureConstraints:");
			p3.print(6, 6);
			System.out.println("P3 length:" + p3.getV().normF());
			
			p4.normalizeSphere();
			System.out.println("Normalize sphere:");
			p4.print(6, 6);
			
			fail();
		}
	}
	
	/** 
	 * Test the conditioning methods for entitis.
	 */
	public static void testConditioningEntity() {
		//check wether conditioning is neccessary or not
		Point_2D np1, np2;
		Point_2D np1Org, np2Org;
		
		//A point which doesn't need to be conditioned:
		np1=new Point_2D(0.09, 0);
		np1Org=(Point_2D)np1.clone();
		
		Entity [] es1 = {np1};
		Entity.doCondition(es1);
		
		//Entities should be equal
		if(!TestStatistics.check(np1,np1Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check whether conditioning is neccessary " +
					"or not (testConditioningEntity)");
			System.out.println("A point which doesn't need to be conditioned:");
			System.out.println("Before: "+np1Org.getV());
			System.out.println("After: "+np1.getV());
			
			fail();
		}
		
		
		//A point which need to be conditioned:
		np2=new Point_2D(2, 0);
		np2Org = (Point_2D)np2.clone();
		
		Entity [] es2 = {np2};
		Entity.doCondition(es2);
		
		//Entities shouldn't be eqaul
		if(TestStatistics.check(np2,np2Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check whether conditioning is neccessary " +
					"or not (testConditioningEntity)");
			System.out.println("A point which need to be conditioned:");
			System.out.println("Before: "+np2Org.getV());
			System.out.println("After: "+np2.getV());
			
			fail();
		}
		
		//Both points (need to be conditioned!)
		np1=new Point_2D(0.09, 0);
		np2=new Point_2D(2, 0);
		np1Org=(Point_2D)np1.clone();
		np2Org=(Point_2D)np2.clone();
		
		Entity [] es3 = {np1,np2};
		Entity.doCondition(es3);
		
		//Entities shouldn't be equal
		if(TestStatistics.check(np1,np1Org,0.1) 
				|| TestStatistics.check(np2,np2Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check whether conditioning is neccessary " +
					"or not (testConditioningEntity)");
			System.out.println("Both points (need to be conditioned!):");
			System.out.println("Before: "+np1Org.getV()+np2Org.getV());
			System.out.println("After: "+np1.getV()+np2.getV());
			
			fail();
		}
		
		//Two points which both need to be conditioned:
		np1=new Point_2D(20, 0);
		np2=new Point_2D(2, 0);
		np1Org=(Point_2D)np1.clone();
		np2Org=(Point_2D)np2.clone();

		Entity [] es4 = {np1,np2};
		Entity.doCondition(es4);

		//Entities shouldn't be equal
		if(TestStatistics.check(np1,np1Org,0.1) 
				|| TestStatistics.check(np2,np2Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check whether conditioning is neccessary " +
					"or not (testConditioningEntity)");
			System.out.println("Two points which both need to be conditioned:");
			System.out.println("Before: "+np1Org.getV()+np2Org.getV());
			System.out.println("After: "+np1.getV()+np2.getV());
			
			fail();
		}

		//Check correctness of conditioning and reconditioning with covariances
		Matrix cov1 = LinearAlgebraFactory.getDiagMatrix(3, 0.1);
		Matrix cov2 = LinearAlgebraFactory.getDiagMatrix(3, 0.1);

		Line_2D l1 = new Line_2D(3, 2, 42312, cov1);
		Point_2D p1 = new Point_2D(312, -2334432, 2, cov2);
		Line_2D l1Org = (Line_2D)l1.clone();
		Point_2D p1Org = (Point_2D)p1.clone();
		
		Entity[] elist = new Entity[2];
		elist[0] = l1;
		elist[1] = p1;

		double f1 = Entity.getConditionFactorF(l1, p1);
		double f2 = Entity.getConditionFactorF(elist);

		//conditioning factors should be equal
		if(f1 != f2) {
			System.out.println("f1: " + f1 + "   f2:" + f2);
			fail();
		}
		
		l1.doCondition(f1);
		p1.doCondition(f1);
		
		//Entities shouldn't be equal
		if(TestStatistics.check(p1,p1Org,0.1) 
				|| TestStatistics.check(l1,l1Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check correctness of conditioning and reconditioning " +
					"with covariances (testConditioningEntity)");
			
			System.out.println("Original values:");
			System.out.println("l1: " + l1Org);
			System.out.println("p1: " + p1Org);
			
			System.out.println("After conditioning:");
			System.out.println("l1: " + l1);
			System.out.println("p1: " + p1);

			fail();
		}
			
		//reconditioning
		l1.doCondition(1 / f1);
		p1.doCondition(1 / f1);

		//Entities should be equal again.
		if(!TestStatistics.check(p1,p1Org,0.1) 
				|| !TestStatistics.check(l1,l1Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check correctness of conditioning and reconditioning " +
					"with covariances (testConditioningEntity)");
			
			System.out.println("Original values:");
			System.out.println("l1: " + l1Org);
			System.out.println("p1: " + p1Org);
			
			System.out.println("After reconditioning:");
			System.out.println("l1: " + l1);
			System.out.println("p1: " + p1);

			fail();
		}
		
		//conditioning as list
		Entity.doCondition(elist);

		//Entities shouldn't be equal
		if(TestStatistics.check(p1,p1Org,0.1) 
				|| TestStatistics.check(l1,l1Org,0.1)){
			System.out.println("=============================================");
			System.out.println("Check correctness of conditioning and reconditioning " +
					"with covariances (testConditioningEntity)");
			
			System.out.println("Original values:");
			System.out.println("l1: " + l1Org);
			System.out.println("p1: " + p1Org);
			
			System.out.println("After conditioning with 1 method:");
			System.out.println("l1: " + l1);
			System.out.println("p1: " + p1);

			fail();
		}
	}
	
	/**
	 * Test Motion_3D estimation
	 * 
	 */
	public static void testMotion_3DEstimation(){
		int nr = 300;
		double sigmaobs_lines = 0.00001;
		double sigmaobs = 0.01;

		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D[] points = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			points[i] =
				new Point_3D(
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					1,
					cov);

		// Original lines
		Line_3D[] lines = new Line_3D[nr];
		Matrix cov6 =
			LinearAlgebraFactory.getDiagMatrix(
				6,
				sigmaobs_lines * sigmaobs_lines);

		for (int i = 0; i < nr; i++) {
			Point_3D P1 =
				new Point_3D(
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					1,
					cov);
			Point_3D P2 =
				new Point_3D(
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					1,
					cov);

			lines[i] = new Line_3D(P1, P2);
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}

		// Original planes
		Plane_3D[] planes = new Plane_3D[nr];
		Matrix cov4 =
			LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, 10, false);
			double r2 = getNextGaussDist(0, 10, false);
			double r3 = getNextGaussDist(0, 10, false);
			double r4 = getNextGaussDist(0, 10, false);
			planes[i] = new Plane_3D(r1, r2, r3, r4, cov4);
		}

		//the real Motion
		Motion_3D target = new Motion_3D(new Translation_3D(5,6,7)
				,new Rotation_3D(Math.PI/2,Math.PI/5,3*Math.PI/4)
				,LinearAlgebraFactory.getDiagMatrix(16,0.0001));
		
		// Transformed points
		Point_3D[] hpoints = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			hpoints[i] = (Point_3D) target.transform(points[i]);

		// noise on points
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Transformed lines
		Line_3D[] hlines = new Line_3D[nr];
		for (int i = 0; i < nr; i++) {
			hlines[i] = (Line_3D) target.transform(lines[i]);
			hlines[i].normalizeSphere();
			hlines[i].setCov(cov6);
		}

		// noise on lines
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			lines[i].normalizeSphere();
			lines[i].ensureConstraints();
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			hlines[i].normalizeSphere();
			hlines[i].ensureConstraints();
		}

		// Transformed planes
		Plane_3D[] hplanes = new Plane_3D[nr];
		for (int i = 0; i < nr; i++) {
			hplanes[i] = (Plane_3D) target.transform(planes[i]);
			hplanes[i].setCov(cov4);
		}

		// Noise on planes
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			double r4 = getNextGaussDist(0, sigmaobs, false);
			Vector v = planes[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			double r4 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hplanes[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
		}

		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rprops[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					Motion_3D.class);
			
		RelationalProperty[] rpropsPoints = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rpropsPoints[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					Motion_3D.class);
	
		RelationalProperty[] rpropsPlanes = new RelationalProperty[nr];

		for (int i = 0; i < nr; i++)
				rpropsPlanes[i] =
					new TriRelationalProperty(
						planes[i],
						hplanes[i],
						Motion_3D.class);

		//estimate using points
		Motion_3D est = new Motion_3D();
		nowEstimate(target, est, rpropsPoints, "testMotion_3D - Points", 0.3);
		
		//estimate using planes
		est = new Motion_3D();
		nowEstimate(target, est, rpropsPlanes, "testMotion_3D - Planes", 0.3);

		//estimate using lines
		est = new Motion_3D();
		nowEstimate(target, est, rprops, "testMotion_3D - Lines", 0.3);
	}
	
	/**
	 * Test estimation of a Motion_3D using 6 points
	 */
	public static void testMotion_3DFewPoints(){
		double[][] val = {	{0.000001,0,0,0},
							{0,0.000001,0,0},
							{0,0,0.000001,0},
							{0,0,0,0.000001}};
		Matrix cov = LinearAlgebraFactory.getMatrix(val);
		
		Point_3D p1 = new Point_3D(100,0,0,cov);
		Point_3D p2 = new Point_3D(0,100,0,cov);
		Point_3D p3 = new Point_3D(0,0,100,cov);
		Point_3D p4 = new Point_3D(100,100,0,cov);
		Point_3D p5 = new Point_3D(0,100,100,cov);
		Point_3D p6 = new Point_3D(100,100,100,cov);
		
		Motion_3D target = new Motion_3D(new Translation_3D(1,-2,5),
				new Rotation_3D(Math.PI/2,1.2,-0.5),
				LinearAlgebraFactory.getDiagMatrix(16,0.001));
		target.normalizeSphere();
		
		Point_3D[] pStrich = new Point_3D[6];
		pStrich[0] = (Point_3D)target.transform(p1);
		pStrich[1] = (Point_3D)target.transform(p2);
		pStrich[2] = (Point_3D)target.transform(p3);
		pStrich[3] = (Point_3D)target.transform(p4);
		pStrich[4] = (Point_3D)target.transform(p5);
		pStrich[5] = (Point_3D)target.transform(p6);
		
		//add noise
		Random ran = new Random();
		for(int i = 0; i < pStrich.length; i++){
			double[] noise = {
					ran.nextGaussian()*0.001,
					ran.nextGaussian()*0.001,
					ran.nextGaussian()*0.001,
					ran.nextGaussian()*0.001};
			pStrich[i].setV((pStrich[i].getV())
					.plus(LinearAlgebraFactory.getVector(noise)));
		}
		
		RelationalProperty[] rprops = new RelationalProperty[6];
		rprops[0] = new TriRelationalProperty(p1,pStrich[0],Motion_3D.class);
		rprops[1] = new TriRelationalProperty(p2,pStrich[1],Motion_3D.class);
		rprops[2] = new TriRelationalProperty(p3,pStrich[2],Motion_3D.class);
		rprops[3] = new TriRelationalProperty(p4,pStrich[3],Motion_3D.class);
		rprops[4] = new TriRelationalProperty(p5,pStrich[4],Motion_3D.class);
		rprops[5] = new TriRelationalProperty(p6,pStrich[5],Motion_3D.class);
		
		//estimate
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] iterations = new int[1];
		Motion_3D est = new Motion_3D();
		
		boolean ok = est.estimate(rprops,true,200,yhat,sigma0,iterations,true);
		if(!ok){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testMotion_3DFewPoints)");
				
			System.out.println("Convergence not reached.");			
			System.out.println("Expected Result: " + target.getTransformationMatrix());
			System.out.println("Estimation Result: " + est.getTransformationMatrix());
			
			System.out.println("Sigma0: " + sigma0);
			fail();
		}
		
		target.normalizeSphereComparable();
		est.normalizeSphereComparable();
		
		if(!compareEntities(target,est,0.4)){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testMotion_3DFewPoints)");
			
			System.out.println("Convergence reached");
			System.out.println("No of iterations: " + iterations);
			System.out.println("sigma0: " + sigma0[0]);
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			
			fail();
		}
		
		pStrich[0].normalizeEuclidean();
		Point_3D p1Strich2 = (Point_3D)est.transform(p1);
		p1Strich2.normalizeEuclidean();
		
		if(!TestStatistics.check(pStrich[0].getV(),p1Strich2.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testMotion_3DFewPoints)");
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			
			System.out.println("Point transformed with original transformation: " + pStrich[0]);
			System.out.println("Point transformed with estimated transformation: " + p1Strich2);
			
			fail();
		}
		

	}

	
	/**
	 * Test ScaledMotion_3D estimation
	 */
	public static void testScaledMotion_3DEstimation(){
		int nr = 300;
		double sigmaobs_lines = 0.00001;
		double sigmaobs = 0.001;

		random = new Random();
		
		// Original points
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov.set(3, 3, 0);

		Point_3D[] points = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			points[i] =
				new Point_3D(
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					getNextGaussDist(0, 50, false),
					1,
					cov);

		// Original lines
		Line_3D[] lines = new Line_3D[nr];
		Matrix cov6 =
			LinearAlgebraFactory.getDiagMatrix(
				6,
				sigmaobs_lines * sigmaobs_lines);

		for (int i = 0; i < nr; i++) {
			Point_3D P1 =
				new Point_3D(
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					1,
					cov);
			Point_3D P2 =
				new Point_3D(
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					getNextGaussDist(0, 40, false),
					1,
					cov);

			lines[i] = new Line_3D(P1, P2);
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}

		// Original planes
		Plane_3D[] planes = new Plane_3D[nr];
		Matrix cov4 =
			LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, 5, false);
			double r2 = getNextGaussDist(0, 5, false);
			double r3 = getNextGaussDist(0, 5, false);
			double r4 = getNextGaussDist(0, 5, false);
			planes[i] = new Plane_3D(r1, r2, r3, r4, cov4);
		}

		//the real Motion
		ScaledMotion_3D target = new ScaledMotion_3D(new Translation_3D(5,6,7)
				,new Rotation_3D(Math.PI/2,Math.PI/5,3*Math.PI/4),5,
				LinearAlgebraFactory.getDiagMatrix(16,0.0001));
		
		// Transformed points
		Point_3D[] hpoints = new Point_3D[nr];
		for (int i = 0; i < nr; i++)
			hpoints[i] = (Point_3D) target.transform(points[i]);

		// noise on points
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = points[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hpoints[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
		}

		// Transformed lines
		Line_3D[] hlines = new Line_3D[nr];
		for (int i = 0; i < nr; i++) {
			hlines[i] = (Line_3D) target.transform(lines[i]);
			lines[i].normalizeSphere();
			lines[i].setCov(cov6);
		}

		// noise on lines
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = lines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			lines[i].normalizeSphere();
			lines[i].ensureConstraints();
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs_lines, false);
			double r2 = getNextGaussDist(0, sigmaobs_lines, false);
			double r3 = getNextGaussDist(0, sigmaobs_lines, false);
			double r4 = getNextGaussDist(0, sigmaobs_lines, false);
			double r5 = getNextGaussDist(0, sigmaobs_lines, false);
			double r6 = getNextGaussDist(0, sigmaobs_lines, false);

			Vector v = hlines[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
			v.set(4, v.get(4) + r5);
			v.set(5, v.get(5) + r6);

			hlines[i].normalizeSphere();
			hlines[i].ensureConstraints();
		}

		// Transformed planes
		Plane_3D[] hplanes = new Plane_3D[nr];
		for (int i = 0; i < nr; i++) {
			hplanes[i] = (Plane_3D) target.transform(planes[i]);
			hplanes[i].setCov(cov4);
		}

		// Noise on planes
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			double r4 = getNextGaussDist(0, sigmaobs, false);
			Vector v = planes[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
		}
		for (int i = 0; i < nr; i++) {
			double r1 = getNextGaussDist(0, sigmaobs, false);
			double r2 = getNextGaussDist(0, sigmaobs, false);
			double r3 = getNextGaussDist(0, sigmaobs, false);
			double r4 = getNextGaussDist(0, sigmaobs, false);
			Vector v = hplanes[i].getV();
			v.set(0, v.get(0) + r1);
			v.set(1, v.get(1) + r2);
			v.set(2, v.get(2) + r3);
			v.set(3, v.get(3) + r4);
		}

		// Relational properties
		RelationalProperty[] rprops = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rprops[i] =
				new TriRelationalProperty(
					lines[i],
					hlines[i],
					ScaledMotion_3D.class);
			
		RelationalProperty[] rpropsPoints = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
			rpropsPoints[i] =
				new TriRelationalProperty(
					points[i],
					hpoints[i],
					ScaledMotion_3D.class);
	
		RelationalProperty[] rpropsPlanes = new RelationalProperty[nr];
		for (int i = 0; i < nr; i++)
				rpropsPlanes[i] =
					new TriRelationalProperty(
						planes[i],
						hplanes[i],
						ScaledMotion_3D.class);

		//estimate using points
		ScaledMotion_3D est = new ScaledMotion_3D();
		nowEstimate(target, est, rpropsPoints, "testScaledMotion_3D - Points", 0.3);
		
		//estimate using lines - sometimes fails, because of bad random observations
		est = new ScaledMotion_3D();
		nowEstimate(target, est, rprops, "testScaledMotion_3D - Lines", 0.3);
		
		//estimate using planes
		est = new ScaledMotion_3D();
		nowEstimate(target, est, rpropsPlanes, "testScaledMotion_3D - Planes", 0.3);
	}
	
	/**
	 * Test estimation of a ScaledMotion_3D using 9 points.
	 */
	public static void testScaledMotion_3DFewPoints(){
		double[][] val = {	{0.000001,0,0,0},
							{0,0.000001,0,0},
							{0,0,0.000001,0},
							{0,0,0,0.000001}};
		Matrix cov = LinearAlgebraFactory.getMatrix(val);
		
		Point_3D p1 = new Point_3D(100,0,0,cov);
		Point_3D p2 = new Point_3D(0,100,0,cov);
		Point_3D p3 = new Point_3D(0,0,100,cov);
		Point_3D p4 = new Point_3D(100,100,0,cov);
		Point_3D p5 = new Point_3D(0,100,100,cov);
		Point_3D p6 = new Point_3D(100,100,100,cov);
		Point_3D p7 = new Point_3D(100,0,100,cov);
		Point_3D p8 = new Point_3D(-100,100,0,cov);
		Point_3D p9 = new Point_3D(100,0,-100,cov);
		
		ScaledMotion_3D target = new ScaledMotion_3D(new Translation_3D(1,-2,5),
				new Rotation_3D(Math.PI/2,1.2,-0.5),6,LinearAlgebraFactory.getDiagMatrix(16,0.001));
		target.normalizeSphere();

		Point_3D[] pStrich = new Point_3D[9];
		pStrich[0] = (Point_3D)target.transform(p1);
		pStrich[1] = (Point_3D)target.transform(p2);
		pStrich[2] = (Point_3D)target.transform(p3);
		pStrich[3] = (Point_3D)target.transform(p4);
		pStrich[4] = (Point_3D)target.transform(p5);
		pStrich[5] = (Point_3D)target.transform(p6);
		pStrich[6] = (Point_3D)target.transform(p7);
		pStrich[7] = (Point_3D)target.transform(p8);
		pStrich[8] = (Point_3D)target.transform(p9);
		
		//verrauschen
		Random ran = new Random();
		for(int i = 0; i < pStrich.length; i++){
			double[] noise = {
					ran.nextGaussian()*0.001,
					ran.nextGaussian()*0.001,
					ran.nextGaussian()*0.001,
					0};
			pStrich[i].setV((pStrich[i].getV())
					.plus(LinearAlgebraFactory.getVector(noise)));
		}
		
		RelationalProperty[] rprops = new RelationalProperty[9];
		rprops[0] = new TriRelationalProperty(p1,pStrich[0],ScaledMotion_3D.class);
		rprops[1] = new TriRelationalProperty(p2,pStrich[1],ScaledMotion_3D.class);
		rprops[2] = new TriRelationalProperty(p3,pStrich[2],ScaledMotion_3D.class);
		rprops[3] = new TriRelationalProperty(p4,pStrich[3],ScaledMotion_3D.class);
		rprops[4] = new TriRelationalProperty(p5,pStrich[4],ScaledMotion_3D.class);
		rprops[5] = new TriRelationalProperty(p6,pStrich[5],ScaledMotion_3D.class);
		rprops[6] = new TriRelationalProperty(p7,pStrich[6],ScaledMotion_3D.class);
		rprops[7] = new TriRelationalProperty(p8,pStrich[7],ScaledMotion_3D.class);
		rprops[8] = new TriRelationalProperty(p9,pStrich[8],ScaledMotion_3D.class);
		
		ScaledMotion_3D est = new ScaledMotion_3D();
		
		//estimate
		boolean ok = est.estimate(rprops);
		if(!ok){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testScaledMotion_3DFewPoints)");
				
			System.out.println("Convergence not reached.");			
			System.out.println("Expected Result: " + target.getTransformationMatrix());
			System.out.println("Estimation Result: " + est.getTransformationMatrix());	
			fail();
		}
		
		target.normalizeSphereComparable();
		est.normalizeSphereComparable();
		
		if(!compareEntities(target,est,0.4)){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testScaledMotion_3DFewPoints)");
			
			
			System.out.println("Convergence reached");
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			
			fail();
		}
		
		pStrich[0].normalizeEuclidean();
		Point_3D p1Strich2 = (Point_3D)est.transform(p1);
		p1Strich2.normalizeEuclidean();
		
		if(!TestStatistics.check(pStrich[0].getV(),p1Strich2.getV(),0.3)){
			System.out.println("=============================================");
			System.out.println("Estimation of a " + est.getClass() + " (testScaledMotion_3DFewPoints)");
			
			System.out.println("Expected Result:" + target.getTransformationMatrix());
			System.out.println("Estimation:" + est.getTransformationMatrix());
			
			System.out.println("Point transformed with original transformation: " + pStrich[0]);
			System.out.println("Point transformed with estimated transformation: " + p1Strich2.getV());
			
			fail();
		}
	}

	/**
	 * Simulates an easy test to check cov-matrix
	 */
	public static void testSimulationOfTest() {
		int MAXITER = 500;
		int nrobs = 100;
		
		random = new Random();
		
		//cov-Matrix
		double[][] covVal = {{0.01,0.0000,0.0000,0},
							 {0.0000,0.01,0.0000,0},
							 {0.0000,0.0000,0.01,0},
							 {0,0,0,0}};
		
		Matrix cov = LinearAlgebraFactory.getMatrix(covVal);
		
		//some points
		Point_3D[] points = new Point_3D[nrobs];
		//the properties for estimation
		RelationalProperty[] rprops = new RelationalProperty[nrobs];
		Point_3D est;
		Point_3D[] estimations = new Point_3D[MAXITER];
		
		//Statistics Object
		TestStatistics stat = new TestStatistics(4);
		
		//random Vectors for simulation of noise
		RandomVector[] randoms = new RandomVector[nrobs];
		
		//make points, randomly distributed, and save them 
		//in the random Vectors for simulation
		for(int i = 0; i < nrobs; i++)
		{
			//// RANDOMS AN DIESER STELLE NICHT MEHR BENOETIGT !!! TL
			points[i] = new Point_3D(1,0.5,-1,cov);
			randoms[i] = new RandomVector(points[i].getV(),cov);
		}
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Simulation of an estimation","",0,MAXITER);
		
		
		//now iterate
		for(int i = 0; i < MAXITER; i++) {
			//generate the properties with noise on points
			for(int j = 0; j < randoms.length; j++){
				Point_3D p = new Point_3D(randoms[j].getNextRandom(),cov);
				rprops[j] = new Equal(p);
			}
			
			//estimate and save as observation
			est = new Point_3D();
			est.estimate(rprops);
			est.normalizeEuclidean();
			estimations[i] = est;
			
			stat.addObservation(est);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//save results of simulation
		Point_3D simulated = new Point_3D(stat.getMue(),stat.getSigma());
		
		//normalize
		simulated.normalizeEuclidean();
		
		for(int i = 0; i < MAXITER; i++){
			//check results
			if(!TestStatistics.check(simulated.getV(),estimations[i].getV(),0.3)
					&& testCov(estimations[i].getCov(),cov.mult(1.0/nrobs),0.3,10e-6))
			{
				System.out.println("=============================================");
				System.out.println("Simulation of a Test to test cov (" 
						+ "testSimulationOfTest)");
				System.out.println("Simulation failed.\n");
				
				System.out.println("Estimate a Point_3D out of " + nrobs + " points:");
				System.out.println("SUGR result: " + estimations[i]);
				System.out.println("Simulation result: " + simulated);
				
				fail();
			}
		}
	}
	
	/**
	 * Simulates an easy test to check cov-matrix
	 */
	public static void testSimulationOfTest2D() {
		int MAXITER = 500;
		int nrobs = 100;
		
		random = new Random();
		
		//cov-Matrix
		double[][] covVal = {{0.01,0.0000,0},
							 {0.0000,0.01,0},
							 {0,0,0}};
		
		Matrix cov = LinearAlgebraFactory.getMatrix(covVal);
		
		//some points
		Point_2D[] points = new Point_2D[nrobs];
		//the properties for estimation
		RelationalProperty[] rprops = new RelationalProperty[nrobs];
		Point_2D est;
		Point_2D[] estimations = new Point_2D[MAXITER];
		
		//Statistics Object
		TestStatistics stat = new TestStatistics(3);
		
		//random Vectors for simulation of noise
		RandomVector[] randoms = new RandomVector[nrobs];
		
		//make points, randomly distributed, and save them 
		//in the random Vectors for simulation
		for(int i = 0; i < nrobs; i++)
		{
			points[i] = new Point_2D(1,-1,cov);
			randoms[i] = new RandomVector(points[i].getV(),cov);
		}
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Simulation of an estimation","using 2D Points",0,MAXITER);
		
		
		//now iterate
		for(int i = 0; i < MAXITER; i++) {
			//generate the properties with noise on points
			for(int j = 0; j < randoms.length; j++){
				Point_2D p = new Point_2D(randoms[j].getNextRandom(),cov);
				rprops[j] = new Equal(p);
			}
			
			//estimate and save as observation
			est = new Point_2D();
			est.estimate(rprops);
			est.normalizeEuclidean();
			estimations[i] = est;
			
			stat.addObservation(est);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//save results of simulation
		Point_2D simulated = new Point_2D(stat.getMue(),stat.getSigma());
		
		//normalize
		simulated.normalizeEuclidean();
		
		for(int i = 0; i < MAXITER; i++){
			//check results
			if(!TestStatistics.check(simulated.getV(),estimations[i].getV(),0.3)
					&& testCov(estimations[i].getCov(),simulated.getCov(),0.3,10e-6))
			{
				System.out.println("=============================================");
				System.out.println("Simulation of a Test to test cov (" 
						+ "testSimulationOfTest2D)");
				System.out.println("Simulation failed.\n");
				
				System.out.println("Estimate a Point_2D out of " + nrobs + " points:");
				System.out.println("SUGR result: " + estimations[i]);
				System.out.println("Simulation result: " + simulated);
				
				fail();
			}
		}
	}
	
	/**
	 * Test conditioning
	 * 
	 * Problem: The estimation needs the Center of Gravity near the 
	 * Origin. If not we need conditioning. 
	 * Scenario: given are some points near the origin, within [-0.1,0.1] on 
	 * all three axis. If we scale the points equally in all directions we need
	 * to switch conditioning on when we use a scale factor arround 10. 
	 * That means, values arround 1 allready need to be conditioned!
	 * 
	 * If we move the points all together into one direction conditioning is
	 * needed when moved ca 0.8. so, again, condidioning is needed with values 
	 * arround 1.
	 * 
	 * If we now move the center of gravity FAR away from the origin,
	 * conditioning just scales them down to the origin. 
	 * So the points will be close to each other.
	 * We get problems when the values are arround 10*10^14. Then you can't use
	 * conditioning anymore, but have to move the center of gravity closer to 
	 * the origin. 
	 */
	public static void conditioningTest() {
		int iterations = 15;
		int nrobs = 10;
		double scaleStep = 1.2; //greater than 1 to scale up
		double moveStep = 0.2;
		double moveStepConditioning = 10.0;
		
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4,0.1*0.1);
		cov.set(3,3,0);
		
		//initialization of arrays.
		Point_3D[] p = new Point_3D[nrobs];	
		Point_3D[] scaled = new Point_3D[nrobs];
		Point_3D[] moved = new Point_3D[nrobs];
		
		RelationalProperty[] rprops = new RelationalProperty[nrobs];
		Point_3D est = new Point_3D();
		Point_3D estScaled = new Point_3D();
		Point_3D estMoved = new Point_3D();
		Point_3D estMovedCond = new Point_3D();
		
		//reset random
		random = new Random();
		                                       
		//generate Relationalproperties
		for(int i = 0; i < nrobs; i++){
			double r1 = Math.random()/10;
			double r2 = Math.random()/10;
			double r3 = Math.random()/10;
		
			Vector noise = LinearAlgebraFactory.getVector(r1,r2,r3,1);
		
			//Vector v = target.getV();
			p[i] = new Point_3D(noise,cov);
			rprops[i] = new Equal(p[i]);
		}
		
		//estimate the small point and plane
		RelationalProperty[] yhat = new RelationalProperty[rprops.length];
		double[] sigma0 = new double[1];
		int[] it = new int[1];
		boolean ok = est.estimate(rprops,false,200,yhat,sigma0,it,true);
		est.normalizeEuclideanComparable();
		
		double factor = 1.0;
		double factorMove = 0.0;
		double factorMoveCond = 1.0;
		for(int j = 0; j < iterations; j++){
			System.out.println("=============================================");
			System.out.println("Test Conditioning (" 
					+ "testConditioning)");
			
			System.out.println("Original Estimation reached convergence: " + ok);
			System.out.println("Value: " + est);
			
			System.out.println("Iteration: " + j + "\n");
			
			factor *= scaleStep;
			
			estScaled = new Point_3D();
			estMoved = new Point_3D();
			estMovedCond = new Point_3D();
			
			try {
				//now scale Points far away from center.
				for(int i = 0; i < nrobs; i++){					
					Vector tmp = p[i].getV();
					Vector v = tmp.mult(factor);
					v.set(3,1);
					
					scaled[i] = new Point_3D(v,cov);
					rprops[i] = new Equal(scaled[i]);
				}
				
				//estimate the scaled point
				yhat = new RelationalProperty[rprops.length];
				sigma0 = new double[1];
				it = new int[1];
				boolean ok2 = estScaled.estimate(rprops, false, 200, yhat, sigma0, it,true);
				
				estScaled.normalizeEuclideanComparable();
				System.out.println("\nScaled Estimation reached convergence: " + ok2);
				System.out.println("Scale Factor: " + factor);
				System.out.println("Value: " + estScaled);		
			} catch(RuntimeException e)
			{
				System.out.println("\nScaled Estimation failed with exception.");
				System.out.println(e);
			}
			
			try {
				factorMove += moveStep;
				//now Move Points far away from center
				Translation_3D move = new Translation_3D(factorMove,factorMove,factorMove);
				for(int i = 0; i < nrobs; i++){
					moved[i] = (Point_3D)move.transform(p[i]);
					moved[i] = new Point_3D(moved[i].getV(),cov);
					rprops[i] = new Equal(moved[i]);
				}

				//estimate the moved point
				yhat = new RelationalProperty[rprops.length];
				sigma0 = new double[1];
				it = new int[1];
				boolean ok3 = estMoved.estimate(rprops, false, 200, yhat, sigma0, it,true);
				
				estMoved.normalizeEuclideanComparable();
				System.out.println("\nMoved Estimation reached convergence: " + ok3);
				System.out.println("Move Factor: " + factorMove);
				System.out.println("Value: " + estMoved);
			}
			catch(RuntimeException e){
				System.out.println("\nMoved Estimation failed with exception.");
				System.out.println(e);
			}
			try {
				factorMoveCond *= moveStepConditioning;
				//now Move Points far away from center
				Translation_3D move = new Translation_3D(factorMoveCond,
							factorMoveCond,factorMoveCond);
				for(int i = 0; i < nrobs; i++){
					moved[i] = (Point_3D)move.transform(p[i]);
					moved[i] = new Point_3D(moved[i].getV(),cov);
					rprops[i] = new Equal(moved[i]);
				}

				//estimate the moved point with conditioning switched on
				yhat = new RelationalProperty[rprops.length];
				sigma0 = new double[1];
				it = new int[1];
				boolean ok4 = estMovedCond.estimate(rprops,true,200,yhat,sigma0,it,true);
				
				estMovedCond.normalizeEuclideanComparable();
				System.out.println("\nMoved Estimation with conditioning reached " +
						"convergence: " + ok4);
				System.out.println("Move Factor: " + factorMoveCond);
				System.out.println("Value: " + estMovedCond);
			}
			catch(RuntimeException e){
				System.out.println("\nMoved Estimation with conditioning failed with exception.");
				System.out.println(e);
			}
		}
	}
	
	/** 
	 * Test the conditioning of transformations. 
	 */
	public static void testConditioningTransformation() {
		double sigmaobs = 0.1;

		Matrix cov4 =
			LinearAlgebraFactory.getDiagMatrix(4, sigmaobs * sigmaobs);
		cov4.set(3, 3, 0.002);
		Point_3D P = new Point_3D(200, 300, 400, cov4);

		// Projective Camera
		Matrix cov12 =
			LinearAlgebraFactory.getDiagMatrix(12, sigmaobs * sigmaobs);
		double[] harray =
			{ 10, 20, 30, 34, -10, -20, -30, -21, 1.1, 1.2, 1.3, 1 };
		Transformation target = new ProjectiveCamera(harray, cov12);

		printOutCondition(P, target, "Projective Camera");

		// Homography 2d
		Matrix cov3 =
			LinearAlgebraFactory.getDiagMatrix(3, sigmaobs * sigmaobs);
		cov3.set(2, 2, 0.002);

		Point_2D p = new Point_2D(312, 12, 1, cov3);
		Matrix cov9 =
			LinearAlgebraFactory.getDiagMatrix(9, sigmaobs * sigmaobs);
		double[] h2d = {10, -10, -21, 20, -11, 1, 0.30, -0.12, 1};
		target = new Homography_2D(h2d, cov9);
		
		printOutCondition(p, target, "Homography 2D");

		// Homography 3d
		Matrix cov16 =
			LinearAlgebraFactory.getDiagMatrix(16, sigmaobs * sigmaobs);
		double[] h3d =
			{10, -10, -21, 1, 20, -11, 1, 2, 30, -12, 2, 3, 2, 0.3, 0.1, 1};
		target = new Homography_3D(h3d, cov16);
		printOutCondition(P, target,"Homography 3D");
	}
	
	

	/**
	 * Used to help testConditioningTransformation
	 * @param P 
	 * @param target
	 */
	private static void printOutCondition(Entity P, Transformation target,
			String functionName) {
		Entity p = target.transform(P);
		Entity unconP = (Entity)P.clone();
		Entity unconp = (Entity)p.clone();
		Transformation unconTarget = (Transformation)target.clone();

		//conditioning
		TriRelationalProperty r =
			new TriRelationalProperty(P, p, target.getClass());
		RelationalProperty[] rprops = new RelationalProperty[1];
		rprops[0] = r;
		double[] f;
		f = TriRelationalProperty.doCondition(rprops);

		target.doCondition(f);
		
		Transformation conTarget = (Transformation)target.clone();

		//test transformation
		Entity ptest = target.transform(P);
		ptest.normalizeEuclidean();

		Entity pn = (Entity) p.clone();
		pn.normalizeEuclidean();
		
		
		//reconditioning
		f[0] = 1.0 / f[0];
		f[1] = 1.0 / f[1];

		target.doCondition(f);
		TriRelationalProperty.doCondition(rprops, f);
		
		//fail if original and reconditioned entities differ
		if(!TestStatistics.check(p,unconp,0.1)
				|| !TestStatistics.check(P,unconP,0.1) 
				|| !TestStatistics.check(target.getV(),unconTarget.getV(),
						target.getCov(),unconTarget.getCov(),0.1)
				|| !TestStatistics.check(ptest,pn,0.1)){
			System.out.println("=============================================");
			System.out.println("Test of Conditioning for Transformations (" 
					+ functionName +")");
			
			System.out.println(
					"Before conditioning: "
						+ unconP
						+ " "
						+ unconp
						+ " Transformation:"
						+ unconTarget.getTransformationMatrix()
						+ " "
						+ target.getCov());
			
			System.out.println("Conditioning factors: " + f[0] + "  " + f[1]);
			target.doCondition(f);
			
			System.out.println(
					"After conditioning: "
						+ P
						+ " "
						+ p
						+ " "
						+ conTarget.getTransformationMatrix()
						+ " "
						+ target.getCov());
			
			System.out.println(
					"(Normalized) Transformed point with conditioned entities"
						+ "(must be the same as conditioned point2d): "
						+ ptest);
			
			System.out.println("p (normalized): " + pn);
			
			System.out.println(
					"After reconditioning: "
						+ P
						+ " "
						+ p
						+ "  Transformation:"
						+ target.getTransformationMatrix()
						+ " "
						+ target.getCov());
			
			fail();
		}
	}

	
	/** Transform 3D points to their center of gravity. 
	 * @param elist 3D points
	 */
	protected static void transformToCenter(Entity[] elist) {
		int dim = elist[0].getEuclideanPart().getDim();
		Vector center = LinearAlgebraFactory.getVector(dim);
		for (int i = 0; i < elist.length; i++) {
			center = center.plus(elist[i].getEuclideanPart());
		}

		center = center.mult(1.0 / elist.length);

		Vector centerh = LinearAlgebraFactory.getVector(4);
		centerh.set(0, center.get(0));
		centerh.set(1, center.get(1));
		centerh.set(2, center.get(2));
		centerh.set(3, 0.0);

		for (int i = 0; i < elist.length; i++) {
			elist[i].setV(elist[i].getV().minus(centerh));
		}

	}
	
	/**
	 * Help function to test two entities. The Vector is testet
	 * with the help of some factor, that tells how much percentage
	 * tolerance is accepted. The cov matrix is not tested.
	 * @param e1 - entity
	 * @param e2 - entity to becompared to e1
	 * @param percentage - percentage of tolerance accpeted in vector
	 * @return - e1 == e2
	 */
	public static boolean compareEntities(
			Element e1, 
			Element e2, 
			double percentage){
		Matrix e1Cov = e1.getCov();
		Matrix e2Cov = e2.getCov();
		
		Vector e1Vec = e1.getV();
		Vector e2Vec = e2.getV();

		//set values near zero to zero, if not relative test will always 
		//fail with zero values
		for(int i = 0; i < e1Vec.getDim(); i++){
			if(Math.abs(e1Vec.get(i)) < 10e-6) e1Vec.set(i,0);
			if(Math.abs(e2Vec.get(i)) < 10e-6) e2Vec.set(i,0);
		}
		
		Vector diffV1 = e1Vec.minus(e2Vec);
		Vector diffV2 = e2Vec.minus(e1Vec);
		
		for(int i = 0; i < e1Vec.getDim(); i++){
			//relative test
			if((Math.abs(diffV1.get(i)) > Math.abs(e1Vec.get(i))*percentage
					&& Math.abs(diffV2.get(i)) > Math.abs(e2Vec.get(i))*percentage)){
				//is difference bigger than cov allows?
				if(Math.abs(diffV1.get(i)) > 
						(Math.sqrt(e1Cov.get(i,i))+Math.sqrt(e2Cov.get(i,i)))/2.0){
					return false;
				}
			}
		}
		
		return true;
	}
	
	/**
	 * Help function to test two Cov Matrices in testSimulation.
	 */
	public static boolean testCov(
			Matrix testValues, 
			Matrix expectedValues, 
			double percentage, double zeroThreshold){

		//everything except the diagonal has to be zero (or nearly zero)
		for(int i = 0; i < testValues.getRowDimension(); i++){
			for(int j = 0; j < testValues.getColumnDimension(); j++){
				if(i == j)
					continue;
				else if(Math.abs(testValues.get(i,j)) > zeroThreshold) return false;
				else if(Math.abs(expectedValues.get(i,j)) > zeroThreshold) return false;
				
				testValues.set(i,j,0.0);
				expectedValues.set(i,j,0.0);
			}
		}
		
		Matrix diff1 = testValues.minus(expectedValues);
		
		//test diagonal values.
		for(int i = 0; i < testValues.getRowDimension(); i++){
			//relative test
			if(Math.abs(diff1.get(i,i)) > Math.abs(testValues.get(i,i))*percentage){				//is difference bigger than cov allows?{
				return false;
			}
		}
		
		return true;
	}
	
	/**
	 * Help function, to generat gaussdistributed random values
	 * @param mean - meanvalue
	 * @param stddev - standard derivation
	 * @param newstart - should a new random object be created?
	 * 					otherwise the one from the previous call
	 * 					is reused.
	 * @return an random value
	 */
	public static double getNextGaussDist(
			double mean,
			double stddev,
			boolean newstart) {
			if (newstart)
				random = new Random();

			return random.nextGaussian() * stddev + mean;

		}
	
	/**
	 * Random Object for the function above.
	 */
	private static Random random = new Random();
}
