/*
 * Created on 19.04.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * @author Bastian Oehler
 * To avoid a memory error just run test with VM param: -Xmx256M
 */
public class Line_2DTest extends TestCase {

	/**
	 * Constructor for Line_2DTest.
	 * @param arg0
	 */
	public Line_2DTest(String arg0) {
		super(arg0);
	}

	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 1 000 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 * If MAXITER is set to 1 000 000, it may happen in an 'Out of Memory'-Error!
	 */
	public static int MAXITER = 1000000;

	/**
	 * Test for int getDegreesOfFreedom()
	 * Uses two differently constructed lines
	 */
	public void testGetDegreesOfFreedom() {
		//construct lines
		Line_2D lineOne = new Line_2D(4,21,16);
		Line_2D lineTwo = new Line_2D(5,7);
		
		//use getDegreesOfFreedom
		int dof = lineOne.getDegreesOfFreedom();
		int dof2 = lineTwo.getDegreesOfFreedom();
		
		//test if it gets the right value and if the value is the same 
		//on every point
		if(dof != 2) fail();
		if(dof != dof2) fail();
	}

	/**
	 * Test for int getDimension()
	 * Uses two differently constructed lines
	 */
	public void testGetDimension() {
		//construct lines
		Line_2D lineOne = new Line_2D(4,21,16);
		Line_2D lineTwo = new Line_2D(5,7);
		
		//use getDegreesOfFreedom
		int dim = lineOne.getDimension();
		int dim2 = lineTwo.getDimension();
		
		//test if it gets the right value and if the value is the same 
		//on every point
		if(dim != 3) fail();
		if(dim != dim2) fail();
	}

	/**
	 * Test for Vector getEuclideanPart()
	 */
	public void testGetEuclideanPart() {
		Line_2D line = new Line_2D(3,7,9);
		Vector euclid = line.getEuclideanPart();
		int dim = euclid.getDim();
		if ((dim!=1)||(euclid.get(0)!=9))fail();
	}

	/**
	 * Test for Vector getHomogeneousPart()
	 */
	public void testGetHomogeneousPart() {
		Line_2D line = new Line_2D(3,7,9);
		Vector hom = line.getHomogeneousPart();
		int dim = hom.getDim();
		if ((dim!=2)||(hom.get(0)!=3)||(hom.get(1)!=7))fail();
	}

	/**
	 * Test for Matrix getJacobianOfRelation(Class, Class)
	 */
	public void testGetJacobianOfRelationClassClass() {
		Line_2D line=new Line_2D(1,0,4);
		Matrix InPt=line.getJacobianOfRelation(Incident.class,Point_2D.class);
		Matrix IdLn=line.getJacobianOfRelation(Equal.class,Line_2D.class);
		Matrix PlLn=line.getJacobianOfRelation(Parallel.class,Line_2D.class);
		Matrix OrLn=line.getJacobianOfRelation(Orthogonal.class,Line_2D.class);
		for(int i=0;i<3;i++){
			if(InPt.get(0,i)!=line.getV().get(i))fail();
		}
		Matrix IdLnCheck=line.SkewSym();
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(IdLnCheck.get(i,j)!=IdLn.get(i,j))fail();
			}
		}
		if(PlLn.get(0,0)!=-line.getV().get(1))fail();
		if(PlLn.get(0,1)!=line.getV().get(0))fail();
		for(int i=0;i<2;i++){
			if(InPt.get(0,i)!=line.getV().get(i))fail();
		}		
		
		//check Ortoghonal Matrix:
		Matrix ortho = LinearAlgebraFactory.getVector(1,0,0).getMatrix().trans();
		if(ortho.minus(OrLn).vec().getLength() != 0)
			fail();
		
		//check for combinations of relations
		Class[][] arr = {{Incident.class,Point_2D.class},{Equal.class,Line_2D.class}
							,{Parallel.class,Line_2D.class},{Orthogonal.class,Line_2D.class}};
		if(!TestTools.helpDofOfRelations(arr,line))fail();
	}

	/**
	 * Test for int getDofOfRelation(Class,Class)
	 */
	public void testGetDofOfRelation() {
		Line_2D line = new Line_2D();
		int InPt=line.getDofOfRelation(Incident.class,Point_2D.class);
		int IdLn=line.getDofOfRelation(Equal.class,Line_2D.class);
		int PlLn=line.getDofOfRelation(Parallel.class,Line_2D.class);
		int OrLn=line.getDofOfRelation(Orthogonal.class,Line_2D.class);
		//check for right values of Dof
		if((InPt!=1)||(IdLn!=2)||(PlLn!=1)||(OrLn!=1))fail();
		//check for combinations of relations
		Class[][] arr = {{Incident.class,Point_2D.class},{Equal.class,Line_2D.class}
							,{Parallel.class,Line_2D.class},{Orthogonal.class,Line_2D.class}};
		if(!TestTools.helpDofOfRelations(arr,line))fail();
	}

	/**
	 * Test for void Line_2D(Point_2D, Point_2D)
	 * TODO: Why does this test fail only sometimes???
	 */
	public void testLine_2DPoint_2DPoint_2D() {
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing Constructor",
					"Line_2D(Point_2D,Point_2D)",0,MAXITER);
		//initialise two points
		double[][] covArrayOne={{0.077,0.002,0},{0.0002,0.05,0},{0,0,0}};
		double[][] covArrayTwo={{0.037,0.0005,0},{0.0005,0.065,0},{0,0,0}};
		Matrix covOne = LinearAlgebraFactory.getMatrix(covArrayOne);
		Matrix covTwo = LinearAlgebraFactory.getMatrix(covArrayTwo);
		Point_2D pointOne = new Point_2D(12,57,1,covOne);
		Point_2D pointTwo = new Point_2D(35,10,1,covTwo);
		//construct line from two points
		Line_2D line = new Line_2D(pointOne,pointTwo);
		//check result (vector)
		Vector vec = pointOne.getV().crossProduct(pointTwo.getV());
		if(!TestStatistics.check(line.getV(),vec,0.2))fail();
		//check statistics
		TestStatistics statistic = new TestStatistics(3);
		RandomVector rv1 = new RandomVector(pointOne.getV(),covOne);
		RandomVector rv2 = new RandomVector(pointTwo.getV(),covTwo);
		Point_2D pointTest1;
		Point_2D pointTest2;
		for (int i=0;i<MAXITER;i++){
			pointTest1 = new Point_2D(rv1.getNextRandom(),covOne);
			pointTest2 = new Point_2D(rv2.getNextRandom(),covTwo);
			Line_2D lineTest = new Line_2D(pointTest1,pointTest2);
			statistic.addObservation(lineTest);
			//break if cancel button is pressed
			if(pm.isCanceled()) break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		Line_2D simulationLine = new Line_2D(statistic.getMue(),statistic.getSigma());
		simulationLine.normalizeEuclidean();
		line.normalizeEuclidean();
		if (!TestStatistics.check(line.getV(),simulationLine.getV(),
									line.getCov(),simulationLine.getCov(),0.5)){
			System.out.println("testLine_2DPoint_2DPoint_2D(): ");
			System.out.println("SUGR: ");
			System.out.println("Vector: " + line.getV());
			System.out.println("Matrix: " + line.getCov());
			System.out.println("Simulation: ");
			System.out.println("Vector: " + simulationLine.getV());
			System.out.println("Matrix: " + simulationLine.getCov());
			fail();
		}
	}

	/**
	 * Test for void normalizeEuclidean()
	 */
	public void testNormalizeEuclidean() {
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing normalizeEuclidean",
					"with Line_2D",0,(int)1000);
		//Test over 1 Million Lines
		for(int i = 0; i < 1000; i++) {
			for(int j = 0; j < 1000; j++){
				Line_2D line = new Line_2D(i-100,(j+100)%1000,j-500.5);
				Vector v = line.getV();
				double factor = 1/(line.getHomogeneousPart().getLength());
				v=v.mult(factor);
				line.normalizeEuclidean();
				if(!TestStatistics.check(line.getV(),v,0.00000001)) {
					fail();
				}
			}
			//break if cancel button is pressed
			if(pm.isCanceled()) break;
			//update progress monitor.
			pm.setProgress(i);
		}
		pm.close();
		//Test Statistics
		Point_2D p1 = new Point_2D(2,6,2.6,LinearAlgebraFactory.getDiagMatrix(3, 0.02));
		if(!TestTools.helpNormalizeEuclideanStatistics(p1,MAXITER)) fail();
		
		double[][] cov = {{0.04,0.01,0.02},{0.01,0.05,0.03},{0.02,0.03,0.07}};
		Point_2D p2 = new Point_2D(264,654,-67,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p2,MAXITER)) fail();
		
		Point_2D p3 = new Point_2D(-54,76,24,LinearAlgebraFactory.getMatrix(cov));
		if(!TestTools.helpNormalizeEuclideanStatistics(p3,MAXITER)) fail();
	}

	/**
	 * Test for double getDistance()
	 */
	public void testGetDistance() {
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing getDistance",
					"with Point_2D",0,(int)1e4);
		outer:
		for(int i=0;i<1e4;i++){
			for(int j=0;j<1e3;j++){
				//break if cancel button is pressed
				if(pm.isCanceled()) break outer;
				//update progress monitor.
				pm.setProgress(i);
				//contruct lines and points
				Line_2D line = new Line_2D(24+i,300-j,i/25);
				Point_2D point = new Point_2D(i,j,1);
				//calculate distance
				double z = Math.abs(line.getV().scalarmult(point.getV()));
				double n = line.getHomogeneousPart().mult(point.getHomogeneousPart().get(0)).normF();
				double distCalc=(z/n);
				double dist = line.getDistance(point);
				//zeroing...
				double zero = Math.abs(dist-(distCalc));
				if(zero>1e-10){
					System.out.println("Fehler mit den Werten: ");
					System.out.println("Point_2D: " + point);
					System.out.println("Line_2D" + line);
					System.out.println("selbst gerechnete Distanz: " + distCalc);
					System.out.println("von sugr: " + dist);
					fail();
				}
			}
		}
		pm.close();
	}

	/**
	 * Test for void Line_2D(Vector, Matrix)
	 */
	public void testLine_2DVectorMatrix() {
		//Construct line...
		Vector v3D = LinearAlgebraFactory.getVector(14,12,1);
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.01);
		Line_2D line = new Line_2D(v3D,cov);
		
		//zeroing...
		Vector nullVector = line.getV().minus(v3D);
		Matrix nullMatrix = line.getCov().minus(cov);
		
		//check results (should be equal to zero)
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(nullMatrix.get(i,j)!=0)fail();
			}
			if(nullVector.get(i)!=0)fail();
		}
	}

	/**
	 * Test for void Line_2D(double, double, double)
	 */
	public void testLine_2Ddoubledoubledouble() {
		Line_2D line = new Line_2D(2,5,Double.MAX_VALUE);
		if(line.getV().get(0)!=2)fail();
		if(line.getV().get(1)!=5)fail();
		if(line.getV().get(2)!=Double.MAX_VALUE)fail();
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(line.getCov().get(i,j)!=0)fail();
			}
		}
	}

	/**
	 * Test for void Line_2D(double, double, double, Matrix)
	 */
	public void testLine_2DdoubledoubledoubleMatrix() {
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,Double.MAX_VALUE);
		Line_2D line = new Line_2D(752,65,5,cov);
		if(line.getV().get(0)!=752)fail();
		if(line.getV().get(1)!=65)fail();
		if(line.getV().get(2)!=5)fail();
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(line.getCov().get(i,j)!=cov.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for void Line_2D(double, double)
	 * angle, distance!!!
	 */
	public void testLine_2Ddoubledouble() {
		Vector vec = LinearAlgebraFactory.getVector(-1,0,-5);
		Line_2D line = new Line_2D(Math.PI,5);
		if(!TestStatistics.check(vec,line.getV(),1e-14))fail();
		//test for all entries of Cov equal to zero 
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(line.getCov().get(i,j)!=0)fail();
			}
		}
	}

	/**
	 * Test for void Line_2D(double, double, Matrix)
	 * angle, distance!!!
	 */
	public void testLine_2DdoubledoubleMatrix() {
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(3,0.003);
		Vector vec = LinearAlgebraFactory.getVector(0,1,0);
		Line_2D line = new Line_2D(Math.PI/2,0,cov);
		if(!TestStatistics.check(vec,line.getV(),1e-14))fail();
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if (i!=j){
					if(line.getCov().get(i,j)!=0)fail();
				}
				else{
					if(line.getCov().get(i,j)!=0.003)fail();
				}
			}
		}
	}

	/**
	 * Test for void Line_2D()
	 */
	public void testLine_2D() {
		Line_2D emptyLine = new Line_2D();
		if (emptyLine==null)fail();
	}

	/**
	 * Test for Line_2D copy()
	 */
	public void testCopy() {
		Matrix cov=LinearAlgebraFactory.getDiagMatrix(3,0.001);
		Vector vec=LinearAlgebraFactory.getVector(2,7,25);
		Line_2D line=new Line_2D(vec,cov);
		//copy line to lineCopy
		Line_2D lineCopy=line.copy();
		//change line
		line=new Line_2D(0,0,1);
		//check if lineCopy was affected by changes in line and if copy is correct
		for(int i=0;i<3;i++){
			if(lineCopy.getV().get(i)!=vec.get(i))fail();
			for(int j=0;j<3;j++)
				if(lineCopy.getCov().get(i,j)!=cov.get(i,j))fail(); 
		}
	}

	/**
	 * Test for Matrix SkewSym()
	 */
	public void testSkewSym() {
		//preparations
		Vector vec = LinearAlgebraFactory.getVector(25,99,3.75);
		double[][] array={{0,-3.75,99},{3.75,0,-25},{-99,25,0}};
		Matrix hand = LinearAlgebraFactory.getMatrix(array);
		Line_2D line = new Line_2D(vec,LinearAlgebraFactory.getDiagMatrix(3,1e-3));
		//use SkewSym()
		Matrix calc=line.SkewSym();
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(i==j)if(calc.get(i,i)!=0)fail();
				else if(calc.get(i,j)!=hand.get(i,j))fail();
			}
		}
	}

	/**
	 * Test for static Matrix SkewSym(Vector)
	 */
	public void testSkewSymVector() {
		//preparations
		Vector vec = LinearAlgebraFactory.getVector(25,99,3.75);
		double[][] array={{0,-3.75,99},{3.75,0,-25},{-99,25,0}};
		Matrix hand = LinearAlgebraFactory.getMatrix(array);
		//use SkewSym(Vector)
		Matrix calc=Line_2D.SkewSym(vec);
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(i==j)if(calc.get(i,i)!=0)fail();
				else if(calc.get(i,j)!=hand.get(i,j))fail();
			}
		}
	}


	/**
	 * Test for double getDistanceToOrigin
	 */
//	public void testGetDistanceToOrigin() {
//		double distOne;
//		double distTwo;
//		double distThree;
//		double distOneCtrl;
//		double distTwoCtrl;
//		double distThreeCtrl;
//		for (int i=0;i<1e2;i++){
//			for (int j=0;j<1e2;j++){
//				for (int k=0;k<1e2;k++){
//					Line_2D lineOne = new Line_2D(i-50,j-50,k+1);
//					Line_2D lineTwo = new Line_2D(i-50,j-50,-(k+1));
//					Line_2D lineThree = new Line_2D(i+1,j-50,k-50);
//					distOne = lineOne.getDistanceToOrigin();
//					distTwo = lineTwo.getDistanceToOrigin();
//					distThree = lineThree.getDistanceToOrigin();
//					if((i==50)&&(j==50)){
//						if((distOne!=Double.POSITIVE_INFINITY)
//							||(distTwo!=Double.POSITIVE_INFINITY)){
//								fail();
//						}
//					}
//					else{
//						lineOne.normalizeEuclidean();
//						lineTwo.normalizeEuclidean();
//						lineThree.normalizeEuclidean();
//						distOneCtrl = Math.abs(Math.abs(lineOne.getEuclideanPart().get(0))-distOne);
//						distTwoCtrl = Math.abs(Math.abs(lineTwo.getEuclideanPart().get(0))-distTwo);
//						distThreeCtrl = Math.abs(Math.abs(lineThree.getEuclideanPart().get(0))-distThree);
//						if (distOneCtrl>1e-13){
//							System.out.println("i: "+i+" j: "+j+" k: "+k);
//							System.out.println("Ctrl: "+distOneCtrl);
//							fail();
//						}
//						if (distTwoCtrl>1e-13){
//							System.out.println("i: "+i+" j: "+j+" k: "+k);
//							System.out.println("Ctrl: "+distTwoCtrl);
//							fail();
//						}
//						if (distThreeCtrl>1e-13){
//							System.out.println("i: "+i+" j: "+j+" k: "+k);
//							System.out.println("Ctrl: "+distThreeCtrl);
//							fail();
//						}
//					}
//				}
//			}
//		}
//	}
}
