/*
 * Created on 04.05.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Tests for class Line_3D.
 * Most important are the constructors which construct a plane out 
 * of other entities.
 * Uses: TestStatistics.class and TestTools.class.
 * 
 * @author Achim Knigs
 */
public class Line_3DTest extends TestCase {

	/**
	 * Constructor for Line_3DTest.
	 * @param arg0
	 */
	public Line_3DTest(String arg0) {
		super(arg0);
	}
	
	/**
	 * int MAXITER determines how many iterations there are for calculation of
	 * the statstic parts. It should be around 1 000 000 for good results, but
	 * this may mean much work for your maschine and some coffee for you,
	 * if you are waiting for the results.
	 */
	public static int MAXITER = 100000;
	
	/**
	 * Method to test the statistic reasoning of a line out 
	 * of two uncertain points.
	 * @param point1 - the point first
	 * @param point2 - the second point to be connected to the first one
	 * @return - true if test is successful
	 */
	public boolean helpLine_3DPoint_3DPoint_3DStatistics(Point_3D point1, Point_3D point2,double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(6);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(point1.getV(),point1.getCov());
		RandomVector vl2 = new RandomVector(point2.getV(),point2.getCov());
		
		//a temp point
		Line_3D connection;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Testing reasoning of a Line","out of two Points",0,MAXITER);
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			//System.out.println(i);
			//get "gaussian" lines
			Point_3D tpoint1 = new Point_3D(vl1.getNextRandom(),point1.getCov());
			Point_3D tpoint2 = new Point_3D(vl2.getNextRandom(),point2.getCov());
			
			//calculate point and add observation
			connection = new Line_3D(tpoint1,tpoint2);
			stat.addObservation(connection);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//calculate values
		connection = new Line_3D(point1,point2);
		
		Line_3D estimated = new Line_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		//connection.normalizeEuclidean();
		//estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(connection,estimated,sigma);
		
		if(!retval) {
			System.out.println("Constructor Failed: Line_3D(Point_3D,Point_3D)");
			System.out.println("Failed Plane: " + connection);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}


	/**
	 * Method to test the statistic reasoning of a line 
	 * out two uncertain planes.
	 * @param plane1 - the first plane
	 * @param plane2 - the second plane to be intersected with the first one
	 * @return - true if test is successful
	 */
	public boolean helpLine_3DPlane_3DPlane_3DStatistics(Plane_3D plane1, Plane_3D plane2,double sigma){
		//Create Statistics Object
		TestStatistics stat = new TestStatistics(6);
		
		//Two Random Vectors for Random Line generation
		RandomVector vl1 = new RandomVector(plane1.getV(),plane1.getCov());
		RandomVector vl2 = new RandomVector(plane2.getV(),plane2.getCov());
		
		//a temp point
		Line_3D intersection;
		
		//ProgressMonitor:
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
							"Testing reasoning of a Line","out of two Planes",0,MAXITER);
		
		//iteration. should be close to calculated values after
		//100,000 iterations
		for(int i = 0; i < MAXITER; i++)
		{
			//System.out.println(i);
			//get "gaussian" lines
			Plane_3D tplane1 = new Plane_3D(vl1.getNextRandom(),plane1.getCov());
			Plane_3D tplane2 = new Plane_3D(vl2.getNextRandom(),plane2.getCov());
			
			//calculate point and add observation
			intersection = new Line_3D(tplane1,tplane2);
			stat.addObservation(intersection);
			
			if(pm.isCanceled()) break;
			pm.setProgress(i);
		}
		pm.close();
		
		//calculate values
		intersection = new Line_3D(plane1,plane2);
		
		Line_3D estimated = new Line_3D(stat.getMue(),stat.getSigma());

		//normalize Planes
		//intersection.normalizeEuclidean();
		//estimated.normalizeEuclidean();

		//compare and return true if equal
		boolean retval = TestStatistics.check(intersection,estimated,sigma);
		
		if(!retval) {
			System.out.println("Constructor Failed: Line_3D(Plane_3D,Plane_3D)");
			System.out.println("Failed Plane: " + intersection);
			System.out.println("Calculated Mue and Sigma: " + estimated);
		}
		return retval;
	}
	
	/**
	 * Test for getDegreesOfFreedom()
	 */	
	public void testGetDegreesOfFreedom() {
		//create line
		Line_3D L = new Line_3D();
		
		//get dof
		int dof = L.getDegreesOfFreedom();
		
		//dof must be 4.
		if(dof != 4) fail();
	}

	/**
	 * Test for getDimension()
	 */
	public void testGetDimension() {
		//create line
		Line_3D L = new Line_3D();
		
		//get dimension
		int dim = L.getDimension();
		
		//dim must be 6
		if(dim != 6) fail();
	}
	
	/**
	 * Test for getEuclideanPart()
	 */
	public void testGetEuclideanPart() {
		//create line
		Line_3D L = new Line_3D(5,6,7,8,9,1);
		
		//get euclidean part:
		Vector euc = L.getEuclideanPart();
		
		//control vector:
		Vector contr = LinearAlgebraFactory.getVector(8,9,1);
		
		//fail if not equal.
		if(!TestStatistics.check(contr,euc,0.001)) fail();
	}
	
	/**
	 * Test for getHomogeneousPart()
	 */
	public void testGetHomogeneousPart() {
		//create line
		Line_3D L = new Line_3D(1,2,3,4,5,6);
		
		//get homogeneous part
		Vector hom = L.getHomogeneousPart();
		
		//generate control vector
		Vector contr = LinearAlgebraFactory.getVector(1,2,3);
		
		 //fail if not equal
		 if(!TestStatistics.check(contr,hom,0.001)) fail();
	}

	/**
	 * Test for Matrix getJacobianOfRelation(Class, Class)
	 * Tests if the Jacobian look like they should according 
	 * to documentation (sugr handbook) and if they do what
	 * they should.
	 */
	public void testGetJacobianOfRelationClassClass() {		
		//line between:  10     0   |  10 <- plane point
		//				  0	   10	|  10
		//				 20	   15 	|   0	
		Line_3D l = new Line_3D(-10,10,-5,-200,-150,100);

		//--------------------------------------------------------------------------------------------------
		//incidence:
		//--------------------------------------------------------------------------------------------------
		//two incident points		
		Point_3D incP = new Point_3D(10,0,20);
		Point_3D incP2 = new Point_3D(0,10,15);
		//not incident point
		Point_3D nIncP = new Point_3D(10,10,0);
		
		//incidente plane and line.
		Plane_3D incPlane = new Plane_3D(incP,incP2,nIncP);
		Line_3D incL = new Line_3D(incP,nIncP);
		
		//get the jacobians
		Matrix incPM = l.getJacobianOfRelation(Incident.class,Point_3D.class);
		Matrix incPlaneM = l.getJacobianOfRelation(Incident.class,Plane_3D.class);
		Matrix incLM = l.getJacobianOfRelation(Incident.class,Line_3D.class);
		
		//should be zero, points are incident with line
		Vector zeroIncP1 = incPM.mult(incP.getV());
		Vector zeroIncP2 = incPM.mult(incP2.getV());
		
		//shouldn't be zero, point is not incident with line
		Vector nonZeroIncP = incPM.mult(nIncP.getV());
		
		//should be zero, plane is incident with line, second line also
		Vector zeroIncPlane = incPlaneM.mult(incPlane.getV());
		Vector zeroIncL = incLM.mult(incL.getV());
		
		//control the values one by one:
		//jacobian for points
		double[][] values = {	{   0,  -5,-10, 200},
								{   5,   0,-10, 150},
								{  10,  10,  0,-100},
								{-200,-150,100,   0}};
		
		Matrix zeroIncPCalc = LinearAlgebraFactory.getMatrix(values).minus(incPM);
		
		//jacobian for planes
		double[][] values2 = {	{0  ,-100,-150,-10},
								{100,   0, 200, 10},
								{150,-200,   0, -5},
								{10 , -10,   5,  0}};

		Matrix zeroIncPlaneCalc = LinearAlgebraFactory.getMatrix(values2).minus(incPlaneM); 

		//jacobian for lines
		Matrix zeroIncLCalc = (Line_3D.DualMatrixC().mult(l.getV()).getMatrix().trans()).minus(incLM);
		
		//fail if wrong values
		if(zeroIncP1.getLength() != 0) fail();
		if(zeroIncP2.getLength() != 0) fail();
		if(nonZeroIncP.getLength() == 0) fail();
		if(zeroIncPlane.getLength() != 0) fail();
		if(zeroIncL.getLength() != 0) fail();	
		if(zeroIncPCalc.vec().getLength() != 0) fail();	
		if(zeroIncPlaneCalc.vec().getLength() != 0) fail();	
		if(zeroIncLCalc.vec().getLength() != 0) fail();
		
		
		//--------------------------------------------------------------------------------------------------
		//equal:
		//--------------------------------------------------------------------------------------------------
		//construct a line equal to first:
		Line_3D eq = new Line_3D(l.getV().mult(7),l.getCov());
		//Line not equal
		Line_3D nEq = new Line_3D(10,-10,-5,-150,-200,100,l.getCov());
		//Line parallel 
		Line_3D eqPar = new Line_3D(-10,10,-5,-20,-15,10,l.getCov());
		Line_3D eqParNichtPluecker = new Line_3D(-20,20,-10,7000,0,1);
		
		//check if zero when multiplied with jacobian (i.e. equal)
		Matrix jac = l.getJacobianOfRelation(Equal.class,Line_3D.class);
		Vector zeroEq1 = jac.mult(eq.getV());
		Vector zeroNEq = jac.mult(nEq.getV());
		Vector zeroEqPar = jac.mult(eqPar.getV());
		Vector zeroEqParNichtPluecker = jac.mult(eqParNichtPluecker.getV());
		//fail if not
		if(zeroEq1.getLength() != 0) fail();
		if(zeroNEq.getLength() == 0) fail();
		if(zeroEqPar.getLength() == 0) fail();
		if(zeroEqParNichtPluecker.getLength() == 0) fail();
		
		//check jacobian matrix
		//test is done in testGetLineLineJacobian:
		Matrix zeroEqJac = jac.minus(
						l.getLineLineJacobian());	
		//if don't match, fail
		if(zeroEqJac.vec().getLength() != 0) fail();
		
		
		
		//--------------------------------------------------------------------------------------------------
		//parallelity:
		//--------------------------------------------------------------------------------------------------
		//construct parallel line
		Line_3D parallel = new Line_3D(-10,10,-5,-200,-150,200);
		//construct parallel plane
		Plane_3D parallelP = new Plane_3D(new Point_3D(10,0,20),new Point_3D(0,10,15),new Point_3D(10,10,0));
		
		//test if constraint is fullfilled:
		Vector zeroPar = l.getJacobianOfRelation(Parallel.class,Line_3D.class).mult(parallel.getV());
		Vector zeroParPlane = l.getJacobianOfRelation(Parallel.class,Plane_3D.class).mult(parallelP.getV());
		//fail if not
		if(zeroPar.getLength() != 0) fail();
		if(zeroParPlane.getLength() != 0) fail();
		
		//construct not parallel line:
		Line_3D notParallel = new Line_3D(-20,10,-5,-200,-150,200);
		//construct not parallel plane:
		Plane_3D notParPlane = new Plane_3D(-10,10,-5,1);
		
		//test if constraint is fullfilled:
		Vector notZeroPar = l.getJacobianOfRelation(Parallel.class,Line_3D.class).mult(notParallel.getV());
		Vector notZeroParPlane = l.getJacobianOfRelation(Parallel.class,Plane_3D.class).mult(notParPlane.getV());
		//fail if zero:
		if(notZeroPar.getLength() == 0) fail();
		if(notZeroParPlane.getLength() == 0)fail();
		
		//test jacobian matrix by value:
		//Line is: (-10,10,-5,-200,-150,100)
		double[][] parL = {	{0,5,10,0,0,0},
							{-5,0,10,0,0,0},
							{-10,-10,0,0,0,0},
							{0,0,0,0,0,0},
							{0,0,0,0,0,0},
							{0,0,0,0,0,0}};
		//get Jacobain Matrix, simply S(L_h), filled with zeros for compability
		Matrix parLMat = LinearAlgebraFactory.getMatrix(parL);
		//compare to the one from sugr
		Matrix zeroParMat = parLMat.minus(l.getJacobianOfRelation(Parallel.class,Line_3D.class));
		
		//fail if different
		if(zeroParMat.vec().getLength() != 0) fail();
		
		//same for plane matrix:
		double[][] parP = {{-10,10,-5,0}};
		//get Jacobian Matrix, simply L_h^T
		Matrix parPMat = LinearAlgebraFactory.getMatrix(parP);
		//compare to the one from sugr:
		Matrix zeroParMatP = parPMat.minus(l.getJacobianOfRelation(Parallel.class,Plane_3D.class));
		
		//fail if different
		if(zeroParMatP.vec().getLength() != 0) fail();
		
		
		//--------------------------------------------------------------------------------------------------
		//orthogonality
		//--------------------------------------------------------------------------------------------------
		//construct orthogonal line and plane
		Line_3D orthoL = new Line_3D(10,10,0,-200,-150,100);
		Plane_3D orthoP = new Plane_3D(-10,10,-5,1);
	
		//test if constraint is fullfilled
		Vector zeroOrthoL = l.getJacobianOfRelation(Orthogonal.class,Line_3D.class).mult(orthoL.getV());
		Vector zeroOrthoP = l.getJacobianOfRelation(Orthogonal.class,Plane_3D.class).mult(orthoP.getV());
		
		//fail if not zero
		if(zeroOrthoL.getLength() != 0) fail();
		if(zeroOrthoP.getLength() != 0) fail();
		
		//construct not orthogonal line and plane:
		Line_3D nOrthoL = new Line_3D(-10,10,-5,-200,-150,100);
		Plane_3D nOrthoP = new Plane_3D(10,10,0,1);
		
		//test if constraint is fullfilled
		Vector nZeroOrthoL = l.getJacobianOfRelation(Orthogonal.class,Line_3D.class).mult(nOrthoL.getV());
		Vector nZeroOrthoP = l.getJacobianOfRelation(Orthogonal.class,Plane_3D.class).mult(nOrthoP.getV());
		
		//fail if zero:
		if(nZeroOrthoL.getLength() == 0) fail();
		if(nZeroOrthoP.getLength() == 0) fail();
		
		//test jacobians by value:
		//Line is: (-10,10,-5,-200,-150,100)
		double[][] dOrthoL = {{-10,10,-5,0,0,0}};
		double[][] dOrthoP = {	{0,5,10,0},
								{-5,0,10,0},
								{-10,-10,0,0},
								{0,0,0,0}};

		//get matrices: simply S(L_h) for plane and L_h^T for Line:								
		Matrix orthoLMat = LinearAlgebraFactory.getMatrix(dOrthoL);
		Matrix orthoPMat = LinearAlgebraFactory.getMatrix(dOrthoP);
		//compare to ones from sugr
		Matrix zeroOrthoLMat = l.getJacobianOfRelation(Orthogonal.class,Line_3D.class).minus(orthoLMat);
		Matrix zeroOrthoPMat = l.getJacobianOfRelation(Orthogonal.class,Plane_3D.class).minus(orthoPMat);
		//fail if not equal
		if(zeroOrthoLMat.vec().getLength() != 0) fail();
		if(zeroOrthoPMat.vec().getLength() != 0) fail(); 
		
		
		//test if all unallowed combinations throw exception
		Class[][] allowed = {	{Incident.class,Point_3D.class},
								{Incident.class,Line_3D.class},
								{Incident.class,Plane_3D.class},
								{Equal.class,Line_3D.class},
								{Parallel.class,Line_3D.class},
								{Parallel.class,Plane_3D.class},
								{Orthogonal.class,Line_3D.class},
								{Orthogonal.class,Plane_3D.class}};
		
		if(!TestTools.helpJacobianOfRelations(allowed,l)) fail();
	}

	/**
	 * Test for getDofOfRelation()
	 */
	public void testGetDofOfRelation() {
		//construct line
		Line_3D l = new Line_3D();
		
		//Incident
		int incPL = l.getDofOfRelation(Incident.class,Point_3D.class);
		int incPlaneL = l.getDofOfRelation(Incident.class,Plane_3D.class);
		int incLL = l.getDofOfRelation(Incident.class,Line_3D.class);
		
		//Identity
		int eq = l.getDofOfRelation(Equal.class,Line_3D.class);
		
		//Parallelity
		int parLL = l.getDofOfRelation(Parallel.class,Line_3D.class);
		int parLP = l.getDofOfRelation(Parallel.class,Plane_3D.class);
		
		//Orthogonality
		int orthoLL = l.getDofOfRelation(Orthogonal.class,Line_3D.class);
		int orthoLP = l.getDofOfRelation(Orthogonal.class,Plane_3D.class);
		
		//test if values are right.
		if(incPL != 2) fail();
		if(incPlaneL != 2) fail();
		if(incLL != 1) fail();
		if(eq != 4) fail();
		if(parLL != 2) fail();
		if(parLP != 1) fail();
		if(orthoLL != 1) fail();
		if(orthoLP != 2) fail();
		
		
		//test if all other combinations throw exception
		Class[][] allowed = {	{Incident.class,Point_3D.class},
								{Incident.class,Plane_3D.class},
								{Incident.class,Line_3D.class},
								{Equal.class,Line_3D.class},
								{Parallel.class,Line_3D.class},
								{Parallel.class,Plane_3D.class},
								{Orthogonal.class,Line_3D.class},
								{Orthogonal.class,Plane_3D.class}};
		
		if(!TestTools.helpDofOfRelations(allowed,l)) fail();
	}

	/**
	 * Test for normailzeEuclidean.
	 * first tests if the method works for "sure" lines without covs
	 * then tests if the cov-matrix gets normalized correctly 
	 */
	public void testNormalizeEuclidean() {
		//Progressmonitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
						"Testing normalize Euclidean","no statistics",0,100*100*100);
		
		//looping over one million lines
		outer:
		for(int i = 0; i < 100; i++){
			for(int j = -50; j < 50; j++){
				for(int k = -100; k < 0; k++){
					//construct line (not "real" lines)
					Line_3D line = new Line_3D(i,j,k,i+j,k+j,i+k);
					
					//normalization factor
					double factor = 1.0/line.getHomogeneousPart().getLength();
					
					//calculate normalized vector
					Vector v = line.getV();
					for(int l = 0; l < 6; l++){
						v.set(l, v.get(l)*factor);
					}
					
					//let sugr calculate
					line.normalizeEuclidean();
					Vector vSugr = line.getV();
					
					//compare
					if(!TestStatistics.check(vSugr, v, 0.000001)) {
						pm.close();
						fail();
					}
					
					//break if cancel button is pressed
					if(pm.isCanceled()) break outer;
					//update progress
					pm.setProgress(i*(j+50)*(k+100));
				}
			}
		}
		pm.close();
		
		//covariances for lines
		double[][] covDoub = 	{	{31.5,0.2,2.2,0.2,0.2,0.92},
									{0.2,21.5,0.2,0.2,0.2,0.82},
									{2.2,0.2,11.5,0.2,0.2,0.72},
									{0.2,0.2,0.2,1.5,1.2,0.2},
									{0.2,0.2,0.2,1.2,1.5,0.12},
									{0.92,0.82,0.72,0.2,0.12,1.5}};

		Matrix cov = LinearAlgebraFactory.getMatrix(covDoub);
		
		//construct some insecure lines:
		Matrix covp=LinearAlgebraFactory.getDiagMatrix(4, 0.01);
		covp.set(3,3,0);
		covp.set(0,1, 0.002);
		covp.set(1,0, 0.002);
		Point_3D P1= new Point_3D(3,60,63,1,covp);
		Point_3D P2= new Point_3D(36,-60,613,1,covp);

		Line_3D l1 = new Line_3D(P1,P2);
		if(!TestTools.helpNormalizeEuclideanStatistics(l1,MAXITER,0.3)) fail();
		
		Line_3D l2 = new Line_3D(100,-100,290,1000,1000,0,cov);
		if(!TestTools.helpNormalizeEuclideanStatistics(l2,MAXITER,0.3)) fail();
		
		Line_3D l3 = new Line_3D(70,70,-170,290,-1140,-350,cov);
		if(!TestTools.helpNormalizeEuclideanStatistics(l3,MAXITER,0.3)) fail();
	}

	/**
	 * Test for getDistance()
	 * Calculates distance between one million points and lines
	 */
	public void testGetDistance() {
		//Progressmonitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
					"Testing getDistance","between lines and points",0,100*100*100);
		outer:
		for(int i = 0; i < 100; i++){
			for(int j = -150; j < -50;j++){
				for(int k = -100;k < 0; k++){
					//construct two lines and a point to get distance
					Line_3D l1 = new Line_3D(i,j,k,-j,-i-k,j);
					Line_3D l2 = new Line_3D(k,j,i,-j,-k-i,j);
					Point_3D p = new Point_3D(j,k,i);
					
					//calculate distance to line
					Vector z1 = l1.getDual().getV();
					Vector z2 = l2.getV();
					double zaehler = z1.getMatrix().trans().mult(z2).getLength();
					Vector n1 = l1.getHomogeneousPart();
					Vector n2 = l2.getHomogeneousPart();
					double nenner = n1.crossProduct(n2).getLength();
					double distCalc = zaehler / nenner;
					
					//let sugr calculate distance
					double distSugr = l1.getDistance(l2);
					
					//compare
					if(distCalc != distSugr) {
						pm.close();
						fail(); 
					}
					
					//calculate distance to point
					z1 = l1.getHomogeneousPart().crossProduct(p.getEuclideanPart());
					z2 = l1.getEuclideanPart().mult(p.getHomogeneousPart().get(0));
					zaehler = z1.plus(z2).getLength();
					n1 = l1.getHomogeneousPart().mult(p.getHomogeneousPart().get(0));
					nenner = n1.getLength();
					distCalc = zaehler / nenner;
					
					//let sugr calculate distance to point
					distSugr = l1.getDistance(p);
					
					//comapre
					if(distSugr != distCalc) {
						pm.close();
						fail(); 
					}
					
					//break if cancel button is pressed
					if(pm.isCanceled()) break outer;
					//update progress
					pm.setProgress(i*(j+150)*(k+100));
				}
			}
		}
		pm.close();
	}

	/**
	 * Test for void Line_3D(Vector, Matrix)
	 */
	public void testLine_3DVectorMatrix() {
		//construct vector and Matrix
		double[] values = {Double.MAX_VALUE,2,14,-24,24,2};
		Vector v = LinearAlgebraFactory.getVector(values);
		Matrix m = LinearAlgebraFactory.getDiagMatrix(6,0.5);
		
		//construct line
		Line_3D l = new Line_3D(v,m);
		
		//compare
		Vector zeroV = v.minus(l.getV());
		Matrix zeroM = m.minus(l.getCov());
		
		if(zeroV.getLength() != 0) fail();
		if(zeroM.vec().getLength() != 0) fail();
	}

	/**
	 * Test for void Line_3D(double, double, double, double, double, double)
	 */
	public void testLine_3Ddoubledoubledoubledoubledoubledouble() {
		//construct line 
		Line_3D l = new Line_3D(Double.MAX_VALUE,0.00000003,3,Double.NEGATIVE_INFINITY,5,6);
		
		//control vector
		double[] values = {Double.MAX_VALUE,0.00000003,3,Double.NEGATIVE_INFINITY,5,6};
		Vector zero = LinearAlgebraFactory.getVector(values).minus(l.getV());
		
		//compare
		if(zero.getLength() >= 0.00000001) fail();
	}

	/**
	 * Test for void Line_3D(double, double, double, double, double, double, Matrix)
	 */
	public void testLine_3DdoubledoubledoubledoubledoubledoubleMatrix() {
		//matrix:
		Matrix m = LinearAlgebraFactory.getDiagMatrix(6,0.5);
		
		//construct line 
		Line_3D l = new Line_3D(0.00000003,-3,Double.NEGATIVE_INFINITY,5,6,Double.MAX_VALUE,m);
		
		//control vector
		double[] values = {0.00000003,-3,Double.NEGATIVE_INFINITY,5,6,Double.MAX_VALUE};
		Vector zeroV = LinearAlgebraFactory.getVector(values).minus(l.getV());
		
		//compare
		Matrix zeroM = m.minus(l.getCov());
		if(zeroV.getLength() >= 0.00000001) fail();
		if(zeroM.vec().getLength() >= 0.00000001) fail();
	}

	/**
	 * Test for void Line_3D(double, double, double, double, double, double, Matrix, boolean)
	 */
	public void testLine_3DdoubledoubledoubledoubledoubledoubleMatrixboolean() {
		//matrix:
		Matrix m = LinearAlgebraFactory.getDiagMatrix(6,0.57);
		
		try {
			//construct line that fails Pluecker condition test
			new Line_3D(1,2,3,4,5,6,m,true);

			//no exception? fail!
			fail();
		}
		catch(IllegalArgumentException e){
			//everithing fine!
		}
		
		try {
			//construct line that shouldn't fail the test
			new Line_3D(-10,10,-5,-200,-150,100,m,true);
			
		}
		catch(IllegalArgumentException e){
			//exception? fail!
			fail();
		}
	}

	/**
	 * Test for Line_3D(Plane_3D, Plane_3D)
	 * That means construct a Line_3D out of two Planes.
	 */
	public void testLine_3DPlane_3DPlane_3D() {				
		//construct two planes
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4,0.0001);
		Plane_3D p1 = new Plane_3D(-3.5,1.8,3.5,5,cov);
		Plane_3D p2 = new Plane_3D(-5.3,-3,-3.2,-5,cov);
		
		if(!helpLine_3DPlane_3DPlane_3DStatistics(p1,p2,0.3)) fail();
		
		//construct two more planes
		Plane_3D p3 = new Plane_3D(5231,4215.2,5621,10000,cov);
		Plane_3D p4 = new Plane_3D(-5233,5423,-803.5,-10000,cov);
		
		if(!helpLine_3DPlane_3DPlane_3DStatistics(p3,p4,0.3)) fail();
	}

	/**
	 * Test for Line_3D(Point_3D, Point_3D)
	 * That means construct a Line_3D out of two points.
	 */
	public void testLine_3DPoint_3DPoint_3D() {
		//covs for points:
		Matrix cov = LinearAlgebraFactory.getDiagMatrix(4,0.001);
		cov.set(3,3,0);
		//construct two points
		Point_3D p1 = new Point_3D(-2,3.5,4,cov);
		Point_3D p2 = new Point_3D(-5.4,-2.1,-5.1,cov);
		
		if(!helpLine_3DPoint_3DPoint_3DStatistics(p1,p2,0.3)) fail();
		
		//construct two more points
		cov = LinearAlgebraFactory.getDiagMatrix(4,0.001);
		cov.set(3,3,0);
		Point_3D p3 = new Point_3D(5212,4235.2,5361,cov);
		Point_3D p4 = new Point_3D(5232,5323,-2232,cov);
		
		if(!helpLine_3DPoint_3DPoint_3DStatistics(p3,p4,0.3)) fail();
	}

	/**
	 * Test for void Line_3D() 
	 */
	public void testLine_3D() {
		Line_3D l = new Line_3D();
		
		if(l == null) fail();
	}


	/**
	 * Test for checkPlueckerCondition()
	 * Tests with two lines. For the first line condition
	 * is not fulfilled so method should fail,
	 * the second fulfills condition, so it must not fail.
	 */
	public void testCheckPlueckerCondition() {
		//construct line that fails Pluecker condition test
		Line_3D fail = new Line_3D(1,2,3,4,5,6);
		
		//construct line that shouldn't fail the test
		Line_3D noFail = new Line_3D(-10,10,-5,-200,-150,100);
		
		//fail if something is wrong
		if(fail.checkPlueckerCondition(0.000001)) fail();
		if(!noFail.checkPlueckerCondition(0.000001)) fail();
	}

	/**
	 * Test for getDual()
	 * Tests calculation of the dual over one million points
	 * includes test for error propagation
	 */
	public void testGetDual() {
		//Progressmonitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
						"Testing getDual()","for one million lines",0,100*100*100);
				
		outer:
		for(int i = 0; i < 100; i++){
			for(int j = -50; j < 50; j++){
				for(int k = -100; k < 0; k++){
					//a covariance matrix:
					Matrix cov = LinearAlgebraFactory.getDiagMatrix(6,1/(i+1)); 
					
					//construct 3dline				
					Line_3D l = new Line_3D(i,j,k,i+j,j-k,k-i,cov);
					
					//let sugr calculate the dual line
					Line_3D ldSugr = l.getDual();
					
					//calculate the dual line
					Vector dual = Line_3D.DualMatrixC().mult(l.getV());
					//simple error propagation: sigma_y = A*sigma_x*A^T
					Matrix dualCov = Line_3D.DualMatrixC().mult(cov).mult(Line_3D.DualMatrixC().trans());
					
					//compare
					Vector zero = ldSugr.getV().minus(dual);
					Matrix zeroCov = dualCov.minus(ldSugr.getCov());
					
					if(zero.getLength() != 0){
						pm.close();
						fail();
					}
					if(zeroCov.vec().getLength() != 0) {
						pm.close();
						fail();
					}
					
					//break when cancel button is pressed
					if(pm.isCanceled()) break outer;
					//set progress
					pm.setProgress(i*(j+50)*(k+100));
				}
			}
		}
		pm.close();
	}
	
	/**
	 * Test for copy()
	 * Just copys a line, then changes original 
	 * to test whether it does a deep copy or not.
	 */
	public void testCopy() {
		//Construct Line
		Line_3D l = new Line_3D(43,73,12,52,54,54);
		
		//Copy Line
		Line_3D copy = l.copy();
		
		//change first line a little
		Vector v = l.getV();
		v.set(4, 56);
		l.setV(v);
		
		//compare values
		Vector two = l.getV().minus(copy.getV());
		if(two.getLength() != 2) fail();
	}

	/**
	 * Test for Matrix GammaDual(Vector)
	 * Lets sugr calculate gamma dual 
	 * then calculates the matrix
	 * and controls the results
	 */
	public void testGammaDualVector() {
		//construct line
		Line_3D l = new Line_3D(1,2,3,4,5,6);
		
		//get Gamma Dual Matrix from sugr.
		Matrix gammaDualSugr = Line_3D.GammaDual(l.getV());
		
		//construct Gamma Dual Matrix 
		double[][] values = {	{0,-3,2,4},
								{3,0,-1,5},
								{-2,1,0,6},
								{-4,-5,-6,0}};
		
		Matrix gammaDualCalc = LinearAlgebraFactory.getMatrix(values);
		
		//control if the are equal:
		Matrix zero = gammaDualSugr.minus(gammaDualCalc);
		if(zero.vec().getLength() != 0) fail();
	}

	/**
	 * Test for Matrix GammaDual()
	 * Does the same as testGammaDual(Vector).
	 */
	public void testGammaDual() {
		//construct line
		Line_3D l = new Line_3D(1,2,3,4,5,6);
		
		//get Gamma Dual Matrix from sugr.
		Matrix gammaDualSugr = l.GammaDual();
		
		//construct Gamma Dual Matrix 
		double[][] values = {	{0,-3,2,4},
								{3,0,-1,5},
								{-2,1,0,6},
								{-4,-5,-6,0}};
		
		Matrix gammaDualCalc = LinearAlgebraFactory.getMatrix(values);
		
		//control if the are equal:
		Matrix zero = gammaDualSugr.minus(gammaDualCalc);
		if(zero.vec().getLength() != 0) fail();
	}

	/**
	 * Test for Matrix Gamma(Vector)
	 */
	public void testGammaVector() {
		//construct line
		Line_3D l = new Line_3D(1,2,3,4,5,6);
		
		//get Gamma Dual Matrix from sugr.
		Matrix gammaSugr = Line_3D.Gamma(l.getV());
		
		//construct Gamma Dual Matrix 
		double[][] values = {	{0,6,-5,-1},
								{-6,0,4,-2},
								{5,-4,0,-3},
								{1,2,3,0}};
		
		Matrix gammaCalc = LinearAlgebraFactory.getMatrix(values);
		
		//control if the are equal:
		Matrix zero = gammaSugr.minus(gammaCalc);
		if(zero.vec().getLength() != 0) fail();
	}

	/**
	 * Test for Matrix Gamma()
	 */
	public void testGamma() {
		//construct line
		Line_3D l = new Line_3D(1,2,3,4,5,6);
		
		//get Gamma Dual Matrix from sugr.
		Matrix gammaSugr = l.Gamma();
		
		//construct Gamma Dual Matrix 
		double[][] values = {	{0,6,-5,-1},
								{-6,0,4,-2},
								{5,-4,0,-3},
								{1,2,3,0}};
		
		Matrix gammaCalc = LinearAlgebraFactory.getMatrix(values);
		
		//control if the are equal:
		Matrix zero = gammaSugr.minus(gammaCalc);
		if(zero.vec().getLength() != 0) fail();
	}

	/**
	 *	Test getLineLineJacobian(), used to calculate 
	 *	the jacobian for equaltest between two lines.
	 */
	public void testGetLineLineJacobian() {
		//dummy line
		Line_3D l = new Line_3D(1,2,3,4,5,6);
		
		//line line jacobian from sugr
		Matrix llSugr = l.getLineLineJacobian();
		
		//line line jacobian using JGamma from sugr, first equatation: (3.60 p.69)
		Matrix llCalc = (l.Gamma().trans().kronecker(
				LinearAlgebraFactory.getDiagMatrix(4,1))).mult(
						Line_3D.JGamma().mult(Line_3D.DualMatrixC()));
		

		if(llSugr.minus(llCalc).vec().getLength() != 0) fail();
	}


	/**
	 * Tests if JGamma is calculated properly
	 * Tests Matrix value by value,
	 * then tests if it fullfills vec(GammaDual(L) = JGamma*C*L  for a L.
	 * then tests if JGamma is usefull to calculate if two lines are equal,
	 * as shown in dissertation
	 */
	public void testJGamma() {
		//construct JGamma Matrix differs from the one in dissertation on page 69,
		//but does the job, the other one doesn't.
		double[][] values = {	{ 0, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 0,-1},
								{ 0, 0, 0, 0, 1, 0},
								{ 1, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 1},
								{ 0, 0, 0, 0, 0, 0},
								{ 0, 0, 0,-1, 0, 0},
								{ 0, 1, 0, 0, 0, 0},
								{ 0, 0, 0, 0,-1, 0},
								{ 0, 0, 0, 1, 0, 0},
								{ 0, 0, 0, 0, 0, 0},
								{ 0, 0, 1, 0, 0, 0},
								{-1, 0, 0, 0, 0, 0},
								{ 0,-1, 0, 0, 0, 0},
								{ 0, 0,-1, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 0}};
		/* valus from dissertation:
		double[][] values = {	{ 0, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 0},
								{ 0, 1, 0, 0, 0, 0},
								{ 0, 0, 0, 1, 0, 0},
								{ 0, 0, 1, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 0},
								{-1, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 1, 0},
								{ 0,-1, 0, 0, 0, 0},
								{ 1, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 0},
								{ 0, 0, 0, 0, 0, 1},
								{ 0, 0, 0,-1, 0, 0},
								{ 0, 0, 0, 0,-1, 0},
								{ 0, 0, 0, 0, 0,-1},
								{ 0, 0, 0, 0, 0, 0}};
		*/
		Matrix myJGamma = LinearAlgebraFactory.getMatrix(values);
		
		//get Matrix from Sugr
		Matrix sugrJGamma = Line_3D.JGamma();
		
		//compare
		Matrix zero = myJGamma.minus(sugrJGamma);
		
		//fail if matrix in sugr differs from the constructed one
		if(zero.vec().getLength() != 0) fail();	
		
		
		//test the constraint vec(GammaDual(L)=JGamma*C*L
		//construct a line
		Line_3D l = new Line_3D(-10,10,-5,-200,-150,100);
		//get vec(GammaDual(L)
		Vector vecGammaDual = l.GammaDual().vec();
		//get JGamma*C*L
		Vector JGammaCL = Line_3D.JGamma().mult(Line_3D.DualMatrixC().mult(l.getV()));

		//compare
		Vector zero2 = vecGammaDual.minus(JGammaCL);
		if(zero2.getLength() != 0) fail();
		
		//construct a second, equal line and test if
		//test returns that they are equal. see dissertation page 69.
		Line_3D l2 = new Line_3D(-20,20,-10,-400,-300,200);
		Matrix v = l2.Gamma().trans().kronecker(LinearAlgebraFactory.getDiagMatrix(4,1));
		v = v.mult(sugrJGamma.mult(Line_3D.DualMatrixC()));
		Vector zeroTest = v.mult(l.getV());
		
		//zero means that they are equal:
		if(zeroTest.getLength() != 0) fail();
	}

	/**
	 * Tests if the Matrix C is "calculated" properly
	 */
	public void testDualMatrixC() {
		//calc c:
		double[][] values = {	{0,0,0,-1,0,0},
								{0,0,0,0,-1,0},
								{0,0,0,0,0,-1},
								{-1,0,0,0,0,0},
								{0,-1,0,0,0,0},
								{0,0,-1,0,0,0}};
		Matrix cCalc = LinearAlgebraFactory.getMatrix(values);
		
		//get c from sugr
		Matrix cSugr = Line_3D.DualMatrixC();
		
		//compare
		Matrix zero = cCalc.minus(cSugr);
		if(zero.vec().getLength() != 0) fail();
	}

	/**
	 * Tests getDistanceToOrigin() over one million points
	 * uses the general formular for distance to points.
	 */
	public void testGetDistanceToOrigin() {
		//Progressmonitor
		TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
					"Testing getDistance","to origin",0,100*100*100);
		
		//origin:
		Point_3D p = new Point_3D(0,0,0);
		
		outer:
		for(int i = 0; i < 100; i++){
			for(int j = -150; j < -50;j++){
				for(int k = -100;k < 0; k++){
					//construct two lines and a point to get distance
					Line_3D l1 = new Line_3D(i,j,k,-j,-i-k,j);
					
					//calculate distance to origin
					Vector z1 = l1.getHomogeneousPart().crossProduct(p.getEuclideanPart());
					Vector z2 = l1.getEuclideanPart().mult(p.getHomogeneousPart().get(0));
					double zaehler = z1.plus(z2).getLength();
					Vector n1 = l1.getHomogeneousPart().mult(p.getHomogeneousPart().get(0));
					double nenner = n1.getLength();
					double distCalc = zaehler / nenner;
					
					//let sugr calculate distance to point
					double distSugr = l1.getDistanceToOrigin();
					
					//comapre
					if(distSugr != distCalc) {
						pm.close();
						fail(); 
					}
					
					//break if cancel button is pressed
					if(pm.isCanceled()) break outer;
					//update progress
					pm.setProgress(i*(j+150)*(k+100));
				}
			}
		}
		pm.close();
	}

}
