/*
 * Created on 28.06.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * @author Bastian Oehler
 *  
 */
public class Motion_2DTest extends TestCase {

	/**
	 * Constructor for Motion_2DTest.
	 * 
	 * @param arg0
	 */
	public Motion_2DTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Motion_2D mot = new Motion_2D();
		int dof = mot.getDegreesOfFreedom();
		if (dof != 3)
			fail();
	}

	/**
	 * Test for void Motion_2D()
	 */
	public void testMotion_2D() {
		Motion_2D mot = new Motion_2D();
		if (mot == null)
			fail();
	}

	/**
	 * Test for void Motion_2D(Translation_2D, Rotation_2D) Note that the
	 * statistical simulation of a varying Rotation needs a lot of iterations to
	 * get close to the covariance calculated by SUGR. The accuracy is not very
	 * good.
	 */
	public void testMotion_2DTranslation_2DRotation_2D() {
		int MAXITER = 1000000;//number of iterations for statistical test
		double[][] covArray = new double[9][9];
		//init covariance of Translation
		covArray[6][6] = 0.4;
		covArray[7][7] = 0.5;
		covArray[6][7] = 0.002;
		covArray[7][6] = 0.002;
		Matrix covMatTrans = LinearAlgebraFactory.getMatrix(covArray);
		//init Translation
		Translation_2D trans = new Translation_2D(3, 5, covMatTrans);
		//init RandomVector for statistical test
		int[] transList = { 6, 7 };
		Vector rvTransVec = LinearAlgebraFactory.getVector(3, 5);
		Matrix rvTransCov = LinearAlgebraFactory.getMatrix(covMatTrans.slice(
				transList, transList));
		RandomVector rvTrans = new RandomVector(rvTransVec, rvTransCov);
		//init covariance of Rotation
		covArray = new double[9][9];
		covArray[0][0] = 0.04;
		covArray[1][1] = 0.08;
		covArray[3][3] = 0.05;
		covArray[4][4] = 0.09;
		covArray[0][1] = covArray[1][0] = 0.00002;
		covArray[0][3] = covArray[3][0] = 0.000008;
		covArray[0][4] = covArray[4][0] = 0.00001;
		covArray[1][3] = covArray[3][1] = 0.000006;
		covArray[1][4] = covArray[4][1] = 0.00003;
		covArray[3][4] = covArray[4][3] = 0.000021;
		Matrix covMatRot = LinearAlgebraFactory.getMatrix(covArray);
		//init Rotation
		Rotation_2D rot = new Rotation_2D(1, covMatRot);
		//init RandomVector for statistical test
		int[] rotList = { 0, 1, 3, 4 };
		Vector rvRotVec = LinearAlgebraFactory.getVector(rot.getV().get(
				rotList[0]), rot.getV().get(rotList[1]), rot.getV().get(
				rotList[2]), rot.getV().get(rotList[3]));
		Matrix rvRotCov = LinearAlgebraFactory.getMatrix(covMatRot.slice(
				rotList, rotList));
		RandomVector rvRot = new RandomVector(rvRotVec, rvRotCov);
		//calculation done by SUGR
		Motion_2D mot = new Motion_2D(trans, rot);
		Matrix motMat = mot.getTransformationMatrix();
		//calculation by hand
		double[][] array = { { Math.cos(1), -Math.sin(1), 3 },
				{ Math.sin(1), Math.cos(1), 5 }, { 0, 0, 1 } };
		//check non statistic results
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 1; j++) {
				if (motMat.get(i, j) != array[i][j])
					fail();
			}
		}
		//statistical calculations
		Vector tmp;
		Vector tmpTrans = LinearAlgebraFactory.getVector(9);
		tmpTrans.set(0, 1);
		tmpTrans.set(4, 1);
		tmpTrans.set(8, 1);
		Vector tmpRot = LinearAlgebraFactory.getVector(9);
		tmpRot.set(8, 1);

		// First: vary only translation
		{
			TestStatistics stats = new TestStatistics(9);

			TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
					"Testing Motion_2D  (vary Translation)", "Statistics", 0,
					MAXITER);

			Translation_2D tr = new Translation_2D(3, 5, covMatTrans);
			Rotation_2D rt = new Rotation_2D(1, LinearAlgebraFactory.getMatrix(
					9, 9));
			mot = new Motion_2D(tr, rt);
			Motion_2D mt = new Motion_2D();
			for (int i = 0; i < MAXITER; i++) {
				tmp = rvTrans.getNextRandom();
				tmpTrans.set(6, tmp.get(0));
				tmpTrans.set(7, tmp.get(1));
				tr.setV(tmpTrans);
				mt = new Motion_2D(tr, rt);
				stats.addObservation(mt);
				if (pm.isCanceled())
					break;
				pm.setProgress(i);
			}
			pm.close();
			//mt: motion from statistical calculations here
			//mot: calculation done by SUGR
			mt.setV(stats.getMue());
			mt.setCov(stats.getSigma());
			//normalize results
			mt.normalizeSphereComparable();
			//System.out.println("statistik: "+mt);
			mot.normalizeSphereComparable();
			//System.out.println("SUGR: "+mot);
			//check results
			if(!TestStatistics.check(mt.getV(),mot.getV(),mt.getCov(),mot.getCov(),1e-1))
			    fail();
		}

		// And now vary rotation
		{
			TestStatistics stats = new TestStatistics(9);

			TestTools.ProgressMonitorWithTaskbar pm = new TestTools.ProgressMonitorWithTaskbar(
					"Testing Motion_2D (vary Rotation)", "Statistics", 0,
					MAXITER);

			Translation_2D tr = new Translation_2D(3, 5, LinearAlgebraFactory
					.getMatrix(9, 9));
			Rotation_2D rt = new Rotation_2D(1, covMatRot);
			mot = new Motion_2D(tr, rt);
			Motion_2D mt = new Motion_2D();
			for (int i = 0; i < MAXITER; i++) {
				tmp = rvRot.getNextRandom();
				tmpRot.set(0, tmp.get(0));
				tmpRot.set(1, tmp.get(1));
				tmpRot.set(3, tmp.get(2));
				tmpRot.set(4, tmp.get(3));
				rt.setV(tmpRot);
				mt = new Motion_2D(tr, rt);
				stats.addObservation(mt);
				if (pm.isCanceled())
					break;
				pm.setProgress(i);
			}
			pm.close();
			//mt: motion from statistical calculations here
			//mot: calculation done by SUGR
			mt.setV(stats.getMue());
			mt.setCov(stats.getSigma());
			//normalize results
			mt.normalizeSphereComparable();
			//System.out.println("statistik: " + mt);
			mot.normalizeSphereComparable();
			//System.out.println("SUGR: " + mot);
			//check results
			if (!TestStatistics.check(mt.getV(), mot.getV(), mt.getCov(), mot
					.getCov(), 0.3))
				fail();
		}

	}

	/**
	 * Test for void Motion_2D(Translation_2D, Rotation_2D, Matrix) no error
	 * propagation is tested, as covariance matrix is given
	 */
	public void testMotion_2DTranslation_2DRotation_2DMatrix() {
		double[][] covArray = new double[9][9];
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				if (i <= j)
					covArray[i][j] = Math.random();
				else
					covArray[i][j] = covArray[j][i];
			}
		}
		Matrix covMat = LinearAlgebraFactory.getMatrix(covArray);
		double[][] array = { { Math.cos(1), -Math.sin(1), 3 },
				{ Math.sin(1), Math.cos(1), 5 }, { 0, 0, 1 } };
		Translation_2D trans = new Translation_2D(3, 5);
		Rotation_2D rot = new Rotation_2D(1);
		Motion_2D mot = new Motion_2D(trans, rot, covMat);
		Matrix motMat = mot.getTransformationMatrix();
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 1; j++) {
				if (motMat.get(i, j) != array[i][j])
					fail();
			}
		}
		Matrix sugrCov = mot.getCov();
		for (int i = 0; i < 81; i++) {
			if (Math.abs((Math.abs(covMat.get(i / 9, i % 9)) - (Math
					.abs(sugrCov.get(i / 9, i % 9))))) > 1e-12)
				fail();
		}
	}

}