/*
 * Created on 12.07.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class Motion_3DTest extends TestCase {

	int MAXITER=100000;
	
	public Vector nineToSixteen(Vector nine){
		double[] dArr=new double[16];
		for(int i=0;i<15;i++){
			if(i!=3&&i!=7&&i<11)dArr[i]=nine.get(i-((i+1)/4));
			else dArr[i]=0;
		}
		dArr[15]=1;
		return LinearAlgebraFactory.getVector(dArr);
	}
	
	public Matrix nineToSixteen(Matrix nine){
		double[][] dArr=new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(i!=3&&i!=7&&i<11&&j!=3&&j!=7&&j<11)dArr[i][j]=nine.get(i-((i+1)/4),j-((j+1)/4));
				else dArr[i][j]=0;
			}
		}
		return LinearAlgebraFactory.getMatrix(dArr);
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Motion_3D mot = new Motion_3D();
		int dof = mot.getDegreesOfFreedom();
		if(dof!=6)fail();
	}

	/**
	 * Test for void Motion_3D()
	 */
	public void testMotion_3D() {
		Motion_3D mot = new Motion_3D();
		if(mot==null)fail();
	}

	/**
	 * Test for void Motion_3D(Translation_3D, Rotation_3D)
	 * vary Translation only
	 */
	public void testMotion_3DTranslation_3DRotation_3DvaryTrans() {
		//init Translation
		double[][] covArrTrans={{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0.2,0.05,0.02,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0.05,0.3,0.04,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0.02,0.04,0.5,0},
								{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}}; 
		Matrix covMatTrans=LinearAlgebraFactory.getMatrix(covArrTrans);
		Vector vecTransConstr=LinearAlgebraFactory.getVector(17,29,45);
		Translation_3D trans=new Translation_3D(vecTransConstr,covMatTrans);
		//init Rotation
		Matrix covMatRot=LinearAlgebraFactory.getMatrix(16,16);
		Rotation_3D rot = new Rotation_3D(45,17,23,covMatRot);
		//get result from SUGR
		Motion_3D mot = new Motion_3D(trans,rot);
		//calculating from Matrices
		Matrix motMatHand=rot.getTransformationMatrix().mult(trans.getTransformationMatrix());
		Matrix motMatSugr=mot.getTransformationMatrix();
		//check results
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(motMatHand.get(i,j)!=motMatSugr.get(i,j))fail();
			}
		}
		
		//statistical test
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing Motion_3D",
					"Statistics (vary Translation)",0,MAXITER);
		TestStatistics stats = new TestStatistics(16);
		double[][] covMatStats={{0.2,0.05,0.02},{0.05,0.3,0.04},{0.02,0.04,0.5}};
		RandomVector rvTrans = new RandomVector(vecTransConstr,LinearAlgebraFactory.getMatrix(covMatStats));
		Translation_3D transStats = new Translation_3D();
		Motion_3D motStats = new Motion_3D();
		for(int i=0;i<MAXITER;i++){
			transStats=new Translation_3D(rvTrans.getNextRandom(),trans.getCov());
			motStats = new Motion_3D(transStats,rot);
			stats.addObservation(motStats);
			if(pm.isCanceled())break;
			pm.setProgress(i);
		}
		pm.close();
		motStats.setV(stats.getMue());
		motStats.setCov(stats.getSigma());
		motStats.normalizeSphereComparable();
		mot.normalizeSphereComparable();
		//System.out.println("Statistics: " + motStats);
		//System.out.println("SUGR: " + mot);
		if(!TestStatistics.check(mot.getV(),motStats.getV(),mot.getCov(),motStats.getCov(),0.3))fail();
	}

	/**
	 * Test for void Motion_3D(Translation_3D, Rotation_3D)
	 * vary Rotation only
	 * TODO: This Test is still failing...
	 */
	public void testMotion_3DTranslation_3DRotation_3DvaryRot(){
		//init Translation
		Matrix covMatTrans=LinearAlgebraFactory.getMatrix(16,16);
		Vector vecTransConstr=LinearAlgebraFactory.getVector(7,9,5);
		Translation_3D trans=new Translation_3D(vecTransConstr,covMatTrans);
		//init Rotation
//		double[][] covArrRot=new double[9][9];
//		for(int i=0;i<9;i++){
//			for(int j=0;j<9;j++){
//				if(i<j)covArrRot[i][j] = Math.random()*1e-4;
//				else if (i==j)covArrRot[i][j] = Math.random()*1e-2;
//				else covArrRot[i][j] =covArrRot[j][i];
//			}
//		}
		double[][]covArrRot={{0.012,0.00050,0.00048,0.00016,0.00022,0.00026,0.00033,0.00043,0.00041}
							,{0.00050,0.011,0.00047,0.00031,0.00036,0.00025,0.00032,0.00029,0.00037}
							,{0.00048,0.00047,0.015,0.00031,0.00027,0.00019,0.00028,0.00034,0.00050}
							,{0.00016,0.00031,0.00031,0.017,0.00042,0.00021,0.00034,0.00016,0.00029}
							,{0.00022,0.00036,0.00027,0.00042,0.019,0.00044,0.00013,0.00018,0.00034}
							,{0.00026,0.00025,0.00019,0.00021,0.00044,0.012,0.00023,0.00043,0.00016}
							,{0.00033,0.00032,0.00028,0.00034,0.00013,0.00023,0.016,0.00025,0.00048}
							,{0.00043,0.00029,0.00034,0.00016,0.00018,0.00043,0.00025,0.014,0.00037}
							,{0.00041,0.00037,0.00050,0.00029,0.00034,0.00016,0.00048,0.00037,0.013}};
		//with random covariance for the nine entries of rot.getV() not being zero --> 9x9 covariance Matrix
		Matrix covMatRot9=LinearAlgebraFactory.getMatrix(covArrRot);
		//and the three angles 1,2,3; making 16x16 covariance Matrix with some zero-rows/columns 
		Rotation_3D rot = new Rotation_3D(1,2,3,nineToSixteen(covMatRot9));
		//get result from SUGR; construct Motion_3D
		Motion_3D mot = new Motion_3D(trans,rot);
		//calculating from Matrices; calulate Motion_3D (Vector, not covariance)
		Matrix motMatHand=rot.getTransformationMatrix().mult(trans.getTransformationMatrix());
		Matrix motMatSugr=mot.getTransformationMatrix();
		//check results
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(motMatHand.get(i,j)!=motMatSugr.get(i,j))fail();
			}
		}

		//statistical test
		//testing all 9 parameters of rotation requires too many iterations,
//		//init taskbar
//		TestTools.ProgressMonitorWithTaskbar pm =
//			new TestTools.ProgressMonitorWithTaskbar("Testing Motion_3D",
//					"Statistics (vary Rotation)",0,MAXITER);
//		TestStatistics stats = new TestStatistics(16);
//		//get those 9 entries from Vector rot.getV(), being different from "0" or "1"
//		Vector rotVec9=LinearAlgebraFactory.getVector(9);
//		rotVec9.set(0,rot.getV().get(0));
//		rotVec9.set(1,rot.getV().get(1));
//		rotVec9.set(2,rot.getV().get(2));
//		rotVec9.set(3,rot.getV().get(4));
//		rotVec9.set(4,rot.getV().get(5));
//		rotVec9.set(5,rot.getV().get(6));
//		rotVec9.set(6,rot.getV().get(8));
//		rotVec9.set(7,rot.getV().get(9));
//		rotVec9.set(8,rot.getV().get(10));
//		//init random Vector with 9x9 covariance Matrix
//		RandomVector rvRot = new RandomVector(rotVec9,covMatRot9);
//		//empty construction for later use
//		Motion_3D motStats = new Motion_3D();
//		for(int i=0;i<MAXITER;i++){
//			//get random 9-Vector and make 16-Vector from it  
//			rot.setV(nineToSixteen(rvRot.getNextRandom()));
//			//construct Motion_3D from non-unsure translation and rotation with covariance  
//			motStats = new Motion_3D(trans,rot);
//			//add new constructed Motion_3D as Observation
//			stats.addObservation(motStats);
//			if(pm.isCanceled())break;
//			pm.setProgress(i);
//		}
//		pm.close();
//		//get result from Observation
//		motStats.setV(stats.getMue());
//		motStats.setCov(stats.getSigma());
//		//normalize results
//		motStats.normalizeSphereComparable();
//		mot.normalizeSphereComparable();
//		if(!TestStatistics.check(mot.getV(),motStats.getV(),mot.getCov(),motStats.getCov(),0.3)){
//			System.out.println("Statistics: " + motStats);
//			System.out.println("SUGR: " + mot);
//			fail();
//		}
		//thus testing each parameter alone
		//this is no longer a Rotation, but if it is not failing for any of this Matrices
		//it should not be failing any other Matrices (when given enough iterations)
		
//		//first test without any correlations bt with variance in each parameter
//		//init taskbar
//		TestTools.ProgressMonitorWithTaskbar pm =
//			new TestTools.ProgressMonitorWithTaskbar("Testing Motion_3D",
//					"Statistics (vary Rotation)",0,9*MAXITER);
//		//As only one parameter is uncertain our RandomVector is 1-dimensional
//		Vector rotVec9=LinearAlgebraFactory.getVector(9);
//		rotVec9.set(0,rot.getV().get(0));
//		rotVec9.set(1,rot.getV().get(1));
//		rotVec9.set(2,rot.getV().get(2));
//		rotVec9.set(3,rot.getV().get(4));
//		rotVec9.set(4,rot.getV().get(5));
//		rotVec9.set(5,rot.getV().get(6));
//		rotVec9.set(6,rot.getV().get(8));
//		rotVec9.set(7,rot.getV().get(9));
//		rotVec9.set(8,rot.getV().get(10));
//		Vector rotVec=LinearAlgebraFactory.getVector(1);
//		rotVec.set(0,rotVec9.get(0));
//		double[][] sigmaArr = {{0.001}};
//		Matrix sigma=LinearAlgebraFactory.getMatrix(sigmaArr);
//		//init random Vector with sigma
//		RandomVector rvRot; 
//		//empty construction for later use
//		TestStatistics stats;
//		Motion_3D motStats = new Motion_3D();
//		Vector zero;
//		Matrix cov;
//		Rotation_3D rotStats = new Rotation_3D();
//		//iterating through all 9 parameters of rotations
//		for(int j=0;j<9;j++){
//			stats = new TestStatistics(16);
//			//set Vector of Rotation for j-th Iteration
//			zero=LinearAlgebraFactory.getVector(9);
//			zero.set(j,rotVec9.get(j));
//			rotStats.setV(nineToSixteen(zero));
//			//construct new RandomVector
//			rotVec.set(0,rotVec9.get(j));
//			rvRot = new RandomVector(rotVec,sigma);
//			//set covariance Matrix for j-th Iteration
//			cov = LinearAlgebraFactory.getMatrix(9,9);
//			cov.set(j,j,sigmaArr[0][0]);
//			rotStats.setCov(nineToSixteen(cov));
//			//reconstruct mot with j-th Rotation
//			mot = new Motion_3D(trans,rotStats);
//			for(int i=0;i<MAXITER;i++){
//				//get random 1-Vector and make 16-Vector from it
//				zero = LinearAlgebraFactory.getVector(9);
//				//vary j-th parameter
//				zero.set(j,rvRot.getNextRandom());
//				rotStats.setV(nineToSixteen(zero));
//				//construct Motion_3D from certain translation and rotation with one uncertain parameter  
//				motStats = new Motion_3D(trans,rotStats);
//				//add new constructed Motion_3D as Observation
//				stats.addObservation(motStats);
//				if(pm.isCanceled()){j+=9;break;};
//				pm.setProgress(i+j*MAXITER);
//			}
//			//get result from Observation
//			motStats.setV(stats.getMue());
//			motStats.setCov(stats.getSigma());
//			//normalize results
//			motStats.normalizeSphereComparable();
//			mot.normalizeSphereComparable();
//			if(!TestStatistics.check(mot.getV(),motStats.getV(),mot.getCov(),motStats.getCov(),0.3)){
//				System.out.println("vary Rotation in "+j+"-th parameter: ");
//				System.out.println("Statistics: " + motStats);
//				System.out.println("SUGR: " + mot);
//				fail();
//			}
//		}
//		pm.close();

		//test with variances and correlation between neighbouring elements
		//init taskbar
		TestTools.ProgressMonitorWithTaskbar pm =
			new TestTools.ProgressMonitorWithTaskbar("Testing Motion_3D",
					"Statistics (vary Rotation)",0,8*MAXITER);
		//Vector with 9 parameters of rotation not equal to zero
		Vector rotVec9=LinearAlgebraFactory.getVector(9);
		rotVec9.set(0,rot.getV().get(0));
		rotVec9.set(1,rot.getV().get(1));
		rotVec9.set(2,rot.getV().get(2));
		rotVec9.set(3,rot.getV().get(4));
		rotVec9.set(4,rot.getV().get(5));
		rotVec9.set(5,rot.getV().get(6));
		rotVec9.set(6,rot.getV().get(8));
		rotVec9.set(7,rot.getV().get(9));
		rotVec9.set(8,rot.getV().get(10));
		//As only two parameters are uncertain our RandomVector is 2-dimensional
		//Vector for construction of RandomVector
		Vector rotVec=LinearAlgebraFactory.getVector(2);
		//setting up covariance Matrix for RandomVector
		double[][] covArr={{0.00001,0.000001},{0.000001,0.00001}};
		Matrix RVCov=LinearAlgebraFactory.getMatrix(covArr);
		//empty constructions for later use
		TestStatistics stats;
		RandomVector rvRot;
		Motion_3D motStats = new Motion_3D();
		Vector zero;
		Matrix cov;
		Rotation_3D rotStats = new Rotation_3D();
		//iterating through all 8 cases of neighbouring parameters in rotation
		for(int j=0;j<8;j++){
			stats = new TestStatistics(16);
			//set Vector of Rotation for j-th Iteration
			zero=LinearAlgebraFactory.getVector(9);
			//set two entries different from zero
			zero.set(j,rotVec9.get(j));
			zero.set(j+1,rotVec9.get(j+1));
			//set vector of rotation for statistics
			rotStats.setV(nineToSixteen(zero));
			//set covariance Matrix for j-th Iteration
			cov = LinearAlgebraFactory.getMatrix(9,9);
			cov.set(j,j,covArr[0][0]);
			cov.set(j,j+1,covArr[0][1]);
			cov.set(j+1,j,covArr[1][0]);
			cov.set(j+1,j+1,covArr[1][1]);
			rotStats.setCov(nineToSixteen(cov));
			//reconstruct mot with j-th Rotation
			//for comparison with statistics
			mot = new Motion_3D(trans,rotStats);
			//construct new RandomVector
			rotVec.set(0,rotVec9.get(j));
			rotVec.set(1,rotVec9.get(j+1));
			rvRot = new RandomVector(rotVec,RVCov);
			//iteration for statistics
			for(int i=0;i<MAXITER;i++){
				//get random 2-Vector and make 16-Vector from it
				zero = LinearAlgebraFactory.getVector(9);
				//vary j-th parameter, set Vector of rotStats also working with 2-dim. vectors
				zero.set(j,rvRot.getNextRandom());
				rotStats.setV(nineToSixteen(zero));
				//construct Motion_3D from certain translation and rotation with two uncertain parameters  
				motStats = new Motion_3D(trans,rotStats);
				motStats.normalizeSphereComparable();
				//add new constructed Motion_3D as Observation
				stats.addObservation(motStats);
				if(pm.isCanceled()){j+=9;break;};
				pm.setProgress(i+j*MAXITER);
			}
			//get result from observations
			motStats.setV(stats.getMue());
			motStats.setCov(stats.getSigma());
			//normalize results
			motStats.normalizeSphereComparable();
			mot.normalizeSphereComparable();
			//check results
			if(!TestStatistics.check(mot.getV(),motStats.getV(),mot.getCov(),motStats.getCov(),0.3)){
				System.out.println("vary Rotation in "+j+"-th parameter: ");
				System.out.println("Statistics: " + motStats);
				System.out.println("SUGR: " + mot);
				fail();
			}
		}
		pm.close();
	}

	/**
	 * Test for void Motion_3D(Translation_3D, Rotation_3D, Matrix)
	 */
	public void testMotion_3DTranslation_3DRotation_3DMatrix() {
		Translation_3D trans = new Translation_3D(1,2,3);
		Rotation_3D rot = new Rotation_3D(4,5,6);
        //array for covariance Matrix
        double[][] covArray=new double[16][16];
        //random Values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else covArray[i][j]=covArray[j][i];
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		Motion_3D mot = new Motion_3D(trans,rot,covMat);
		Matrix motMatHand=trans.getTransformationMatrix().mult(rot.getTransformationMatrix());
		Matrix motMatSugr=mot.getTransformationMatrix();
		for(int i=0;i<3;i++){
			for(int j=0;j<1;j++){
				if(motMatHand.get(i,j)!=motMatSugr.get(i,j))fail();
			}
		}
		Matrix covMatSugr=mot.getCov();
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(Math.abs(Math.abs(covMatSugr.get(i,j))-Math.abs(covMat.get(i,j)))>1e-12)fail();
			}
		}
	}

}
