/*
 * Created on 15.06.2004
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * Test for Orthogonal Relation.
 * @author Achim Knigs
 */
public class OrthogonalTest extends TestCase {

	/**
	 * Constructor for OrthogonalTest.
	 * @param arg0
	 */
	public OrthogonalTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for getJacobian 
	 * for Orthogonal between lines and planes
	 */
	public void testGetJacobian() {
		//Line_2D to Line_2D:
		//L_h^T
		Line_2D l2D = new Line_2D(1,2,3);
		Orthogonal ortho = new Orthogonal(l2D,Line_2D.class);
		
		//calc Jacobian and compare it by value to the one from sugr
		double[][] l2DValues = {{1,2,0}};
		Matrix l2DJac = LinearAlgebraFactory.getMatrix(l2DValues);
		double l2DZero = l2DJac.minus(ortho.getJacobian()).vec().getLength();
		
		if(l2DZero != 0) fail();
		
		//Line_3D to Line_3D:
		//L_h^T
		Line_3D l3D = new Line_3D(1,2,3,4,5,6);
		ortho = new Orthogonal(l3D,Line_3D.class);
		
		//calc Jacobian and compare it by calue to the one from sugr 
		double[][] l3DValues = {{1,2,3,0,0,0}};
		Matrix l3DJac = LinearAlgebraFactory.getMatrix(l3DValues);
		double l3DZero = l3DJac.minus(ortho.getJacobian()).vec().getLength();
		
		if(l3DZero != 0) fail();
		
		//Plane_3D to Plane_3D:
		Plane_3D plane = new Plane_3D(1,2,3,4);
		ortho = new Orthogonal(plane,Plane_3D.class);
		
		//calc Jacobian and compare it by calue to the one from sugr
		//A_h^T
		double[][] planeValues = {{1,2,3,0}};
		Matrix planeJac = LinearAlgebraFactory.getMatrix(planeValues);
		double planeZero = planeJac.minus(ortho.getJacobian()).vec().getLength();
		
		if(planeZero != 0) fail();
		
		//Plane_3D to Line_3D:
		ortho = new Orthogonal(plane,Line_3D.class);
		
		//calc Jacobian and compare it by calue to the one from sugr
		//-S(A_h)
		double[][] plValues = {	{0,-3,2,0},
								{3,0,-1,0},
								{-2,1,0,0},
								{0,0,0,0},
								{0,0,0,0},
								{0,0,0,0}};
		Matrix plJac = LinearAlgebraFactory.getMatrix(plValues).trans();
		double plZero = plJac.minus(ortho.getJacobian()).vec().getLength();
		
		if(plZero != 0) fail();
		
		//Line_3D to Plane_3D:
		ortho = new Orthogonal(l3D,Plane_3D.class);
		
		//calc Jacobian and compare it by calue to the one from sugr
		//S(L_h)
		double[][] lpValues = {	{0,-3,2,0},
								{3,0,-1,0},
								{-2,1,0,0},
								{0,0,0,0}};
		Matrix lpJac = LinearAlgebraFactory.getMatrix(lpValues);
		double lpZero = lpJac.minus(ortho.getJacobian()).vec().getLength();
		
		if(lpZero != 0) fail();
	}
	
	/**
	 * Test for getDof
	 */
	public void testGetDof() {
		//construct some entities
		Orthogonal l2 = new Orthogonal(new Line_2D(1,2,3),Line_2D.class);
		Orthogonal l3 = new Orthogonal(new Line_3D(5,3,2,-1,1,1),Line_3D.class);
		Orthogonal plane = new Orthogonal(new Plane_3D(1,2,34,4),Plane_3D.class);
		Orthogonal pl = new Orthogonal(new Line_3D(5,3,2,-1,1,1),Plane_3D.class);
		
		//test dofs:
		if(l2.getDof() != 1) fail();
		if(l3.getDof() != 1) fail();
		if(plane.getDof() != 1) fail();
		if(pl.getDof() != 2) fail();
	}

	/**
	 * Test for void Orthogonal(Line_2D, Class)
	 * This tests a Constructor of Orthogonal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testOrthogonalLine_2DClass() {
		//construct Orthogonal
		Line_2D l = new Line_2D(1,2,4,
				LinearAlgebraFactory.getDiagMatrix(3,0.01));
		//second parameter has to be Line_2D.class
		Orthogonal ortho = new Orthogonal(l,Line_2D.class);
		
		if(!TestStatistics.check(ortho.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for void Orthogonal(Line_3D, Class)
	 * This tests a Constructor of Orthogonal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testOrthogonalLine_3DClass() {
		//construct Orthogonal
		Line_3D l = new Line_3D(2,2,4,1,1,-1,
				LinearAlgebraFactory.getDiagMatrix(6,0.01));
		//second parameter has to be Line_3D.class or Plane_3D.class.
		Orthogonal ortho = new Orthogonal(l,Line_3D.class);
		
		if(!TestStatistics.check(ortho.getFirstEntity(),l,1e-17)) fail();
	}

	/**
	 * Test for void Orthogonal(Plane_3D, Class)
	 * This tests a Constructor of Orthogonal, which only stores
	 * the given Entity in a RelationalProperty.
	 * If you are looking for more tests of this Relation 
	 * look in RelationTests.testDecision()
	 * and the getJacobianOfRelation() tests.
	 */
	public void testOrthogonalPlane_3DClass() {
		//construct Orthogonal
		Plane_3D p = new Plane_3D(1,2,4,7,
				LinearAlgebraFactory.getDiagMatrix(4,0.01));
		//second parameter has to be Line_3D.class or Plane_3D.class.
		Orthogonal ortho = new Orthogonal(p,Line_3D.class);
		
		if(!TestStatistics.check(ortho.getFirstEntity(),p,1e-17)) fail();
	}

}
