/*
 * Created on 28.06.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import junit.framework.TestCase;

/**
 * @author Bastian Oehler
 *
 */
public class Rotation_2DTest extends TestCase {

	/**
	 * Constructor for Rotation_2DTest.
	 * @param arg0
	 */
	public Rotation_2DTest(String arg0) {
		super(arg0);
	}

	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		Rotation_2D rot = new Rotation_2D();
		int dof = rot.getDegreesOfFreedom();
		if(dof!=1)fail();
	}

	/**
	 * Test for void Rotation_2D()
	 */
	public void testRotation_2D() {
		Rotation_2D rot = new Rotation_2D();
		if(rot==null)fail();
	}

	/**
	 * Test for void Rotation_2D(double)
	 */
	public void testRotation_2Ddouble() {
		double d = Math.PI+2.11192;
		Rotation_2D rot = new Rotation_2D(d);
		Matrix sugrRot = rot.getTransformationMatrix();
		double[][] handArray = {{Math.cos(d),-Math.sin(d),0},{Math.sin(d),Math.cos(d),0},{0,0,1}};
		Matrix handRot = LinearAlgebraFactory.getMatrix(handArray);
		for (int i=0;i<9;i++){
			if(Math.abs((Math.abs(handRot.get(i/3,i%3))-(Math.abs(sugrRot.get(i/3,i%3)))))>1e-12)fail();
		}
	}

	/**
	 * Test for void Rotation_2D(double, Matrix)
	 */
	public void testRotation_2DdoubleMatrix() {
		double d = Math.PI+2.11192;
		double[][] covArray = new double[9][9];
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(i<=j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		Rotation_2D rot = new Rotation_2D(d,covMat);
		Matrix sugrRot = rot.getTransformationMatrix();
		double[][] handArray = {{Math.cos(d),-Math.sin(d),0},{Math.sin(d),Math.cos(d),0},{0,0,1}};
		Matrix handRot = LinearAlgebraFactory.getMatrix(handArray);
		for (int i=0;i<9;i++){
			if(Math.abs((Math.abs(handRot.get(i/3,i%3))-(Math.abs(sugrRot.get(i/3,i%3)))))>1e-12)fail();
		}
		Matrix sugrCov = rot.getCov();
		for (int i=0;i<81;i++){
			if(Math.abs((Math.abs(covMat.get(i/9,i%9))-(Math.abs(sugrCov.get(i/9,i%9)))))>1e-12)fail();
		}
	}

	/**
	 * Test for void Rotation_2D(double, double)
	 */
	public void testRotation_2Ddoubledouble() {
		double angle = 3.5539116;
		double a=Math.cos(angle);
		double b=Math.sin(angle);
		Rotation_2D rot = new Rotation_2D(a,b);
		Matrix sugr2=rot.getTransformationMatrix();
		Matrix sugr1=(new Rotation_2D(angle)).getTransformationMatrix();
		for (int i=0;i<9;i++){
			if(Math.abs((Math.abs(sugr1.get(i/3,i%3))-(Math.abs(sugr2.get(i/3,i%3)))))>1e-12)fail();
		}
	}

	/**
	 * Test for void Rotation_2D(double, double, Matrix)
	 */
	public void testRotation_2DdoubledoubleMatrix() {
		double angle = 2.431218;
		double a=Math.cos(angle);
		double b=Math.sin(angle);
		double[][] covArray = new double[9][9];
		for(int i=0;i<3;i++){
			for(int j=0;j<3;j++){
				if(i<=j)covArray[i][j]=Math.random();
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		Rotation_2D rot = new Rotation_2D(a,b,covMat);
		Matrix sugr2=rot.getTransformationMatrix();
		Matrix sugr1=(new Rotation_2D(angle)).getTransformationMatrix();
		for (int i=0;i<9;i++){
			if(Math.abs((Math.abs(sugr1.get(i/3,i%3))-(Math.abs(sugr2.get(i/3,i%3)))))>1e-12)fail();
		}
		Matrix sugr2Cov = rot.getCov();
		for (int i=0;i<81;i++){
			if(Math.abs((Math.abs(covMat.get(i/9,i%9))-(Math.abs(sugr2Cov.get(i/9,i%9)))))>1e-12)fail();
		}
	}

}
