/*
 * Created on 21.03.2005
 *
 */
package sugr.test;

import java.util.Random;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class ScaledMotion_3DTest extends TestCase {
	/**
	 * Test for int getDegreesOfFreedom()
	 */
	public void testGetDegreesOfFreedom() {
		ScaledMotion_3D sm=new ScaledMotion_3D();
		if(sm.getDegreesOfFreedom()!=7)fail();
	}

	/**
	 * Test for void ScaledMotion_3D()
	 */
	public void testScaledMotion_3D() {
		ScaledMotion_3D sm=new ScaledMotion_3D();
		if(sm==null)fail();
	}

	/**
	 * Test for void ScaledMotion_3D(Translation_3D, Rotation_3D, double)
	 * check covariance matrix from Translation and Rotation (through Motion_3D)
	 */
	public void testScaledMotion_3DTranslation_3DRotation_3Ddouble() {
		//set up scale, scaleMat
		double scale = 2.34;
		//init Translation
		double[][] covArrayTrans = new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(i<j)covArrayTrans[i][j] = Math.random()*1e-4;
				else if (i==j)covArrayTrans[i][j] = Math.random()*1e-2;
				else covArrayTrans[i][j] =covArrayTrans[j][i];
			}
		}
		Matrix covMatTrans = LinearAlgebraFactory.getMatrix(covArrayTrans);
		Translation_3D trans = new Translation_3D(1,2,3,covMatTrans);
		//init Rotation
		double[][] covArrayRot = new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(i<j)covArrayRot[i][j] = Math.random()*1e-4;
				else if (i==j)covArrayRot[i][j] = Math.random()*1e-2;
				else covArrayRot[i][j] =covArrayRot[j][i];
			}
		}
		Matrix covMatRot = LinearAlgebraFactory.getMatrix(covArrayRot);
		Rotation_3D rot = new Rotation_3D(1,2,3,covMatRot);
		//get result from SUGR Motion_3D
		Motion_3D mot = new Motion_3D(trans,rot);
		//calculate scale
		Matrix motScale=mot.getTransformationMatrix();
		double[][] scaleArray={{scale,0,0,0},{0,scale,0,0},{0,0,scale,0},{0,0,0,1}};
		Matrix scaleMat=LinearAlgebraFactory.getMatrix(scaleArray);
		mot.setV(motScale.mult(scaleMat).vec());
		//get result from SUGR ScaledMotion_3D
		ScaledMotion_3D sm=new ScaledMotion_3D(trans,rot,scale);
		//check results
		mot.normalizeSphereComparable();
		sm.normalizeSphereComparable();
		if(!TestStatistics.check(mot.getV(),sm.getV(),mot.getCov(),sm.getCov(),1e-5))fail();
	}

	/**
	 * Test for void ScaledMotion_3D(Translation_3D, Rotation_3D, double, Matrix)
	 * no Test for covariance is needed as it is set by user
	 */
	public void testScaledMotion_3DTranslation_3DRotation_3DdoubleMatrix() {
		//set up random covariance matrix
		double[][] covArray=new double[16][16];
		for(int i=0;i<16;i++){
			for(int j=0;j<16;j++){
				if(i<j)covArray[i][j]=Math.random()*1e-4;
				else if (i==j)covArray[i][j]=Math.random()*1e-2;
				else covArray[i][j]=covArray[j][i];
			}
		}
		Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
		//set up scale, scaleMat
		double scale = 2.34;
		//init Translation
		Translation_3D trans = new Translation_3D(1,2,3);
		//init Rotation
		Rotation_3D rot = new Rotation_3D(1,2,3);
		//get result from SUGR Motion_3D
		Motion_3D mot = new Motion_3D(trans,rot,covMat);
		//calculate scale
		Matrix motScale=mot.getTransformationMatrix();
		double[][] scaleArray={{scale,0,0,0},{0,scale,0,0},{0,0,scale,0},{0,0,0,1}};
		Matrix scaleMat=LinearAlgebraFactory.getMatrix(scaleArray);
		mot.setV(motScale.mult(scaleMat).vec());
		//get result from SUGR ScaledMotion_3D
		ScaledMotion_3D sm=new ScaledMotion_3D(trans,rot,scale,covMat);
		//check results
		mot.normalizeSphereComparable();
		sm.normalizeSphereComparable();
		if(!TestStatistics.check(mot.getV(),sm.getV(),mot.getCov(),sm.getCov(),1e-5))fail();
	}
	
	/** Test the special approximation function and the estimation function with
	 * points
	 *
	 */
	public void testApproximatePoints() {
		//Maximal distance of vec(Target) and vec(Approximation) and vec(Estimation)
		double maxdisttrans_app=1.0;
		double maxdisttrans_est=0.4;
		
		// 6 test points
		double sigma = 0.05;
		double[][] val = { { sigma * sigma, 0, 0, 0 },
				{ 0, sigma * sigma, 0, 0 }, { 0, 0, sigma * sigma, 0 },
				{ 0, 0, 0, 0 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(val);

		Point_3D[] p = new Point_3D[6];

		p[0] = new Point_3D(100, 0, 0, cov);
		p[1] = new Point_3D(0, -100, 0, cov);
		p[2] = new Point_3D(0, 0, 100, cov);
		p[3] = new Point_3D(100, 50, 0, cov);
		p[4] = new Point_3D(0, 200, 100, cov);
		p[5] = new Point_3D(100, 200, 300, cov);

		Motion_3D target = new Motion_3D(new Translation_3D(1, -2, 5),
				new Rotation_3D(Math.PI / 2, 1.2, -0.5), LinearAlgebraFactory
						.getDiagMatrix(16, 0.0));

		Point_3D[] pStrich = new Point_3D[6];
		for (int i = 0; i < 6; i++)
			pStrich[i] = (Point_3D) target.transform(p[i]);

		// add noise
		Random ran = new Random();
		for (int i = 0; i < pStrich.length; i++) {
			double[] noise = { ran.nextGaussian() * sigma,
					ran.nextGaussian() * sigma, ran.nextGaussian() * sigma, 0 };
			pStrich[i].setV((pStrich[i].getV()).plus(LinearAlgebraFactory
					.getVector(noise)));
		}

		// Test with 4 to 6 points
		// With 3 points the approximation does not work, because the approximation
		// calculation an affinity with 12 parameters !!!
		for (int np = 4; np <= 6; np++) {
			RelationalProperty[] rprops = new RelationalProperty[np];
			for (int i = 0; i < np; i++)
				rprops[i] = new TriRelationalProperty(p[i], pStrich[i],
						Motion_3D.class);

			// estimate
			Motion_3D est = new Motion_3D();

			est.approximate(rprops);

			// Output
			//System.out.println("========================================");
			//System.out.println("Test with " + np + " points");
			Matrix T = target.getTransformationMatrix();
			T = T.mult(1 / T.get(3, 3));
			//System.out.println("Target:" + T);

			Matrix Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			double d= (Te.vec().minus(T.vec()).normF());			
			//System.out.println("Approximation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d + "\n\n");
			if (d>maxdisttrans_app) fail();

			est.estimate(rprops);

			Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			d= (Te.vec().minus(T.vec()).normF());
			//System.out.println("Estimation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d);
			if (d>maxdisttrans_est) fail();

		}

	}

	/** Test the special approximation function and the estimation function with
	 *  planes
	 *
	 */
	public void testApproximatePlanes() {
		//Maximal distance of vec(Target) and vec(Approximation) and vec(Estimation)
		double maxdisttrans=0.03;

		// 6 test planes
		double sigma = 1e-2;
		double[][] val = { { sigma * sigma, 0, 0, 0 },
				{ 0, sigma * sigma, 0, 0 }, { 0, 0, sigma * sigma, 0 },
				{ 0, 0, 0, 0 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(val);

		Plane_3D[] p = new Plane_3D[6];

		p[0] = new Plane_3D(100, 10, 20,3, cov);
		p[1] = new Plane_3D(0, -100, 1,3, cov);
		p[2] = new Plane_3D(0, 6, 100,-2, cov);
		p[3] = new Plane_3D(100, 50, 0,20, cov);
		p[4] = new Plane_3D(0, 200, 100,200, cov);
		p[5] = new Plane_3D(100, 200, 300, 40, cov);

		Motion_3D target = new Motion_3D(new Translation_3D(1, -2, 5),
				new Rotation_3D(Math.PI / 2, 1.2, -0.5), LinearAlgebraFactory
						.getDiagMatrix(16, 0.0));

		Plane_3D[] pStrich = new Plane_3D[6];
		for (int i = 0; i < 6; i++)
			pStrich[i] = (Plane_3D) target.transform(p[i]);

		// add noise
		Random ran = new Random();
		for (int i = 0; i < pStrich.length; i++) {
			double[] noise = { ran.nextGaussian() * sigma,
					ran.nextGaussian() * sigma, ran.nextGaussian() * sigma, 0 };
			pStrich[i].setV((pStrich[i].getV()).plus(LinearAlgebraFactory
					.getVector(noise)));
		}

		// Test with 4 to 6 planes
		// With 3 points the approximation does not work, because the approximation
		// calculation an affinity with 12 parameters !!!
		for (int np = 4; np <= 6; np++) {
			RelationalProperty[] rprops = new RelationalProperty[np];
			for (int i = 0; i < np; i++)
				rprops[i] = new TriRelationalProperty(p[i], pStrich[i],
						Motion_3D.class);

			// estimate
			Motion_3D est = new Motion_3D();

			est.approximate(rprops);

			// Output
			//System.out.println("========================================");
			//System.out.println("Test with " + np + " planes");
			Matrix T = target.getTransformationMatrix();
			T = T.mult(1 / T.get(3, 3));
			//System.out.println("Target:" + T);

			Matrix Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			double d= (Te.vec().minus(T.vec()).normF());
			//System.out.println("Approximation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d+ "\n\n");
			if (d>maxdisttrans) fail();

			est.estimate(rprops);

			Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			d= (Te.vec().minus(T.vec()).normF());
			//System.out.println("Estimation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d+"\n\n");
			if (d>maxdisttrans) fail();

		}

	}
	
	/** Test the special approximation function and the estimation function with
	 *  lines
	 *
	 */
	public void testApproximateLines() {
		//Maximal distance of vec(Target) and vec(Approximation) and vec(Estimation)
		double maxdisttrans=0.01;


		// construct 6 lines
		double sigma = 1e-2;
		
		// factor for noise on lines
		double randfactor=1e-3;
		
		double[][] val = { { sigma * sigma, 0, 0, 0 },
				{ 0, sigma * sigma, 0, 0 }, { 0, 0, sigma * sigma, 0 },
				{ 0, 0, 0, 0 } };
		Matrix cov = LinearAlgebraFactory.getMatrix(val);

		Point_3D[] p = new Point_3D[6];

		p[0] = new Point_3D(100, 10, 20,3, cov);
		p[1] = new Point_3D(0, -100, 1,3, cov);
		p[2] = new Point_3D(0, 6, 100,-2, cov);
		p[3] = new Point_3D(100, 50, 0,20, cov);
		p[4] = new Point_3D(0, 200, 100,200, cov);
		p[5] = new Point_3D(100, 200, 300, 40, cov);
		
		Line_3D[] l = new Line_3D[6];
		
		l[0]=new Line_3D(p[0],p[1]);
		l[1]=new Line_3D(p[2],p[3]);
		l[2]=new Line_3D(p[4],p[5]);
		l[3]=new Line_3D(p[3],p[4]);
		l[4]=new Line_3D(p[2],p[5]);
		l[5]=new Line_3D(p[5],p[0]);
		
		

		Motion_3D target = new Motion_3D(new Translation_3D(1, -2, 5),
				new Rotation_3D(Math.PI / 2, 1.2, -0.5), LinearAlgebraFactory
						.getDiagMatrix(16, 0.0));

		Line_3D[] lStrich = new Line_3D[6];
		for (int i = 0; i < 6; i++)
			lStrich[i] = (Line_3D) target.transform(l[i]);

		// add noise (this is not stricty according to covariance !)
		Random ran = new Random();
		for (int i = 0; i < lStrich.length; i++) {
			double[] noise = { ran.nextGaussian() * sigma,
					ran.nextGaussian() * randfactor, ran.nextGaussian() * randfactor, 
					ran.nextGaussian() * randfactor,ran.nextGaussian() * randfactor,
					ran.nextGaussian() * randfactor};
			lStrich[i].setV((lStrich[i].getV()).plus(LinearAlgebraFactory
					.getVector(noise)));
		}

		// Test with 3 to 6 lines
		for (int np = 3; np <= 6; np++) {
			RelationalProperty[] rprops = new RelationalProperty[np];
			for (int i = 0; i < np; i++)
				rprops[i] = new TriRelationalProperty(l[i], lStrich[i],
						Motion_3D.class);

			// estimate
			Motion_3D est = new Motion_3D();

			est.approximate(rprops);

			// Output
			//System.out.println("========================================");
			//System.out.println("Test with " + np + " lines");
			Matrix T = target.getTransformationMatrix();
			T = T.mult(1 / T.get(3, 3));
			//System.out.println("Target:" + T);

			Matrix Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			double d= (Te.vec().minus(T.vec()).normF());
			//System.out.println("Approximation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d + "\n\n");
			if (d>maxdisttrans) fail();
			
			est.estimate(rprops);

			Te = est.getTransformationMatrix();
			Te = Te.mult(1 / Te.get(3, 3));
			d= (Te.vec().minus(T.vec()).normF());
			//System.out.println("Estimation:" + Te);
			//System.out.println("Distance to target (normF(vec-vec)):"
			//		+ d+"\n\n");
			if (d>maxdisttrans) fail();
			
		}

	}

	
	
	
}
