/*
 * Created on 27.01.2004
 *
 */
package sugr.test;

import sugr.*;
import sugr.linalg.*;
import java.util.Random;

/**
 * Compute statistics to test covariance matrices.
 * 
 * @author Thomas Laebe, ipb
 * 
 */
public class TestStatistics {

	/** Size of vector */
	private int size;

	/** current number of observations */
	private int n = 0;

	/** observations. */
	private java.util.Vector obs = new java.util.Vector();

	/**
	 * Construct a new TestStatistics for an Entity.
	 * 
	 * @param size
	 *            Size of vector of that Entity.
	 */
	public TestStatistics(int size) {
		this.size = size;
	}

	/**
	 * This method checks if two Vectors and Matrices have nearly the same
	 * entries. No test included if dimensions agree!
	 * 
	 * @param vec1
	 *            to be compared to vec2
	 * @param vec2
	 * @param mat1
	 *            to be compared to mat2
	 * @param mat2
	 * @param precision
	 *            factor for comparison
	 * @return if vec1,mat1 meet vec2,mat2
	 */
	public static boolean check(Vector vec1, Vector vec2, Matrix mat1,
			Matrix mat2, double precision) {
		Vector diffV = vec1.minus(vec2);
		Matrix diffM = mat1.minus(mat2);
		for (int i = 0; i < diffV.getDim(); i++) {
			if ((Math.abs(diffV.get(i)) > precision * Math.abs(vec1.get(i)))
					&& (vec1.get(i) != 0) && (vec1.get(i) != -0))
				return false;
		}
		for (int i = 0; i < diffM.getRowDimension(); i++)
			for (int j = 0; j < diffM.getColumnDimension(); j++)
				if ((Math.abs(diffM.get(i, j)) > precision
						* Math.abs(mat1.get(i, j)))
						&& (mat1.get(i, j) != 0) && (mat1.get(i, j) != -0))
					return false;
		return true;
	}

	/**
	 * This method checks if two Vectors and Matrices have nearly the same
	 * entries. No test included if dimensions agree!
	 * 
	 * @param vec1
	 *            to be compared to vec2
	 * @param vec2
	 * @param mat1
	 *            to be compared to mat2
	 * @param mat2
	 * @param precision
	 *            factor for comparison
	 * @param null_threshold
	 *            all differences less than this value are regarded as being 0: ONLY
	 *            for the matrixes !
	 * @return if vec1,mat1 meet vec2,mat2
	 */
	public static boolean check(Vector vec1, Vector vec2, Matrix mat1,
			Matrix mat2, double precision, double null_threshold) {


		Vector diffV = vec1.minus(vec2);
		Matrix diffM = mat1.minus(mat2);
		for (int i = 0; i < diffV.getDim(); i++) {
			if ((Math.abs(diffV.get(i)) > precision * Math.abs(vec1.get(i)))
					&& (vec1.get(i) != 0) && (vec1.get(i) != -0))
				return false;
		}
		for (int i = 0; i < diffM.getRowDimension(); i++)
			for (int j = 0; j < diffM.getColumnDimension(); j++)
				if (Math.abs(diffM.get(i, j)) > null_threshold)
					if ((Math.abs(diffM.get(i, j)) > precision
							* Math.abs(mat1.get(i, j)))) {
						System.out.println("Vergleich bei ("+i+","+j+") fehlgeschlagen.");
					
						return false;
					}
		return true;
	}

	/**
	 * This method checks if two Matrices have nearly the same entries No test
	 * included if dimensions agree.
	 * 
	 * @param m1
	 *            to be compared to m2
	 * @param m2
	 * @param precision
	 *            factor for comparison
	 * @return if m1 meets m2
	 */
	public static boolean check(Matrix m1, Matrix m2, double precision) {
		Matrix diff = m1.minus(m2);

		for (int i = 0; i < diff.getRowDimension(); i++)
			for (int j = 0; j < diff.getColumnDimension(); j++)
				if ((Math.abs(diff.get(i, j)) > precision
						* Math.abs(m1.get(i, j)))
						&& (m1.get(i, j) != 0) && (m1.get(i, j) != -0))
					return false;
		return true;
	}

	/**
	 * This method checks if two Matrices have nearly the same entries with an
	 * absolute difference between the two matrix entries No test included if
	 * dimensions agree.
	 * 
	 * @param m1
	 *            to be compared to m2
	 * @param m2
	 * @param precision
	 *            factor for comparison
	 * @return if m1 meets m2
	 */
	public static boolean checkAbsolute(Matrix m1, Matrix m2, double precision) {
		Matrix diff = m1.minus(m2);

		for (int i = 0; i < diff.getRowDimension(); i++)
			for (int j = 0; j < diff.getColumnDimension(); j++)
				if (Math.abs(diff.get(i, j)) > precision)
					return false;
		return true;
	}

	/**
	 * Checks if two vectors have nearly the same entries. No test included if
	 * dimensions agree!
	 * 
	 * @param vec1
	 *            to be compared to vec2
	 * @param vec2
	 * @param precision
	 *            factor for comparison
	 * @return if vec1 meets vec2
	 */
	public static boolean check(Vector vec1, Vector vec2, double precision) {
		Vector diffV = vec1.minus(vec2);
		for (int i = 0; i < diffV.getDim(); i++) {
			if ((Math.abs(diffV.get(i)) > precision * Math.abs(vec1.get(i)))
					&& (vec1.get(i) != 0) && (vec1.get(i) != -0))
				return false;
		}
		return true;
	}

	/**
	 * Checks if two entities contain the same values. Uses check method for
	 * vector and covaciance.
	 * 
	 * @param one -
	 *            first entity
	 * @param two -
	 *            second entity
	 * @param precision -
	 *            precision factor for comparison
	 * @return one == two
	 */
	public static boolean check(Entity one, Entity two, double precision) {
		return check(one.getV(), two.getV(), one.getCov(), two.getCov(),
				precision);
	}

	/**
	 * Add a new observation. The vector of the observation is stored.
	 * 
	 * @param e
	 *            the new observation.
	 */
	public void addObservation(Element e) {
		n++;
		obs.add(e.getV().clone());
	}

	/** Get the (empirical) mean value. */
	public Vector getMue() {
		Vector sum_mue = LinearAlgebraFactory.getVector(size);

		for (int i = 0; i < n; i++) {
			Vector ob = (Vector) obs.elementAt(i);
			sum_mue = sum_mue.plus(ob);
		}
		Vector mue = sum_mue.mult(1.0 / n);
		return mue;
	}

	/** Get the (empirical) covariance matrix */
	public Matrix getSigma() {
		Matrix sum_sigma = LinearAlgebraFactory.getMatrix(size, size);

		Vector mue = getMue();
		for (int k = 0; k < n; k++) {
			Vector ob = (Vector) obs.elementAt(k);

			for (int i = 0; i < size; i++)
				for (int j = 0; j < size; j++)
					sum_sigma.set(i, j, sum_sigma.get(i, j)
							+ (ob.get(i) - mue.get(i))
							* (ob.get(j) - mue.get(j)));
		}

		Matrix sigma = sum_sigma.mult(1.0 / (n - 1.0));
		return sigma;
	}

	public static void main(String[] args) {
		test3();
	}

	/** Example: Test a constructor. */
	public static void test1() {

		double sigma = 0.1;

		// Error free
		Matrix cov1 = LinearAlgebraFactory.getDiagMatrix(3, sigma * sigma);
		cov1.set(2, 2, 0);
		cov1.set(0, 1, 0.002);
		cov1.set(1, 0, 0.002);

		Vector v1 = LinearAlgebraFactory.getVector(1, 2, 1);
		Vector v2 = LinearAlgebraFactory.getVector(3, 4, 1);

		Point_2D p1 = new Point_2D(v1, cov1);
		Point_2D p2 = new Point_2D(v2, cov1);

		Line_2D res = new Line_2D(p1, p2);

		System.out.println("Line: " + res);

		TestStatistics statistic = new TestStatistics(3);
		RandomVector rv1 = new RandomVector(v1, cov1);
		RandomVector rv2 = new RandomVector(v2, cov1);

		for (int i = 0; i < 100000; i++) {
			p1 = new Point_2D(rv1.getNextRandom(), cov1);
			p2 = new Point_2D(rv2.getNextRandom(), cov1);
			Line_2D l = new Line_2D(p1, p2);
			statistic.addObservation(l);
		}

		System.out.println("Mue: " + statistic.getMue());
		System.out.println("Sigma: " + statistic.getSigma());
	}

	/**
	 * Test how good the gaussian distribution of the random values is.
	 * 
	 */
	public static void test3() {
		Matrix sigsum = LinearAlgebraFactory.getMatrix(3, 3);
		int anz = 20;
		for (int k = 0; k < anz; k++) {

			TestStatistics statistic = new TestStatistics(3);
			Point_2D p1 = new Point_2D(1, 2);

			for (int i = 0; i < 1000000; i++) {
				p1.getV().set(0, getNextGauss(0.1));
				p1.getV().set(1, getNextGauss(0.1));
				statistic.addObservation(p1);
			}

			System.out.println("Mue: " + statistic.getMue());
			Matrix sigma = statistic.getSigma();
			System.out.println("Sigma: " + sigma);

			for (int r = 0; r < 3; r++)
				for (int c = 0; c < 3; c++)
					sigsum.set(r, c, sigsum.get(r, c)
							+ Math.abs(sigma.get(r, c)));
		}

		System.out.println("Mittleres sigma: " + sigsum.mult(1.0 / anz));

	}

	/** Test distribution of a randeom value with covariance matrix. */
	public static void test2() {
		double sigma = 0.1;
		Matrix cov1 = LinearAlgebraFactory.getDiagMatrix(3, sigma * sigma);
		cov1.set(0, 0, 0.01);
		cov1.set(0, 1, 0.002);
		cov1.set(0, 2, 0.003);

		cov1.set(1, 0, 0.002);
		cov1.set(1, 1, 0.04);
		cov1.set(1, 2, 0.006);

		cov1.set(2, 0, 0.003);
		cov1.set(2, 1, 0.006);
		cov1.set(2, 2, 0.09);

		TestStatistics statistic = new TestStatistics(3);

		Vector v = LinearAlgebraFactory.getVector(1, 2, 3);

		RandomVector rv = new RandomVector(v, cov1);
		for (int i = 0; i < 1000000; i++) {
			Vector next = rv.getNextRandom();

			Point_2D p1 = new Point_2D(next, cov1);
			statistic.addObservation(p1);
		}

		System.out.println("Mue: " + statistic.getMue());
		System.out.println("Sigma: " + statistic.getSigma());

		System.out.println("Original cov: " + cov1);

	}

	public static double getNextGauss(double stddev) {
		return random.nextGaussian() * stddev;

	}

	private static Random random = new Random();
}
