/*
 * Created on 05.07.2004
 *
 */
package sugr.test;

import junit.framework.TestCase;
import sugr.*;
import sugr.linalg.*;

/**
 * @author Bastian Oehler
 *
 */
public class Translation_3DTest extends TestCase {

    /**
     * Test for int getDegreesOfFreedom()
     *
     */
    public void testGetDegreesOfFreedom() {
        Translation_3D trans = new Translation_3D();
        if(trans.getDegreesOfFreedom()!=3)fail();
    }

    /**
     * Test for void Translation_3D()
     */
    public void testTranslation_3D() {
        Translation_3D trans = new Translation_3D();
        if(trans==null)fail();
    }

    /**
     * Test for void Translation_3D(double, double, double)
     */
    public void testTranslation_3Ddoubledoubledouble() {
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        Translation_3D trans = new Translation_3D(4,3,2);
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
    }

    /**
     * Test for void Translation_3D(double, double, double, Matrix)
     */
    public void testTranslation_3DdoubledoubledoubleMatrix() {
        //array to check transformation Vector
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        //array for covariance Matrix
        double[][] covArray=new double[16][16];
        //random Values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else covArray[i][j]=covArray[j][i];
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
        //construct Translation_3D(double,double,double,Matrix)
        Translation_3D trans = new Translation_3D(4,3,2,covMat);
        //check Vector
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
        //check covariance Matrix
        Matrix transCov = trans.getCov();
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(Math.abs(Math.abs(transCov.get(i,j))-Math.abs(covMat.get(i,j)))>1e-12)fail();
            }
        }
    }

    /**
     * Test for void Translation_3D(Vector)
     */
    public void testTranslation_3DVector() {
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        Vector vec = LinearAlgebraFactory.getVector(4,3,2);
        Translation_3D trans = new Translation_3D(vec);
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
    }

    /**
     * Test for void Translation_3D(Vector, Matrix)
     */
    public void testTranslation_3DVectorMatrix() {
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        Vector vec = LinearAlgebraFactory.getVector(4,3,2);
        //array for covariance Matrix
        double[][] covArray=new double[16][16];
        //random Values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else covArray[i][j]=covArray[j][i];
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
        Translation_3D trans = new Translation_3D(vec,covMat);
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
        //check covariance Matrix
        Matrix transCov = trans.getCov();
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(Math.abs(Math.abs(transCov.get(i,j))-Math.abs(covMat.get(i,j)))>1e-12)fail();
            }
        }
    }

    /**
     * Test for void Translation_3D(Point_3D)
     */
    public void testTranslation_3DPoint_3D() {
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        Point_3D p= new Point_3D(4,3,2);
        Translation_3D trans = new Translation_3D(p);
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
    }

    /**
     * Test for void Translation_3D(Point_3D, Matrix)
     */
    public void testTranslation_3DPoint_3DMatrix() {
        double[][] array = {{1,0,0,4},{0,1,0,3},{0,0,1,2},{0,0,0,1}};
        Point_3D p= new Point_3D(4,3,2);
        //array for covariance Matrix
        double[][] covArray=new double[16][16];
        //random Values for cov Matrix
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(i<=j)covArray[i][j]=Math.random();
                else covArray[i][j]=covArray[j][i];
            }
        }
        Matrix covMat=LinearAlgebraFactory.getMatrix(covArray);
        Translation_3D trans = new Translation_3D(p,covMat);
        Matrix transmat=trans.getTransformationMatrix();
        for(int i=0;i<4;i++){
            for(int j=0;j<4;j++){
                if(array[i][j]!=transmat.get(i,j))fail();
            }
        }
        //check covariance Matrix
        Matrix transCov = trans.getCov();
        for(int i=0;i<16;i++){
            for(int j=0;j<16;j++){
                if(Math.abs(Math.abs(transCov.get(i,j))-Math.abs(covMat.get(i,j)))>1e-12)fail();
            }
        }
    }
}
